package com.starbuds.server.service.finance.cloud.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.service.finance.cloud.CloudAccountClientManage;
import com.starbuds.server.service.finance.cloud.request.*;
import com.starbuds.server.service.finance.cloud.response.*;
import com.starbuds.server.service.finance.util.CloudAccountUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/27 10:18
 * @Description 云账户服务提供者
 */
@Component
public class CloudAccountProvider {

    @Resource
    CloudAccountClientManage cloudAccountClientManage;

    @Value("${cloudAccount.dealerId}")
    private String dealerId;

    @Value("${cloudAccount.brokerId}")
    private String brokerId;

    @Value("${cloudAccount.desKey}")
    private String desKey;

    @Value("${cloudAccount.appKey}")
    private String appKey;

    @Value("${cloudAccount.orderNotifyUrl}")
    private String orderNotifyUrl;

    /**
     * 身份证实名验证
     *
     * @param idCard
     * @param realName
     * @return
     */
    public VerifyIdResponse verifyId(String idCard, String realName) {
        VerifyIdRequest request = new VerifyIdRequest();
        request.setIdCard(idCard);
        request.setRealName(realName);
        return cloudAccountClientManage.getClient().execute(request);
    }

    /**
     * 银⾏卡四要素请求鉴权（下发短信验证码）
     *
     * @param cardNo
     * @param idCard
     * @param realName
     * @param mobile
     * @return
     */
    public VerifyResponse verifyRequest(String cardNo, String idCard, String realName, String mobile) {
        VerifyRequest fourRequest = new VerifyRequest();
        fourRequest.setCardNo(cardNo);
        fourRequest.setIdCard(idCard);
        fourRequest.setRealName(realName);
        fourRequest.setMobile(mobile);
        return cloudAccountClientManage.getClient().execute(fourRequest);
    }

    /**
     * 银⾏卡四要素确认鉴权（上传短信验证码）
     *
     * @param cardNo
     * @param idCard
     * @param realName
     * @param mobile
     * @param captcha
     * @param ref
     * @return
     */
    public VerifyConfirmResponse verifyConfirm(String cardNo, String idCard, String realName, String mobile, String captcha, String ref) {
        VerifyConfirmRequest confirmRequest = new VerifyConfirmRequest();
        confirmRequest.setCaptcha(captcha);
        confirmRequest.setCardNo(cardNo);
        confirmRequest.setIdCard(idCard);
        confirmRequest.setRealName(realName);
        confirmRequest.setMobile(mobile);
        confirmRequest.setRef(ref);
        return cloudAccountClientManage.getClient().execute(confirmRequest);
    }

    /**
     * 银⾏卡信息查询接⼝
     *
     * @param cardNo
     * @param bankName
     * @return
     */
    public CardQueryResponse queryCard(String cardNo, String bankName) {
        CardQueryRequest queryRequest = new CardQueryRequest();
        queryRequest.setCardNo(cardNo);
        queryRequest.setBankName(bankName);
        return cloudAccountClientManage.getClient().execute(queryRequest);
    }

    /**
     * 银⾏卡实时下单
     *
     * @param orderId
     * @param realName
     * @param cardNo
     * @param idCard
     * @param pay
     * @param payRemark
     * @return
     */
    public PaymentOrderResponse payOrder(Long orderId, String realName, String cardNo, String idCard, String pay, String payRemark) {
        PaymentOrderRequest orderRequest = new PaymentOrderRequest();
        orderRequest.setBrokerId(brokerId);
        orderRequest.setDealerId(dealerId);
        orderRequest.setOrderId(Convert.toStr(orderId));
        orderRequest.setRealName(realName);
        orderRequest.setCardNo(cardNo);
        orderRequest.setIdCard(idCard);
        orderRequest.setPay(pay);
        orderRequest.setPayRemark(payRemark);
        orderRequest.setNotifyUrl(orderNotifyUrl);
        return cloudAccountClientManage.getClient().execute(orderRequest);
    }

    /**
     * 解密
     *
     * @param encryptStr
     * @return
     */
    public String decodeData(String encryptStr) {
        return CloudAccountUtil.decodeData(encryptStr, desKey);
    }

    /**
     * 获取签名
     *
     * @param encryptStr
     * @param mess
     * @param timestamp
     */
    public String getSign(String encryptStr, String mess, Long timestamp) {
        return CloudAccountUtil.getSign(encryptStr, mess, timestamp, appKey);
    }

}
