package com.starbuds.server.service.finance.cloud.request;

import cn.hutool.http.Method;
import com.alibaba.fastjson.annotation.JSONField;
import com.starbuds.server.service.finance.cloud.constant.CloudAccountApi;
import com.starbuds.server.service.finance.cloud.response.PaymentOrderResponse;
import lombok.Data;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/25 10:49
 * @Description 银⾏卡实时下单
 */
@Data
public class PaymentOrderRequest implements CloudAccountRequest<PaymentOrderResponse> {

    /**
     * 商户订单号，由商户保持唯⼀性(必填)，64个
     * 英⽂字符以内
     */
    @JSONField(name = "order_id")
    private String orderId;
    /**
     * 商户代码
     */
    @JSONField(name = "dealer_id")
    private String dealerId;
    /**
     * 综合服务主体
     */
    @JSONField(name = "broker_id")
    private String brokerId;
    /**
     * 银⾏开户姓名
     */
    @JSONField(name = "real_name")
    private String realName;
    /**
     * 银⾏开户卡号
     */
    @JSONField(name = "card_no")
    private String cardNo;
    /**
     * 银⾏开户身份证号
     */
    @JSONField(name = "id_card")
    private String idCard;
    /**
     * 打款⾦额(单位为元, 必填)
     */
    private String pay;
    /**
     * 打款备注
     */
    @JSONField(name = "pay_remark")
    private String payRemark;
    /**
     * 回调地址(选填)
     */
    @JSONField(name = "notify_url")
    private String notifyUrl;

    private transient String apiUrl;

    private transient Method method;

    private transient Class<PaymentOrderResponse> responseClass;


    @Override
    public String getApiUrl() {
        return CloudAccountApi.PAYMENT_ORDER;
    }

    @Override
    public Method getMethod() {
        return Method.POST;
    }

    @Override
    public Class<PaymentOrderResponse> getResponseClass() {
        return PaymentOrderResponse.class;
    }

}
