package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 抽奖活动中用户获取的抽奖资格 Mapper
 */
@Mapper
@Repository
public interface LotteryGameUserQualificationHisDAO {

    /**
     * 插入数据
     */
    int insert(LotteryGameUserQualificationHisDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<LotteryGameUserQualificationHisDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(LotteryGameUserQualificationHisDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    LotteryGameUserQualificationHisDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<LotteryGameUserQualificationHisDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<LotteryGameUserQualificationHisDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("hisId")
    Map<Long, LotteryGameUserQualificationHisDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 根据抽奖游戏活动执行快照id获得抽奖资格列表
     *
     * @param snapId
     * @return
     */
    @Select("SELECT hisId,userId " +
            "FROM tb_lottery_game_user_qualification_his " +
            "WHERE snapId = #{snapId} " +
            "AND isEffective = #{isEffective} " +
            "ORDER BY createTime ASC")
    List<LotteryGameUserQualificationHisDO> selectListBySnapId(@Param("snapId") Long snapId,
                                                               @Param("isEffective") int isEffective);

}