package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.db.PayOrderDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/18
 */
@Mapper
@Repository
public interface PayOrderDAO {
    /**
     * 创建支付订单
     *
     * @param payOrderDO
     * @return
     */
    int insert(PayOrderDO payOrderDO);

    /**
     * 更新 ApplePay 交易流水号
     *
     * @param orderId
     * @param appleTransactionId
     * @param payStatus
     * @return
     */
    int updateAppleTransactionId(@Param("orderId") Long orderId, @Param("appleTransactionId") String appleTransactionId, @Param("payStatus") Integer payStatus, @Param("appleReceiptData") String appleReceiptData);

    /**
     * 更新 appleReceiptData 数据(用于后续检查订单状态)
     *
     * @param orderId
     * @param appleReceiptData
     * @return
     */
    int updateAppleReceiptData(@Param("orderId") Long orderId, @Param("appleTransactionId") String appleTransactionId, @Param("appleReceiptData") String appleReceiptData);

    /**
     * 更新订单支付状态
     *
     * @param orderId
     * @param payStatus
     * @param wherePayStatus
     * @return
     */
    int updatePayStatus(@Param("orderId") Long orderId, @Param("payStatus") Integer payStatus, @Param("wherePayStatus") Integer wherePayStatus);

    /**
     * 根据 id 统计
     *
     * @param orderId
     * @return
     */
    int countById(@Param("orderId") Long orderId);


    /**
     * 获取用户首冲订单
     *
     * @param userId
     * @return
     */
    int getUserPayOrderCount(@Param("userId") Long userId);

    /**
     * 根据 id 查询
     *
     * @param orderId
     * @return
     */
    PayOrderDO selectById(@Param("orderId") Long orderId);

    /**
     * 根据 id 查询支付参数
     *
     * @param orderId
     * @return
     */
    String selectPayParamsById(@Param("orderId") Long orderId);

    /**
     * 查询订单支付状态
     *
     * @param orderId
     * @return
     */
    Integer selectPayStatusById(@Param("orderId") Long orderId);

    /**
     * 查询订单支付类型
     *
     * @param orderId
     * @return
     */
    Integer selectPayTypeById(@Param("orderId") Long orderId);

    /**
     * 查询订单支付方式
     *
     * @param orderId
     * @return
     */
    Integer selectPayWayById(@Param("orderId") Long orderId);

    /**
     * 查询 ApplePay 交易流水号
     *
     * @param orderId
     * @return
     */
    String selectAppleTransactionIdById(@Param("orderId") Long orderId);

    /**
     * 根据用户和订单状态统计
     *
     * @param userId
     * @param payStatus
     * @param beginTime
     * @param endTime
     * @return
     */
    int countByUserAndStatusWithTimeRange(@Param("userId") Long userId, @Param("payStatus") Integer payStatus, @Param("beginTime") long beginTime, @Param("endTime") long endTime);

    /**
     * 更新记录为首充
     *
     * @param orderId
     * @param firstRecharge
     * @return
     */
    int updateFirstRecharge(@Param("orderId") Long orderId, @Param("firstRecharge") int firstRecharge);

    /**
     * 更新记录为当日首充
     *
     * @param orderId
     * @param dailyFirstRecharge
     * @return
     */
    int updateDailyFirstRecharge(@Param("orderId") Long orderId, @Param("dailyFirstRecharge") int dailyFirstRecharge);
}
