package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.db.SendGiftSnapshotDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 送礼物快照 Mapper
 */
@Mapper
@Repository
public interface SendGiftSnapshotDAO {

    /**
     * 插入数据
     */
    int insert(SendGiftSnapshotDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<SendGiftSnapshotDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(SendGiftSnapshotDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    SendGiftSnapshotDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<SendGiftSnapshotDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<SendGiftSnapshotDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("snapId")
    Map<Long, SendGiftSnapshotDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 获取个播场次总流水
     *
     * @param roomId
     * @param createTime
     * @return
     */
    Long selectSingleRtcRoomRoundFlow(@Param("roomId") Long roomId, @Param("createTime") Long createTime);

    /**
     * 获取个播场次直刷总流水
     *
     * @param roomId
     * @param createTime
     * @return
     */
    Long selectSingleRtcRoomRoundDirectFlow(@Param("roomId") Long roomId, @Param("createTime") Long createTime);

    /**
     * 获取个播场次直刷总流水
     *
     * @param roomId
     * @param createTime
     * @return
     */
    Long selectSingleRtcRoomRoundSendGiftPeople(@Param("roomId") Long roomId, @Param("createTime") Long createTime);

    /**
     * 获取个播场次送礼列表
     *
     * @param roomId
     * @param createTime
     * @return
     */
    List<SendGiftSnapshotDO> selectSingleRtcRoomRoundList(@Param("roomId") Long roomId, @Param("createTime") Long createTime, @Param("isFromBag") Integer isFromBag);

    /**
     * 个播送礼人数统计
     *
     * @param beginTime
     * @param endTime
     * @param businessType
     * @return
     */
    Integer getSendGiftPeople(@Param("beginTime") Long beginTime, @Param("endTime") Long endTime, @Param("businessType") Integer businessType);
}