package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.db.SkillOrderDO;
import com.starbuds.server.common.pojo.finance.SkillOrderVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/22 16:05
 * @Description 技能订单
 */
@Mapper
@Repository
public interface SkillOrderDAO {

    /**
     * 保存
     *
     * @param orderDO
     * @return
     */
    int insert(SkillOrderDO orderDO);

    /**
     * 判断订单是否存在
     *
     * @param orderId
     * @return
     */
    int countById(@Param("orderId") Long orderId);

    /**
     * 根据id查询订单
     *
     * @param orderId
     * @return
     */
    SkillOrderDO selectById(@Param("orderId") Long orderId);

    /**
     * 更新
     *
     * @param orderDO
     * @param wherePayStatus
     * @param whereOrderStatus
     * @return
     */
    int updateById(@Param("orderDO") SkillOrderDO orderDO, @Param("wherePayStatus") Integer wherePayStatus, @Param("whereOrderStatus") Integer whereOrderStatus);

    /**
     * 获取订单状态
     *
     * @param orderId
     * @return
     */
    Integer selectOrderStatus(@Param("orderId") Long orderId);

    /**
     * 获取支付状态
     *
     * @param orderId
     * @return
     */
    Integer selectPayStatus(@Param("orderId") Long orderId);

    /**
     * 获取订单详情
     *
     * @param orderId
     * @return
     */
    SkillOrderVO selectDetail(@Param("orderId") Long orderId);

    /**
     * 获取记录列表
     *
     * @param pageCursor
     * @param pageSize
     * @param orderStatus
     * @param payStatus
     * @param receiveUserId
     * @param userId
     * @return
     */
    List<SkillOrderVO> selectOrderList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("orderStatus") Integer orderStatus, @Param("payStatus") Integer payStatus, @Param("receiveUserId") Long receiveUserId, @Param("userId") Long userId);

    /**
     * 获取记录大小
     *
     * @param orderStatus
     * @param payStatus
     * @param receiveUserId
     * @param userId
     * @return
     */
    int countOrderList(@Param("orderStatus") Integer orderStatus, @Param("payStatus") Integer payStatus, @Param("receiveUserId") Long receiveUserId, @Param("userId") Long userId);

    /**
     * 获取接单人用户id
     *
     * @param orderId
     * @return
     */
    Long selectReceiveUserId(@Param("orderId") Long orderId);

    /**
     * 获取下单人用户id
     *
     * @param orderId
     * @return
     */
    Long selectUserId(@Param("orderId") Long orderId);

    /**
     * 获取技能id
     *
     * @param orderId
     * @return
     */
    Long selectSkillId(@Param("orderId") Long orderId);
}
