package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.db.UserBagDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/18
 */
@Mapper
@Repository
public interface UserBagDAO {
    /**
     * 创建背包物品
     *
     * @param userBagDO
     * @return
     */
    int insert(UserBagDO userBagDO);

    /**
     * 创建或更新背包物品
     *
     * @param userBagDO
     * @return
     */
    int insertOrUpdate(UserBagDO userBagDO);

    /**
     * 根据 id 查询
     *
     * @param id
     * @return
     */
    UserBagDO selectById(@Param("id") String id);

    /**
     * 修改数量
     *
     * @param id
     * @param quantity
     * @param whereVersion
     * @param updateTIme
     * @return
     */
    int updateQuantity(@Param("id") String id, @Param("quantity") Integer quantity, @Param("whereVersion") Long whereVersion, @Param("updateTIme") Long updateTIme);

    /**
     * 根据 id 和 类型查询
     *
     * @param userId
     * @param itemId
     * @param itemType
     * @param curTime
     * @return
     */
    List<UserBagDO> selectAllByIdAndType(@Param("userId") Long userId, @Param("itemId") Long itemId, @Param("itemType") Integer itemType, @Param("curTime") Long curTime);

    /**
     * 查询用户背包礼物列表
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<GiftItemVO>> selectGiftItemsByUserId(@Param("userId") Long userId,
                                                      @Param("itemType") Integer itemType,
                                                      @Param("curTime") long curTime);

    /**
     * 查询用户背包道具列表
     *
     * @param userId
     * @param itemType
     * @param propsType
     * @param curTime
     * @return
     */
    List<BagItem<PropsVO>> selectPropsItemsByUserId(@Param("userId") Long userId,
                                                    @Param("itemType") Integer itemType,
                                                    @Param("propsType") Integer propsType,
                                                    @Param("curTime") long curTime);

    /**
     * 根据 id 查询道具
     *
     * @param bagId
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<PropsVO>> selectPropsItemsById(@Param("bagId") String bagId, @Param("userId") Long userId, @Param("itemType") Integer itemType, @Param("curTime") long curTime);

    /**
     * 查询用户背包坐骑列表
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<MountsVO>> selectMountsItemsByUserId(@Param("userId") Long userId,
                                                      @Param("itemType") Integer itemType,
                                                      @Param("curTime") long curTime);

    /**
     * 查询用户背包勋章列表
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<MedalVO>> selectMedalItemsByUserId(@Param("userId") Long userId,
                                                    @Param("itemType") Integer itemType,
                                                    @Param("curTime") long curTime);

    /**
     * 查询用户背包头饰列表
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<HeadwearVO>> selectHeadwearItemsByUserId(@Param("userId") Long userId,
                                                          @Param("itemType") Integer itemType,
                                                          @Param("curTime") long curTime);

    /**
     * 查询用户背包聊天气泡列表
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    List<BagItem<ChatBubbleVO>> selectChatBubbleItemsByUserId(@Param("userId") Long userId,
                                                              @Param("itemType") Integer itemType,
                                                              @Param("curTime") long curTime);

    /**
     * 根据物品 id 计算物品数量
     *
     * @param userId
     * @param itemType
     * @param itemId
     * @param curTime
     * @return
     */
    Integer sumQuantityByItemId(@Param("userId") Long userId,
                                @Param("itemType") Integer itemType,
                                @Param("itemId") Long itemId,
                                @Param("curTime") long curTime);

    /**
     * 根据道具类型统计数量
     *
     * @param userId
     * @param itemType
     * @param propsType
     * @param curTime
     * @return
     */
    Integer sumPropsQuantityByType(@Param("userId") Long userId,
                                   @Param("itemType") Integer itemType,
                                   @Param("propsType") Integer propsType,
                                   @Param("curTime") long curTime);

    /**
     * 根据类型统计
     *
     * @param userId
     * @param itemType
     * @param curTime
     * @return
     */
    Integer sumQuantityByItemType(@Param("userId") Long userId, @Param("itemType") Integer itemType, @Param("curTime") long curTime);

    /**
     * 获取用户背包礼物列表
     *
     * @param userId
     * @return
     */
    List<UserBagDO> getUserBagDoList(@Param("userId") Long userId);

    /**
     * 通过礼物ID获取数量
     *
     * @param giftIds
     * @param userId
     * @return
     */
    List<UserBagDO> getUserBagByGiftIds(@Param("giftIds") List<Long> giftIds, @Param("userId") Long userId);
}
