package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.finance.BankcardVO;
import com.starbuds.server.common.pojo.finance.UserBankcardDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/28 10:44
 * @Description 用户银行卡持久层
 */
@Mapper
@Repository
public interface UserBankcardDAO {

    /**
     * 保存或更新
     *
     * @param bankcardDO
     * @return
     */
    int saveOrUpdate(UserBankcardDO bankcardDO);

    /**
     * 获取银行卡
     *
     * @param userId
     * @return
     */
    List<BankcardVO> selectByUserId(@Param("userId") Long userId);

    /**
     * 判断是否绑卡
     *
     * @param userId
     * @return
     */
    int countByUserId(@Param("userId") Long userId);

    /**
     * 判断卡号是否绑定
     *
     * @param userId
     * @param cardNo
     * @return
     */
    int countByCardNo(@Param("userId") Long userId, @Param("cardNo") String cardNo);

    /**
     * 更新状态
     *
     * @param cardId
     * @param status
     * @param updateTime
     * @return
     */
    int updateStatus(@Param("cardId") String cardId,
                     @Param("status") Integer status,
                     @Param("updateTime") Long updateTime);

    /**
     * 判断卡号是否存在
     *
     * @param cardId
     * @return
     */
    int countById(@Param("cardId") String cardId);

    /**
     * 获取用户id
     *
     * @param cardId
     * @return
     */
    Long selectUserIdByCardId(@Param("cardId") String cardId);

    /**
     * 获取银行卡
     *
     * @param cardId
     * @return
     */
    BankcardVO selectCardById(String cardId);
}
