package com.starbuds.server.service.finance.dao;

import com.starbuds.server.common.pojo.db.UserWalletHisDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 用户钱包变更记录
 *
 * @author Lay
 * @date 2020/1/17
 */
@Mapper
@Repository
public interface UserWalletHisDAO {
    /**
     * 创建钱包变动记录
     *
     * @param userWalletHisDO
     * @return
     */
    int insert(UserWalletHisDO userWalletHisDO);

    /**
     * 查询钱包历史记录
     *
     * @param walletIds
     * @param lastId
     * @param pageSize
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    List<UserWalletHisDO> selectAll(@Param("walletIds") Set<String> walletIds,
                                    @Param("lastId") Long lastId,
                                    @Param("pageSize") int pageSize,
                                    @Param("isIncome") Integer isIncome,
                                    @Param("beginTime") Long beginTime,
                                    @Param("endTime") Long endTime);

    /**
     * 求和
     *
     * @param walletId
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    Long sumByTypeAndTime(@Param("walletId") String walletId,
                          @Param("isIncome") Integer isIncome,
                          @Param("beginTime") Long beginTime,
                          @Param("endTime") Long endTime);

    /**
     * 一个用户一天内总收入
     *
     * @param userId
     * @param date   日期（格式 2020-07-30）
     * @param type   钱包类型
     * @return
     */
    Long sumDailyBalance(@Param("userId") Long userId,
                         @Param("date") String date,
                         @Param("type") Integer type);

    /**
     * 一个用户一天消耗总数
     *
     * @param userId
     * @param date   日期（格式 2020-07-30）
     * @param type   钱包类型
     * @return
     */
    Long sumDailyCost(@Param("userId") Long userId,
                      @Param("date") String date,
                      @Param("type") Integer type);


    /**
     * 获取用户粉钻流水
     *
     * @param userId
     * @return
     */
    List<UserWalletHisDO> selectUserWalletHisByOrderIds(@Param("orderIds") List<Long> orderIds, @Param("userId") Long userId, @Param("type") Integer type);
}
