package com.starbuds.server.service.finance.easyTax.client;

import cn.hutool.core.convert.Convert;
import cn.hutool.http.ContentType;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.service.finance.easyTax.request.EasyTaxRequest;
import com.starbuds.server.service.finance.easyTax.response.EasyTaxResponse;
import com.starbuds.server.service.finance.util.EasyTaxUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Map;

/**
 * 云账户客户端
 */
@Slf4j
public abstract class AbstractEasyTaxClient implements CloudAccountClient {

    public static final String signType = "sha256";

    private String baseUrl;
    private String dealerId;
    private String brokerId1;
    private String brokerId2;
    private String appKey;
    private String securityKey;

    public AbstractEasyTaxClient(String baseUrl, String dealerId, String brokerId1, String brokerId2, String appKey, String securityKey) {
        this.baseUrl = baseUrl;
        this.dealerId = dealerId;
        this.brokerId1 = brokerId1;
        this.brokerId2 = brokerId2;
        this.appKey = appKey;
        this.securityKey = securityKey;
    }

    /**
     * 执行请求
     *
     * @param request
     * @return
     */
    @Override
    public <T extends EasyTaxResponse> T execute(EasyTaxRequest<T> request) {

        long timestamp = System.currentTimeMillis() / 1000;

        log.info("原始请求入参:\n{}", JSON.toJSONString(request, true));

        String requestStr = JSON.toJSONString(request);

        String encryptData = EasyTaxUtil.encryptData(requestStr, securityKey, appKey);

        String requestId = Convert.toStr(System.currentTimeMillis());

        // 请求头
        Map<String, String> headerMap = new HashMap<>(3);
        headerMap.put("dealer-id", dealerId);
        headerMap.put("request-id", requestId);
        headerMap.put("encrypt", "false");

        // json
        Map<String, Object> paramsMap = new HashMap<>(7);
        paramsMap.put("data", encryptData);
        paramsMap.put("timestamp", timestamp);
        paramsMap.put("sign_type", signType);
        paramsMap.put("sign", EasyTaxUtil.createSign(encryptData, Convert.toStr(timestamp), appKey, signType, requestId, dealerId));

        log.info("请求路径:{}", request.getApiUrl());
        log.info("请求头:{}", headerMap);
        log.info("请求参数:{}", paramsMap);

        // 构造基本请求头
        HttpResponse response = new HttpRequest(baseUrl + request.getApiUrl())
                .contentType(ContentType.JSON.toString())
                .method(request.getMethod())
                .headerMap(headerMap, true)
                .body(JSON.toJSONString(paramsMap))
                .timeout(30 * 1000)
                .execute();

        if (response.isOk()) {
            log.info("请求结果:\n{}", JSON.toJSONString(response.body(), true));
            return JSON.parseObject(response.body(), request.getResponseClass());
        }
        log.info("请求出错，状态码:{}", response.getStatus());
        return null;
    }
}
