package com.starbuds.server.service.finance.easyTax.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.finance.WithdrawPayTypeEnum;
import com.starbuds.server.service.finance.easyTax.EasyTaxClientManage;
import com.starbuds.server.service.finance.easyTax.request.PaymentBankPayRequest;
import com.starbuds.server.service.finance.easyTax.response.PaymentBankPayResponse;
import com.starbuds.server.service.finance.util.EasyTaxUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/27 10:18
 * @Description 易纳税服务提供者
 */
@Component
public class EasyTaxProvider {

    public static final String signType = "sha256";

    @Resource
    EasyTaxClientManage easyTaxClientManage;

    @Value("${easyTax.dealerId}")
    private String dealerId;

    @Value("${easyTax.brokerId1}")
    private String brokerId1;

    @Value("${easyTax.brokerId2}")
    private String brokerId2;

    @Value("${easyTax.brokerId3}")
    private String brokerId3;

    @Value("${easyTax.brokerId4}")
    private String brokerId4;

    @Value("${easyTax.appKey}")
    private String appKey;

    @Value("${easyTax.securityKey}")
    private String securityKey;

    @Value("${easyTax.orderNotifyUrl}")
    private String orderNotifyUrl;


    /**
     * 银⾏卡实时下单
     *
     * @param orderId
     * @param realName
     * @param cardNo
     * @param idCard
     * @param pay
     * @param payRemark
     * @return
     */
    public PaymentBankPayResponse paymentBankPay(Long orderId, String realName, String cardNo, String idCard, String pay, String payRemark, WithdrawPayTypeEnum payTypeEnum) {
        PaymentBankPayRequest orderRequest = new PaymentBankPayRequest();
        orderRequest.setDealerId(dealerId);
        orderRequest.setBrokerId(getBrokerId(payTypeEnum));
        orderRequest.setDealerOrderNo(Convert.toStr(orderId));
        orderRequest.setRealName(realName);
        orderRequest.setIdCardType("身份证");
        orderRequest.setIdCardNo(idCard);
        orderRequest.setPhoneNo("");
        orderRequest.setCurrency("CNY");
        orderRequest.setBankcardNo(cardNo);
        orderRequest.setTradeAmount(pay);
        orderRequest.setTradeRemark(payRemark);
        orderRequest.setNotifyUrl(orderNotifyUrl);
        return easyTaxClientManage.getClient().execute(orderRequest);
    }

    /**
     * 解密数据
     *
     * @param encryptStr
     * @return
     */
    public String decodeData(String encryptStr) {
        return EasyTaxUtil.decryptData(encryptStr, securityKey, appKey);
    }

    /**
     * 获取签名
     *
     * @param encryptStr
     * @param timestamp
     * @param requestId
     * @return
     */
    public String getSign(String encryptStr, String timestamp, String requestId) {
        return EasyTaxUtil.createSign(encryptStr, timestamp, appKey, signType, requestId, dealerId);
    }

    private String getBrokerId(WithdrawPayTypeEnum withdrawPayType) {
        switch (withdrawPayType) {
            case EASY_TAX_TRANSFER_1:
                return brokerId1;
            case EASY_TAX_TRANSFER_2:
                return brokerId2;
            case EASY_TAX_TRANSFER_3:
                return brokerId3;
            case EASY_TAX_TRANSFER_4:
                return brokerId4;
            default:
        }
        return null;
    }
}
