package com.starbuds.server.service.finance.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftSceneTypeEnum;
import com.starbuds.server.common.constant.media.GiftSourceTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.db.SendGiftSnapshotDO;
import com.starbuds.server.common.pojo.db.UserBagDO;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.finance.BagItemAddDTO;
import com.starbuds.server.common.pojo.finance.LuckyGiftLotteryConfig;
import com.starbuds.server.common.pojo.finance.gift.*;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.mq.QueueGiftPaidMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcGiftPaidMsg;
import com.starbuds.server.common.pojo.mq.QueueVoiceGiftPaidMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoomGiftHis;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.SendGiftSnapshotMsgService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.rtc.RtcRoomGiftHisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.impl.gift.GiftRate;
import com.starbuds.server.service.finance.lucky.LuckyGiftProvider;
import com.starbuds.server.service.finance.provider.*;
import com.starbuds.server.service.finance.provider.gift.GiftExecute;
import com.starbuds.server.service.finance.provider.ratio.RtcRatioProvider;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.utils.DateUtil.getTimestamp;
import static com.starbuds.server.service.finance.util.SpringContextUtil.isDev;

/**
 * 背包服务实现
 *
 * @author Lay
 * @date 2019/12/24
 */
@Service(version = "1.0.0")
@Slf4j
public class BagServiceImpl implements BagService {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Resource
    IBagProvider bagProvider;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    LuckyGiftProvider luckyGiftProvider;

    @Resource
    SettleProvider settleProvider;

    @Resource
    WalletProvider walletProvider;

    @Resource
    FamilyWalletProvider familyWalletProvider;

    @Resource
    private GiftProvider giftProvider;

    @Resource
    JpaTransactionManager transactionManager;

    @Resource
    TransactionDefinition transactionDefinition;


    @Reference(version = "1.0.0")
    RtcRoomGiftHisService rtcRoomGiftHisService;

    @Reference(version = "1.0.0")
    SendGiftSnapshotMsgService sendGiftSnapshotMsgService;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Value("${gift.familyUserRatio}")
    private double giftFamilyUserRatio;

    @Resource
    private CacheProvider cacheProvider;

    @Resource
    private SendGiftSnapshotProvider sendGiftSnapshotProvider;

    @Resource
    private RtcRatioProvider rtcRatioProvider;

    /**
     * 存入物品
     *
     * @param userId
     * @param itemId
     * @param itemType
     * @param quantity
     * @param expireTime
     * @param remarks
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addItem(Long userId, Long itemId, Integer itemType, Integer quantity, Long expireTime, String remarks, Long orderId) {
        return bagProvider.addQuantity(userId, itemId, itemType, quantity, expireTime, remarks, orderId);
    }

    /**
     * 批量存入物品
     *
     * @param bagItemAddDTOList
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean batchAddItem(List<BagItemAddDTO> bagItemAddDTOList) {
        try {
            for (BagItemAddDTO bagItemAddDTO : bagItemAddDTOList) {
                if (!bagProvider.addQuantity(bagItemAddDTO.getUserId(), bagItemAddDTO.getItemId(), bagItemAddDTO.getItemType(), bagItemAddDTO.getQuantity(), bagItemAddDTO.getExpireTime(), bagItemAddDTO.getRemarks(), bagItemAddDTO.getOrderId())) {
                    throw new Exception(JSON.toJSONString(bagItemAddDTO));
                }
            }
            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("批量存入物品失败", e);
        }
        return false;
    }

    /**
     * 存入礼物
     *
     * @param userId
     * @param giftId
     * @param quantity
     * @param expireTime
     * @param remarks
     * @param orderId
     * @return
     */
    @Override
    public boolean addGift(Long userId, Long giftId, Integer quantity, Long expireTime, String remarks, Long orderId) {
        return bagProvider.addQuantity(userId, giftId, BagItemTypeEnum.GIFT.getValue(), quantity, expireTime, remarks, orderId);
    }

    /**
     * 取出礼物
     *
     * @param userId
     * @param giftId
     * @param quantity
     * @param remarks
     * @param orderId
     * @return
     */
    @Override
    public Integer takeGift(Long userId, Long giftId, Integer quantity, String remarks, Long orderId) {
        return bagProvider.reduceQuantity(userId, giftId, BagItemTypeEnum.GIFT.getValue(), quantity, remarks, orderId);
    }

    /**
     * 从背包赠送礼物
     *
     * @param receiverUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Override
    public Integer sendGift(Long receiverUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId) {
        return sendGift(receiverUserId, senderUserId, giftId, quantity, channelId, GiftSceneTypeEnum.Live, null);
    }

    /**
     * 从背包赠送礼物
     *
     * @param receiverUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelName
     * @return
     */
    @Override
    public Integer sendGift(Long receiverUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId, GiftSceneTypeEnum giftSceneType, String channelName) {
        Long orderId = identityService.getId();
        UserCardVO anchor = userService.getCard(receiverUserId);
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);
        if (anchor != null && giftPayItem != null) {

            Long amount = giftPayItem.getGiftPrice() * quantity;
            Long anchorAmount = Long.valueOf(amount);

            LuckyGiftLotteryConfig lotteryConfig = luckyGiftProvider.getConfig();
            if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                if (lotteryConfig == null) {
                    log.error("幸运礼物未配置");
                    return null;
                }

                anchorAmount = BigDecimal.valueOf(amount).multiply(new BigDecimal(String.valueOf(lotteryConfig.getAnchorRatio()))).setScale(0, RoundingMode.DOWN).longValue();
            }

            String remarks = String.format("赠送给(%s)[%d]", anchor.getUserName(), anchor.getUserId());
            Integer quantityRemain = bagProvider.reduceQuantity(senderUserId, giftId, BagItemTypeEnum.GIFT.getValue(), quantity, remarks, orderId);
            if (quantityRemain < 0) {
                return quantityRemain;
            }

            // 异步主播结算
            settleProvider.settleByGift(giftPayItem, receiverUserId, senderUserId, quantity, anchorAmount, orderId);

            // 数量足够
            QueueGiftPaidMsg queueGiftPaidMsg = new QueueGiftPaidMsg();
            queueGiftPaidMsg.setOrderId(orderId);
            queueGiftPaidMsg.setGiftId(giftId);
            queueGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
            queueGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
            queueGiftPaidMsg.setQuantity(quantity);
            queueGiftPaidMsg.setAnchorAmount(anchorAmount);
            queueGiftPaidMsg.setSenderUserId(senderUserId);
            queueGiftPaidMsg.setReceiverUserId(receiverUserId);
            queueGiftPaidMsg.setFromBag(true);
            queueGiftPaidMsg.setChannelId(channelId);
            queueGiftPaidMsg.setGiftSceneType(giftSceneType.getValue());
            queueGiftPaidMsg.setChannelName(channelName);
            queueGiftPaidMsg.setSourceType(GiftSourceTypeEnum.BAG.getValue());
            queueGiftPaidMsg.setGiftType(giftPayItem.getGiftType());

            // 幸运礼物异步抽奖
            if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                luckyGiftProvider.asyncLottery(queueGiftPaidMsg);
            } else {
                // 丢出礼物赠送消息到消息队列
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.GIFT_PAID, queueGiftPaidMsg.serialize());
            }

            return quantityRemain;
        }
        return null;
    }

    /**
     * 从背包赠送语音房礼物
     *
     * @param roomId
     * @param targetUserSeats -> userId:seatNo
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Override
    public SendGiftOrder sendRtcGift(Long roomId, Map<Long, Integer> targetUserSeats, Long senderUserId, Long giftId, Integer quantity, Long channelId, String appVer) {
        Long orderId = identityService.getId();
        boolean allSeat = targetUserSeats.size() > 7 && targetUserSeats.size() == rtcSeatService.getSeatUserCount(roomId, false, senderUserId);

        // 礼物信息 目前仅支持普通星钻礼物
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);
        if (giftPayItem == null || WalletTypeEnum.STAR_DIAMOND.getValue() != giftPayItem.getGiftPriceType()) {
            return null;
        }
        if (GiftTypeEnum.COMMON.getValue() != giftPayItem.getGiftType()) {
            return null;
        }
        if (CollectionUtil.isEmpty(targetUserSeats)) {
            return null;
        }

        // 单个用户价格
        Long singleAmount = giftPayItem.getGiftPrice() * quantity;
        // 获取用户信息
        Map<Long, UserCardVO> userCardMap = userService.getNameAndAvatarByIds(new ArrayList<>(targetUserSeats.keySet()))
                .stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
        UserCardVO sender = userService.getCard(senderUserId);
        // 用户家族map userId:familyId
        Map<Long, Long> userFamilyMap = familyService.getFamilyIds(targetUserSeats.keySet());
        RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(roomId);

        // 尝试扣除数量
        Integer quantityRemain = null;
        int roomFamilyUserCount = 0;
        TransactionStatus transactionStatus = transactionManager.getTransaction(transactionDefinition);
        try {
            // 房间家族
//            Long roomFamilyId = rtcRoomService.getFamilyId(roomId);
            Long roomFamilyId = rtcRatioProvider.getFamilyId(rtcRoomRatio);
            List receiverUserIds = new ArrayList<>(targetUserSeats.size());
            List receiverUserNames = new ArrayList<>(targetUserSeats.size());
            Long totalPay = 0L;
            for (Long receiverUserId : targetUserSeats.keySet()) {
                UserCardVO receiver = userCardMap.get(receiverUserId);
                quantityRemain = bagProvider.reduceQuantity(senderUserId, giftId, BagItemTypeEnum.GIFT.getValue(), quantity,
                        String.format("赠送给(%s)[%d]%d个%s", receiver.getUserName(), receiverUserId, quantity, giftPayItem.getGiftName()), orderId);
                if (quantityRemain == null || quantityRemain < 0) {
                    throw new RuntimeException("数量不足");
                }
                receiverUserIds.add(receiverUserId);
                receiverUserNames.add(receiver.getUserName());
                totalPay += singleAmount;

                /*
                // 用户实际分成比例
                BigDecimal ratio = null;
                // 用户家族
                Long userFamilyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                // 是否设置分成比例
                Double userRatio = rtcRoomService.getUserRatio(roomId);
                if (userRatio != null) {
                    ratio = Convert.toBigDecimal(userRatio);
                } else {
                    ratio = Convert.toBigDecimal(giftNoFamilyUserRatio);
                    // 用户家族
                    if (roomFamilyId != null && roomFamilyId.equals(userFamilyId)) {
                        ratio = Convert.toBigDecimal(giftFamilyUserRatio);
                        roomFamilyUserCount++;
                    }
                }*/
                // 用户家族
                Long userFamilyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                if (roomFamilyId != null && roomFamilyId.equals(userFamilyId)) {
                    roomFamilyUserCount++;
                }
                // 用户实际分成比例
                BigDecimal userRatio = rtcRatioProvider.getUserRatio(userFamilyId, rtcRoomRatio);
                rtcRatioProvider.checkRatio(userRatio);
                log.info("roomFamilyId {} userFamilyId {} ratio {} receiverUserId {}", roomFamilyId, userFamilyId, userRatio, receiverUserId);
                // 接收者
                Long dustAmount = BigDecimal.valueOf(singleAmount).multiply(userRatio).setScale(0, RoundingMode.DOWN).longValue();
                String remark = String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName());
                walletProvider.addBalance(WalletTypeEnum.STAR_DUST, receiverUserId, dustAmount, remark, orderId);

                //增加礼物记录
                RtcRoomGiftHis rmh = new RtcRoomGiftHis();
                rmh.setId(identityService.getId());
                rmh.setUserId(senderUserId);
                rmh.setTargetUserId(receiverUserId);
                rmh.setTargetUserfamilyId(userFamilyId == null ? 0L : userFamilyId);
                rmh.setRoomFamilyId(roomFamilyId == null ? 0L : roomFamilyId);
                rmh.setGiftId(giftPayItem.getGiftId());
                rmh.setGiftType(StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue());
                rmh.setRoomId(roomId);
                rmh.setOrderId(orderId);
                rmh.setPrice(dustAmount);
                rmh.setRemark(remark);
                setInsertTime(rmh);
                rtcRoomGiftHisService.insertRtcRoomGiftHis(rmh);

                // 是否家族房间
                if (roomFamilyId != null) {
//                    BigDecimal giftFamilyRatio = cacheProvider.familyRatio(roomFamilyId);
                    BigDecimal giftFamilyRatio = rtcRatioProvider.getFamilyRatio(rtcRoomRatio);
                    rtcRatioProvider.checkRatio(userRatio, giftFamilyRatio);
                    BigDecimal presidentAmount = BigDecimal.valueOf(singleAmount)
                            .multiply(giftFamilyRatio)
                            .setScale(2, RoundingMode.DOWN);
                    // 返到家族钱包
                    familyWalletProvider.addBalance(roomFamilyId, presidentAmount,
                            String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName()), orderId);
                    log.info(LogUtil.devLog("从背包赠送语音房礼物-familyIncome-{} Gift{id={} price={} quantity={}} {} * {}=[{}]"),
                            roomFamilyId, giftPayItem.getGiftId(), giftPayItem.getGiftPrice(), quantity, singleAmount, giftFamilyRatio, presidentAmount);
                }
            }

            SendGiftSnapshotDO sendGiftSnapshot = new SendGiftSnapshotDO();
            sendGiftSnapshot.setOrderId(orderId);
            sendGiftSnapshot.setSenderUserId(senderUserId);
            sendGiftSnapshot.setSenderUserName(sender.getUserName());
            sendGiftSnapshot.setReceiverUserIds(StringUtils.join(receiverUserIds, ","));
            sendGiftSnapshot.setReceiverUserNames(StringUtils.join(receiverUserNames, ","));
            sendGiftSnapshot.setGiftId(giftId);
            sendGiftSnapshot.setGiftName(giftPayItem.getGiftName());
            sendGiftSnapshot.setGiftType(giftPayItem.getGiftType());
            sendGiftSnapshot.setGiftPrice(giftPayItem.getGiftPrice());
            sendGiftSnapshot.setGiftPriceType(giftPayItem.getGiftPriceType());
            sendGiftSnapshot.setQuantity(quantity);
            sendGiftSnapshot.setIsFromBag(BoolIntVal.True);
            sendGiftSnapshot.setTotalPay(totalPay);
            sendGiftSnapshot.setRoomId(roomId);
            sendGiftSnapshot.setRoomFamilyId(roomFamilyId);
            if (rtcRoomRatio != null && rtcRoomRatio.getBusinessType() != null) {
                sendGiftSnapshot.setRoomBusinessType(rtcRoomRatio.getBusinessType().getValue());
            }
            SendGiftSnapshotDO snapshotDO = sendGiftSnapshotProvider.create(sendGiftSnapshot);

            sendGiftSnapshotMsgService.sendMQMsg(snapshotDO);

            transactionManager.commit(transactionStatus);
        } catch (Exception e) {
            transactionManager.rollback(transactionStatus);
            if (!"数量不足".equals(e.getMessage())) {
                log.error("背包礼物赠送失败", e);
            }
            return null;
        }

        QueueRtcGiftPaidMsg rtcGiftPaidMsg = new QueueRtcGiftPaidMsg();
        rtcGiftPaidMsg.setOrderId(orderId);
        rtcGiftPaidMsg.setGiftId(giftId);
        rtcGiftPaidMsg.setGiftType(giftPayItem.getGiftType());
        rtcGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
        rtcGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
        rtcGiftPaidMsg.setQuantity(quantity);
        rtcGiftPaidMsg.setSenderUserId(senderUserId);
        rtcGiftPaidMsg.setTargetUserSeat(targetUserSeats);
        rtcGiftPaidMsg.setFromBag(true);
        rtcGiftPaidMsg.setChannelId(channelId);
        rtcGiftPaidMsg.setRoomId(roomId);
        rtcGiftPaidMsg.setAllSeat(BoolIntVal.fromBool(allSeat));
        rtcGiftPaidMsg.setRoomFamilyUserCount(roomFamilyUserCount);
        rtcGiftPaidMsg.setAppVer(appVer);
        rtcGiftPaidMsg.setRoomBusinessType(rtcRoomRatio.getBusinessType());
        // 丢出礼物赠送消息到消息队列
        boolean send = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_GIFT_PAID, rtcGiftPaidMsg.serialize());
        if (Boolean.FALSE.equals(send)) {
            String errMsg = "丢出礼物赠送消息到消息队列失败 msg=";
            if (isDev()) {
                log.error(LogUtil.devLog(errMsg + JSON.toJSONString(rtcGiftPaidMsg, true)));
            } else {
                log.error(errMsg + rtcGiftPaidMsg);
            }
        }
        SendGiftOrder order = new SendGiftOrder();
        order.setOrderId(orderId);
        order.setQuantity(quantityRemain);
        return order;
    }


    /**
     * 从背包赠送语音速配礼物
     *
     * @param hisId
     * @param receiveUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Override
    public Integer sendVoiceGift(Long hisId, Long receiveUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId) {
        Long orderId = identityService.getId();

        // 礼物信息 目前仅支持普通星钻礼物
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);
        if (WalletTypeEnum.STAR_DIAMOND.getValue() != giftPayItem.getGiftPriceType()) {
            return null;
        }
        if (GiftTypeEnum.COMMON.getValue() != giftPayItem.getGiftType()) {
            return null;
        }

        if (receiveUserId != null && giftPayItem != null) {
            Long totalAmount = giftPayItem.getGiftPrice() * quantity;

            // 获取用户信息
            UserCardVO sender = userService.getCard(senderUserId);
            UserCardVO receiver = userService.getCard(receiveUserId);

            // 尝试扣除数量
            Integer quantityRemain = null;
            try {
                quantityRemain = bagProvider.reduceQuantity(senderUserId, giftId, BagItemTypeEnum.GIFT.getValue(), quantity, String.format("赠送给(%s)[%d]%d个%s", receiver.getUserName(), receiveUserId, quantity, giftPayItem.getGiftName()), orderId);
                if (quantityRemain == null || quantityRemain < 0) {
                    throw new RuntimeException("数量不足");
                }

                BigDecimal ratio = Convert.toBigDecimal(giftFamilyUserRatio);
                // 用户结算
                if (familyService.getFamilyId(receiveUserId) == null) {
                    ratio = Convert.toBigDecimal(giftNoFamilyUserRatio);
                }
                Long dustAmount = BigDecimal.valueOf(totalAmount).multiply(ratio).setScale(0, RoundingMode.DOWN).longValue();
                walletProvider.addBalance(WalletTypeEnum.STAR_DUST, receiveUserId, dustAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName()), orderId);
            } catch (Exception e) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.info("出现异常", e);
                return null;
            }

            QueueVoiceGiftPaidMsg voiceGiftPaidMsg = new QueueVoiceGiftPaidMsg();
            voiceGiftPaidMsg.setOrderId(orderId);
            voiceGiftPaidMsg.setHisId(hisId);
            voiceGiftPaidMsg.setGiftId(giftId);
            voiceGiftPaidMsg.setGiftType(giftPayItem.getGiftType());
            voiceGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
            voiceGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
            voiceGiftPaidMsg.setQuantity(quantity);
            voiceGiftPaidMsg.setSenderUserId(senderUserId);
            voiceGiftPaidMsg.setReceiveUserId(receiveUserId);
            voiceGiftPaidMsg.setFromBag(true);
            voiceGiftPaidMsg.setChannelId(channelId);

            // 丢出礼物赠送消息到消息队列
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.VOICE_GIFT_PAID, voiceGiftPaidMsg.serialize());

            return quantityRemain;
        }

        return null;
    }

    /*@Override
    @Transactional(rollbackFor = Exception.class)
    public Long sendGift(SendGiftByFinanceAbstract sendGift) {
        log.info("接收到发送礼物对象 {}", JSON.toJSONString(sendGift));
        // 校验数据
        String errMsg = checkSendGiftByFinanceBO(sendGift);
        if (errMsg != null) {
            log.error("发送礼物失败 {}", errMsg);
            return null;
        }

        // 根据发送礼物类型建造发送礼物业务对象
        GiftExecute giftExecute = giftProvider.giftProvider(sendGift);
        sendGift = giftExecute.buildSendGift(sendGift);
        sendGift.setOrderId(identityService.getId());
        Long result;
        if (sendGift.getIsFromBag()) {
            // 消耗礼物，从背包赠送
            result = sendGiftByBag(sendGift, giftExecute);
        } else {
            // 消耗星钻赠送
            result = sendGiftByDiamond(sendGift, giftExecute);
        }

        return result;
    }*/

    /**
     * 发送礼物-从背包赠送
     *
     * @param bo
     * @param giftExecute
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Deprecated
    public Long sendGiftByBag(SendGiftByFinanceAbstract bo, GiftExecute giftExecute) {
        GiftRate giftRate = giftExecute.giftRate();
        Long orderId = bo.getOrderId();
        final Integer quantity = bo.getQuantity();
        List<SendGiftResult> results = new ArrayList<>(bo.getReceiverUsers().size());

        Long senderUserId = bo.getSenderUserId();
        String senderUserName = bo.getSenderUserName();
        Long giftId = bo.getGiftId();
        String giftName = bo.getGiftName();
        Integer bagRemainder = null;
        List receiverUserIds = new ArrayList<>(bo.getReceiverUsers().size());
        List receiverUserNames = new ArrayList<>(bo.getReceiverUsers().size());
        Long totalPay = 0L;
        for (GiftReceiver receiver : bo.getReceiverUsers()) {
            Long receiverUserId = receiver.getUserId();
            String receiverUserName = receiver.getUserName();

            // 减少礼物发送者的背包中的物品数量
            bagRemainder = bagProvider.reduceQuantity(senderUserId, giftId, BagItemTypeEnum.GIFT.getValue(),
                    quantity,
                    String.format("赠送给(%s)[%d]%d个%s", receiverUserName, receiverUserId, quantity, giftName),
                    orderId);
            if (bagRemainder == null || bagRemainder < 0) {
                // 回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                // 数量不足时终止处理
                return Convert.toLong(bagRemainder);
            }
            receiverUserIds.add(receiverUserId);
            receiverUserNames.add(receiver.getUserName());
            Long unitPrice = bo.getGiftPrice() * quantity;
            totalPay += unitPrice;

            // 计算礼物收益比率 💪 封装变化
            BigDecimal rate = giftRate.calculate(bo, receiver);
            // 单个用户获得的礼物价值
            // 接收者收到的金额
            Long receiveAmount = BigDecimal.valueOf(unitPrice).multiply(rate).setScale(0, RoundingMode.DOWN).longValue();
            // 增加礼物接收者的粉钻
            WalletTypeEnum walletType = WalletTypeEnum.STAR_DUST;
            walletProvider.addBalance(walletType, receiverUserId, receiveAmount,
                    String.format("[%s](%d)赠送%d个%s", senderUserName, senderUserId, quantity, giftName),
                    orderId);

            SendGiftResult result = new SendGiftResult();
            result.setRate(rate);
            result.setUnitPrice(unitPrice);
            result.setReceiveAmount(receiveAmount);
            result.setWalletType(walletType);
            result.setBagRemainder(bagRemainder);
            result.setReceiverUserId(receiverUserId);
            result.setReceiverUserName(receiverUserName);
            result.setSenderUserId(senderUserId);
            result.setSenderUserName(senderUserName);
            result.setGiftId(giftId);
            result.setGiftName(giftName);
            result.setQuantity(quantity);
            results.add(result);
        }

        if (CollectionUtil.isNotEmpty(results)) {
            giftExecute.doAfterSendByBag(bo, results);
            // 发送消息
            giftExecute.sendMsg(bo, results);
        }

        SendGiftSnapshotDO sendGiftSnapshot = new SendGiftSnapshotDO();
        sendGiftSnapshot.setOrderId(orderId);
        sendGiftSnapshot.setSenderUserId(senderUserId);
        sendGiftSnapshot.setSenderUserName(senderUserName);
        sendGiftSnapshot.setReceiverUserIds(StringUtils.join(receiverUserIds, ","));
        sendGiftSnapshot.setReceiverUserNames(StringUtils.join(receiverUserNames, ","));
        sendGiftSnapshot.setGiftId(giftId);
        sendGiftSnapshot.setGiftName(giftName);
        sendGiftSnapshot.setGiftType(bo.getGiftType());
        sendGiftSnapshot.setGiftPrice(bo.getGiftPrice());
        sendGiftSnapshot.setGiftPriceType(bo.getGiftPriceType());
        sendGiftSnapshot.setQuantity(quantity);
        sendGiftSnapshot.setIsFromBag(BoolIntVal.True);
        sendGiftSnapshot.setTotalPay(totalPay);
        if (bo instanceof SendGiftRtcUnderMaiBO) {
            SendGiftRtcUnderMaiBO rtcBo = (SendGiftRtcUnderMaiBO) bo;
            sendGiftSnapshot.setRoomId(rtcBo.getRoomId());
            sendGiftSnapshot.setRoomFamilyId(rtcBo.getRoomFamilyId());
        }
        SendGiftSnapshotDO snapshotDO = sendGiftSnapshotProvider.create(sendGiftSnapshot);

        sendGiftSnapshotMsgService.sendMQMsg(snapshotDO);
        return Convert.toLong(bagRemainder);
    }

    /**
     * 发送礼物-消耗星钻赠送
     *
     * @param bo
     * @param giftExecute
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Deprecated
    public Long sendGiftByDiamond(SendGiftByFinanceAbstract bo, GiftExecute giftExecute) {
        GiftRate giftRate = giftExecute.giftRate();
        boolean isLotteryGift = GiftTypeEnum.LOTTERY.getValue() == bo.getGiftType();
        WalletTypeEnum walletType = WalletTypeEnum.STAR_DUST;
        Long orderId = bo.getOrderId();
        final Integer quantity = bo.getQuantity();
        List<SendGiftResult> results = new ArrayList<>(bo.getReceiverUsers().size());

        Long balance = null;
        Long senderUserId = bo.getSenderUserId();
        String senderUserName = bo.getSenderUserName();
        Long giftId = bo.getGiftId();
        String giftName = bo.getGiftName();
        List receiverUserIds = new ArrayList<>(bo.getReceiverUsers().size());
        List receiverUserNames = new ArrayList<>(bo.getReceiverUsers().size());
        Long totalPay = 0L;
        for (GiftReceiver receiver : bo.getReceiverUsers()) {
            Long receiverUserId = receiver.getUserId();
            String receiverUserName = receiver.getUserName();

            // 计算礼物收益比率 💪 封装变化
            BigDecimal rate = giftRate.calculate(bo, receiver);
            // 单个用户获得的礼物价值
            Long unitPrice = bo.getGiftPrice() * quantity;
            // 接收者收到的金额
            Long receiveAmount = BigDecimal.valueOf(unitPrice).multiply(rate).setScale(0, RoundingMode.DOWN).longValue();

            // 尝试扣费
            balance = walletProvider.reduceBalance(bo.getGiftPriceType(), senderUserId, -unitPrice,
                    String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverUserId, quantity, bo.getGiftName()),
                    orderId);
            if (balance == null || balance < 0) {
                // 回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return balance;
            }
            receiverUserIds.add(receiverUserId);
            receiverUserNames.add(receiver.getUserName());
            totalPay += unitPrice;

            if (!isLotteryGift) {
                // 用户结算
                walletProvider.addBalance(walletType, receiverUserId, receiveAmount,
                        String.format("[%s](%d)赠送%d个%s", bo.getSenderUserName(), senderUserId, quantity, bo.getGiftName()),
                        orderId);
            }

            SendGiftResult result = new SendGiftResult();
            result.setRate(rate);
            result.setUnitPrice(unitPrice);
            result.setReceiveAmount(receiveAmount);
            result.setWalletType(walletType);
            result.setReceiverUserId(receiverUserId);
            result.setReceiverUserName(receiverUserName);
            result.setSenderUserId(senderUserId);
            result.setSenderUserName(senderUserName);
            result.setGiftId(giftId);
            result.setGiftName(giftName);
            result.setQuantity(quantity);
            result.setBalance(balance);
            results.add(result);
        }

        if (CollectionUtil.isNotEmpty(results)) {
            // 家族收益
            giftExecute.doAfterSendByDiamond(bo, results);
            // 发送消息
            giftExecute.sendMsg(bo, results);
        }

        SendGiftSnapshotDO sendGiftSnapshot = new SendGiftSnapshotDO();
        sendGiftSnapshot.setOrderId(orderId);
        sendGiftSnapshot.setSenderUserId(senderUserId);
        sendGiftSnapshot.setSenderUserName(senderUserName);
        sendGiftSnapshot.setReceiverUserIds(StringUtils.join(receiverUserIds, ","));
        sendGiftSnapshot.setReceiverUserNames(StringUtils.join(receiverUserNames, ","));
        sendGiftSnapshot.setGiftId(giftId);
        sendGiftSnapshot.setGiftName(giftName);
        sendGiftSnapshot.setGiftType(bo.getGiftType());
        sendGiftSnapshot.setGiftPrice(bo.getGiftPrice());
        sendGiftSnapshot.setGiftPriceType(bo.getGiftPriceType());
        sendGiftSnapshot.setQuantity(quantity);
        sendGiftSnapshot.setIsFromBag(BoolIntVal.False);
        sendGiftSnapshot.setTotalPay(totalPay);
        if (bo instanceof SendGiftRtcUnderMaiBO) {
            SendGiftRtcUnderMaiBO rtcBo = (SendGiftRtcUnderMaiBO) bo;
            sendGiftSnapshot.setRoomId(rtcBo.getRoomId());
            sendGiftSnapshot.setRoomFamilyId(rtcBo.getRoomFamilyId());
        }
        SendGiftSnapshotDO snapshotDO = sendGiftSnapshotProvider.create(sendGiftSnapshot);

        sendGiftSnapshotMsgService.sendMQMsg(snapshotDO);
        return balance;
    }

    /**
     * 获取背包礼物列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<BagItem<GiftItemVO>> getGiftList(Long userId) {
        List<BagItem<GiftItemVO>> userBagItems = bagProvider.getUserGiftItems(userId);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取背包道具列表
     *
     * @param userId
     * @param propsType
     * @return
     */
    @Override
    public List<BagItem<PropsVO>> getPropsList(Long userId, Integer propsType) {
        List<BagItem<PropsVO>> userBagItems = bagProvider.getUserPropsItems(userId, propsType);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取背包道具
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Override
    public BagItem<PropsVO> getProps(String bagId, Long userId) {
        return bagProvider.getProps(bagId, userId);
    }

    /**
     * 获取背包坐骑列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<BagItem<MountsVO>> getMountsList(Long userId) {
        List<BagItem<MountsVO>> userBagItems = bagProvider.getUserMountsItems(userId);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取背包勋章列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<BagItem<MedalVO>> getMedalList(Long userId) {
        List<BagItem<MedalVO>> userBagItems = bagProvider.getUserMedalItems(userId);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取头饰列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<BagItem<HeadwearVO>> getHeadwearList(Long userId) {
        List<BagItem<HeadwearVO>> userBagItems = bagProvider.getUserHeadwearItems(userId);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取聊天气泡列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<BagItem<ChatBubbleVO>> getChatBubbleList(Long userId) {
        List<BagItem<ChatBubbleVO>> userBagItems = bagProvider.getUserChatBubbleItems(userId);
        if (CollectionUtil.isNotEmpty(userBagItems)) {
            return userBagItems;
        }
        return Collections.emptyList();
    }

    /**
     * 获取背包物品数量
     *
     * @param userId
     * @param itemType
     * @param itemId
     * @return
     */
    @Override
    public int getItemQuantity(Long userId, BagItemTypeEnum itemType, Long itemId) {
        return bagProvider.getQuantityByItemId(userId, itemType, itemId);
    }

    /**
     * 查询道具数量
     *
     * @param userId
     * @param propsType
     * @return
     */
    @Override
    public int getPropsQuantityByType(Long userId, PropsTypeEnum propsType) {
        return bagProvider.getPropsQuantityByType(userId, propsType);
    }

    /**
     * 根据道具类型取出道具
     *
     * @param userId
     * @param propsType
     * @param quantity
     * @param remarks
     * @param orderId
     * @return
     */
    @Override
    public Integer takePropsByType(Long userId, PropsTypeEnum propsType, Integer quantity, String remarks, Long orderId) {
        List<Long> propsIds = propsService.getPropsIdsByType(propsType);
        if (CollectionUtil.isNotEmpty(propsIds)) {
            for (Long propId : propsIds) {
                Integer remain = bagProvider.reduceQuantity(userId, propId, BagItemTypeEnum.PROPS.getValue(), quantity, remarks, orderId);
                if (remain > -1) {
                    return remain;
                }
            }
            return -1;
        }
        return null;
    }

    /**
     * 取出物品
     *
     * @param userId
     * @param itemId
     * @param itemType
     * @param quantity
     * @param hisRemarks
     * @param orderId
     * @return
     */
    @Override
    public Integer takeItem(Long userId, Long itemId, Integer itemType, Integer quantity, String hisRemarks, Long orderId) {
        return bagProvider.reduceQuantity(userId, itemId, itemType, quantity, hisRemarks, orderId);
    }

    /**
     * 兑换坐骑
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addMountsByBagId(String bagId, Long userId) {
        BagItem<PropsVO> bagItem = getProps(bagId, userId);

        // 有效性判断
        if (bagItem == null || bagItem.getQuantity() < 1 || PropsTypeEnum.MOUNTS_CARD != PropsTypeEnum.toEnum(bagItem.getItem().getPropsType())) {
            return false;
        }

        boolean autoEnable = bagProvider.getQuantityByType(userId, BagItemTypeEnum.MOUNTS) == 0;

        // 增加坐骑
        Long expireTime = Optional.ofNullable(bagItem.getItem().getPropsExtLong()).orElse(-1L);
        if (expireTime > 0) {
            expireTime += System.currentTimeMillis();
        }

        Long mountsId = Convert.toLong(bagItem.getItem().getPropsExtString());
        if (bagProvider.addQuantity(userId, mountsId, BagItemTypeEnum.MOUNTS.getValue(), 1, expireTime, String.format("%s兑换", bagItem.getItem().getPropsName()), 0L)) {
            Integer quantity = bagProvider.reduceQuantity(userId, bagItem.getItemId(), BagItemTypeEnum.PROPS.getValue(), 1, "兑换坐骑", 0L);
            if (quantity == null || quantity < 0) {
                throw new RuntimeException("坐骑卡消耗失败");
            }
        }

        // 判断当前用户是否有坐骑 没有的话默认启用
        if (autoEnable) {
            UserSwitchDO switchDO = new UserSwitchDO();
            switchDO.setUserId(userId);
            switchDO.setItemId(mountsId);
            switchDO.setSwitchId(String.format("%d:%d", userId, SwitchTypeEnum.MOUNTS.getCode()));
            switchDO.setSwitchStatus(SwitchStatusEnum.OPEN.getCode());
            switchDO.setType(SwitchTypeEnum.MOUNTS.getCode());
            userService.setSwitch(switchDO);
        }

        return true;
    }

    /**
     * 兑换头饰
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addHeadwearByBagId(String bagId, Long userId) {
        BagItem<PropsVO> bagItem = getProps(bagId, userId);

        // 有效性判断
        if (bagItem == null || bagItem.getQuantity() < 1 || PropsTypeEnum.HEADWEAR_CARD != PropsTypeEnum.toEnum(bagItem.getItem().getPropsType())) {
            return false;
        }

        boolean autoEnable = bagProvider.getQuantityByType(userId, BagItemTypeEnum.HEADWEAR) == 0;

        // 增加头饰
        Long expireTime = Optional.ofNullable(bagItem.getItem().getPropsExtLong()).orElse(-1L);
        if (expireTime > 0) {
            expireTime += System.currentTimeMillis();
        }

        Long mountsId = Convert.toLong(bagItem.getItem().getPropsExtString());
        if (bagProvider.addQuantity(userId, mountsId, BagItemTypeEnum.HEADWEAR.getValue(), 1, expireTime, String.format("%s兑换", bagItem.getItem().getPropsName()), 0L)) {
            Integer quantity = bagProvider.reduceQuantity(userId, bagItem.getItemId(), BagItemTypeEnum.PROPS.getValue(), 1, "兑换头饰", 0L);
            if (quantity == null || quantity < 0) {
                throw new RuntimeException("头饰卡消耗失败");
            }
        }

        // 判断当前用户是否有头饰 没有的话默认启用
        if (autoEnable) {
            UserSwitchDO switchDO = new UserSwitchDO();
            switchDO.setUserId(userId);
            switchDO.setItemId(mountsId);
            switchDO.setSwitchId(String.format("%d:%d", userId, SwitchTypeEnum.HEADWEAR.getCode()));
            switchDO.setSwitchStatus(SwitchStatusEnum.OPEN.getCode());
            switchDO.setType(SwitchTypeEnum.HEADWEAR.getCode());
            userService.setSwitch(switchDO);
        }

        return true;
    }

    /**
     * 碎片兑换礼物
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addGiftByFragment(String bagId, Long userId) {
        BagItem<PropsVO> bagItem = getProps(bagId, userId);

        // 有效性判断
        if (bagItem == null || bagItem.getQuantity() < bagItem.getItem().getPropsExtInt() || PropsTypeEnum.GIFT_FRAGMENT != PropsTypeEnum.toEnum(bagItem.getItem().getPropsType())) {
            return false;
        }

        // 计算可兑换礼物数量
        int giftQuantity = 1;

        // 需要碎片数量
        int needPropsQuantity = giftQuantity * bagItem.getItem().getPropsExtInt();
        Long giftId = Convert.toLong(bagItem.getItem().getPropsExtString());
        if (addGift(userId, giftId, giftQuantity, -1L, "碎片兑换", 0L)) {
            // 减少碎片数量
            Integer quantity = bagProvider.reduceQuantity(userId, bagItem.getItemId(), BagItemTypeEnum.PROPS.getValue(), needPropsQuantity, "兑换礼物", 0L);
            if (quantity == null || quantity < 0) {
                throw new RuntimeException("碎片兑换礼物失败");
            }
        }

        return true;
    }

    /**
     * 兑换聊天气泡
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addChatBubbleByBagId(String bagId, Long userId) {
        BagItem<PropsVO> bagItem = getProps(bagId, userId);

        // 有效性判断
        if (bagItem == null || bagItem.getQuantity() < 1 || PropsTypeEnum.CHAT_BUBBLE_CARD != PropsTypeEnum.toEnum(bagItem.getItem().getPropsType())) {
            return false;
        }

        boolean autoEnable = bagProvider.getQuantityByType(userId, BagItemTypeEnum.CHAT_BUBBLE) == 0;

        // 增加聊天气泡
        Long expireTime = Optional.ofNullable(bagItem.getItem().getPropsExtLong()).orElse(-1L);
        if (expireTime > 0) {
            expireTime += System.currentTimeMillis();
        }

        Long bubbleId = Convert.toLong(bagItem.getItem().getPropsExtString());
        if (bagProvider.addQuantity(userId, bubbleId, BagItemTypeEnum.CHAT_BUBBLE.getValue(), 1, expireTime, String.format("%s兑换", bagItem.getItem().getPropsName()), 0L)) {
            Integer quantity = bagProvider.reduceQuantity(userId, bagItem.getItemId(), BagItemTypeEnum.PROPS.getValue(), 1, "兑换聊天气泡", 0L);
            if (quantity == null || quantity < 0) {
                throw new RuntimeException("聊天气泡卡消耗失败");
            }
        }

        // 判断当前用户是否有聊天气泡 没有的话默认启用
        if (autoEnable) {
            UserSwitchDO switchDO = new UserSwitchDO();
            switchDO.setUserId(userId);
            switchDO.setItemId(bubbleId);
            switchDO.setSwitchId(String.format("%d:%d", userId, SwitchTypeEnum.CHAT_BUBBLE.getCode()));
            switchDO.setSwitchStatus(SwitchStatusEnum.OPEN.getCode());
            switchDO.setType(SwitchTypeEnum.CHAT_BUBBLE.getCode());
            userService.setSwitch(switchDO);
        }

        return true;
    }

    /**
     * 兑换勋章
     *
     * @param bagId
     * @param userId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addMedalByBagId(String bagId, Long userId) {
        BagItem<PropsVO> bagItem = getProps(bagId, userId);

        // 有效性判断
        if (bagItem == null || bagItem.getQuantity() < 1
                || PropsTypeEnum.MEDAL_CARD != PropsTypeEnum.toEnum(bagItem.getItem().getPropsType())) {
            return false;
        }

        boolean autoEnable = bagProvider.getQuantityByType(userId, BagItemTypeEnum.MEDAL) == 0;

        // 增加勋章
        Long expireTime = Optional.ofNullable(bagItem.getItem().getPropsExtLong()).orElse(-1L);
        if (expireTime > 0) {
            expireTime += System.currentTimeMillis();
        }

        Long medalId = Convert.toLong(bagItem.getItem().getPropsExtString());
        if (bagProvider.addQuantity(userId, medalId, BagItemTypeEnum.MEDAL.getValue(), 1, expireTime, String.format("%s兑换", bagItem.getItem().getPropsName()), 0L)) {
            Integer quantity = bagProvider.reduceQuantity(userId, bagItem.getItemId(), BagItemTypeEnum.PROPS.getValue(), 1, "兑换勋章", 0L);
            if (quantity == null || quantity < 0) {
                throw new RuntimeException("勋章消耗失败");
            }
        }

        // 判断当前用户是否有头饰 没有的话默认启用
        if (autoEnable) {
            UserSwitchDO switchDO = new UserSwitchDO();
            switchDO.setUserId(userId);
            switchDO.setItemId(medalId);
            switchDO.setSwitchId(String.format("%d:%d", userId, SwitchTypeEnum.MEDAL.getCode()));
            switchDO.setSwitchStatus(SwitchStatusEnum.OPEN.getCode());
            switchDO.setType(SwitchTypeEnum.MEDAL.getCode());
            userService.setSwitch(switchDO);
        }

        return true;
    }

    private void setInsertTime(RtcRoomGiftHis rmh) {
        long timestamp = System.currentTimeMillis();
        rmh.setTimeY(getTimestamp(timestamp, 4L));
        rmh.setTimeM(getTimestamp(timestamp, 3L));
        rmh.setTimeD(getTimestamp(timestamp, 2L));
        rmh.setTimeH(getTimestamp(timestamp, 1L));
        rmh.setCreatetime(timestamp);
    }


    /**
     * 获取用户背包礼物数量
     *
     * @param giftIds
     * @param userId
     * @return
     */
    @Override
    public Map<Long, Integer> getGiftOwnNumMap(List<Long> giftIds, Long userId) {
        if (CollectionUtil.isNotEmpty(giftIds)) {
            Map<Long, Integer> res = new HashMap<>(giftIds.size());
            List<UserBagDO> userBagDOList = bagProvider.getUserBagDoList(userId);
            if (CollectionUtil.isNotEmpty(userBagDOList)) {
                userBagDOList.forEach(i -> {
                    res.put(i.getItemId(), i.getQuantity());
                });
            }
            return res;
        } else {
            return new HashMap<>();
        }
    }

    /**
     * 通过用户礼物列表获取
     *
     * @param giftIds
     * @param userId
     * @return
     */
    @Override
    public List<UserBagDO> getUserBagByGiftIds(List<Long> giftIds, Long userId) {
        return bagProvider.getUserBagByGiftIds(giftIds, userId);
    }
}
