package com.starbuds.server.service.finance.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.domain.AlipayFundTransUniTransferModel;
import com.alipay.api.domain.Participant;
import com.alipay.api.request.AlipayFundTransUniTransferRequest;
import com.alipay.api.response.AlipayFundTransUniTransferResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.service.WxPayService;
import com.github.binarywang.wxpay.service.impl.WxPayServiceImpl;
import com.starbuds.server.common.constant.cache.RedisChatKeys;
import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.PlayTopTypeEnum;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.finance.*;
import com.starbuds.server.common.constant.media.ChatStatusEnum;
import com.starbuds.server.common.constant.media.GiftSceneTypeEnum;
import com.starbuds.server.common.constant.media.GiftSourceTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.message.push.PushActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.*;
import com.starbuds.server.common.pojo.finance.gift.SendGiftOrder;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.finance.PayResultNotifyMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveUserFansMsg;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.rtc.RtcRoomGiftHis;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.pojo.user.ChannelPayConfig;
import com.starbuds.server.common.pojo.user.UserAlipayVO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.SendGiftSnapshotMsgService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomGiftHisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.task.PushService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.constant.ApplePayConstant;
import com.starbuds.server.service.finance.dao.*;
import com.starbuds.server.service.finance.lucky.LuckyGiftProvider;
import com.starbuds.server.service.finance.pojo.AlipayAppArgs;
import com.starbuds.server.service.finance.pojo.HuaweiPayArgs;
import com.starbuds.server.service.finance.pojo.WxpayAppArgs;
import com.starbuds.server.service.finance.pojo.db.PayAppleReceiptDO;
import com.starbuds.server.service.finance.provider.*;
import com.starbuds.server.service.finance.provider.ratio.RtcRatioProvider;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import com.starbuds.server.service.finance.util.HuaweiSignUtil;
import com.starbuds.server.service.finance.util.SpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.utils.DateUtil.getTimestamp;
import static com.starbuds.server.service.finance.util.SpringContextUtil.isDev;

/**
 * 支付服务实现
 *
 * @author Lay
 * @date 2019/12/18
 */
@Service(version = "1.0.0")
@Slf4j
public class PayServiceImpl implements PayService {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    SingleChatService singleChatService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    ThirdAuthService thirdAuthService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    SendGiftSnapshotMsgService sendGiftSnapshotMsgService;

    @Resource
    PayOrderDAO payOrderDAO;

    @Resource
    PayAppleReceiptDAO payAppleReceiptDAO;

    @Resource
    CoinRechargeOrderDAO coinRechargeOrderDAO;

    @Resource
    PayDAO payDAO;

    @Resource
    PayConfigAlipayDAO payConfigAlipayDAO;

    @Resource
    PayConfigWxDAO payConfigWxDAO;

    @Resource
    PriceConfigDAO priceConfigDAO;

    @Resource
    PayConfigHuaweiDAO payConfigHuaweiDAO;

    @Resource
    VipOrderDAO vipOrderDAO;

    @Resource
    TransferHisDAO transferHisDAO;

    @Resource
    ChannelRechargeHisDAO channelRechargeHisDAO;

    @Resource
    PayProvider payProvider;

    @Resource
    ChatTimeWalletProvider chatTimeWalletProvider;

    @Resource
    LuckyGiftProvider luckyGiftProvider;

    @Resource
    IWalletProvider walletProvider;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    SettleProvider settleProvider;

    @Resource
    RechargeProvider rechargeProvider;

    @Resource
    VipProvider vipProvider;

    @Resource
    ChannelWalletProvider channelWalletProvider;

    @Resource
    SkillOrderDAO skillOrderDAO;

    @Resource
    SkillOrderProvider skillOrderProvider;

    @Resource
    VoiceChatHisDAO voiceChatHisDAO;

    @Resource
    FamilyWalletProvider familyWalletProvider;

    @Reference(version = "1.0.0")
    PushService pushService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Resource
    JpaTransactionManager transactionManager;

    @Resource
    TransactionDefinition transactionDefinition;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Value("${gift.familyUserRatio}")
    private double giftFamilyUserRatio;

    @Resource
    private CacheProvider cacheProvider;

    @Reference(version = "1.0.0")
    RtcRoomGiftHisService rtcRoomGiftHisService;

    @Resource
    private SendGiftSnapshotProvider sendGiftSnapshotProvider;

    @Resource
    private RtcRatioProvider rtcRatioProvider;

    /**
     * 判断支付订单是否存在
     *
     * @param orderId
     * @return
     */
    @Override
    public boolean isPayOrderExists(Long orderId) {
        return payOrderDAO.countById(orderId) > 0;
    }

    /**
     * 获取用户首充订单
     *
     * @param userId
     * @return
     */
    @Override
    public int getUserPayOrderCount(@Param("userId") Long userId) {

        String countStr = redisService.get(RedisUserKeys.getUserPayOrderCount(userId));
        int count = 0;
        if (StringUtils.isEmpty(countStr)) {
            count = payOrderDAO.getUserPayOrderCount(userId);
            if (count <= 0) {
                redisService.set(RedisUserKeys.getUserPayOrderCount(userId), "0", 1, TimeUnit.MINUTES);
            } else {
                redisService.set(RedisUserKeys.getUserPayOrderCount(userId), String.valueOf(count), 1, TimeUnit.MINUTES);
            }

        }
        return count;
    }


    /**
     * 获取支付参数
     *
     * @param orderId
     * @return
     */
    @Override
    public String getPayParams(Long orderId) {
        return payOrderDAO.selectPayParamsById(orderId);
    }

    /**
     * 创建支付订单
     *
     * @param orderId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String addPayOrder(Long orderId) {
        return addPayOrder(orderId, null);
    }

    /**
     * 创建支付订单
     *
     * @param orderId
     * @param openId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String addPayOrder(Long orderId, String openId) {
        // 查询支付订单
        PayOrderDO payOrderDO = payOrderDAO.selectById(orderId);

        if (payOrderDO != null) {
            // 判断支付类型
            PayTypeEnum payType = PayTypeEnum.toEnum(payOrderDO.getPayType());

            PayDO payDO = new PayDO();
            payDO.setPayId(identityService.getId());
            payDO.setOrderId(orderId);
            payDO.setPayType(payType.getValue());
            payDO.setPayStatus(PayStatusEnum.WAITTING.getValue());
            payDO.setPayCreateTime(System.currentTimeMillis());
            payDO.setPayWay(payOrderDO.getPayWay());
            payDO.setPayAmount(payOrderDO.getOrderAmount());
            payDO.setPaySubject(payOrderDO.getPaySubject());
            payDO.setPayBody(payOrderDO.getPayBody());

            try {
                // 持久化支付记录
                payDAO.insert(payDO);

                // 生成支付参数
                ChannelPayConfig payConfig = channelService.getPayConfig(payOrderDO.getChannelId());
                PayWayEnum payWay = PayWayEnum.toEnum(payDO.getPayWay());
                String payParams;
                PayConfigWxDO payConfigWxDO;
                PayConfigAlipayDO payConfigAlipayDO;
                AlipayAppArgs alipayAppArgs;
                BigDecimal payAmount = SpringContextUtil.isDev() ? new BigDecimal("0.01") : payDO.getPayAmount();
                switch (payWay) {
                    case ALI_APP:
                        // 获取支付宝配置
                        payConfigAlipayDO = payConfigAlipayDAO.selectById(payConfig.getAlipayConfigId());
                        alipayAppArgs = AlipayAppArgs.builder()
                                .appId(payConfigAlipayDO.getAppId())
                                .privateKey(payConfigAlipayDO.getPrivateKey())
                                .alipayPublicKey(payConfigAlipayDO.getAlipayPublicKey())
                                .signType(payConfigAlipayDO.getSignType())
                                .totalAmount(payAmount.setScale(2, RoundingMode.HALF_UP).toPlainString())
                                .subject(payDO.getPaySubject())
                                .body(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .configId(payConfig.getAlipayConfigId())
                                .build();
                        payParams = payProvider.getAlipayAppParams(alipayAppArgs);
                        break;
                    case ALI_PC:
                        // 获取支付宝配置
                        payConfigAlipayDO = payConfigAlipayDAO.selectById(payConfig.getAlipayConfigId());
                        alipayAppArgs = AlipayAppArgs.builder()
                                .appId(payConfigAlipayDO.getAppId())
                                .privateKey(payConfigAlipayDO.getPrivateKey())
                                .alipayPublicKey(payConfigAlipayDO.getAlipayPublicKey())
                                .signType(payConfigAlipayDO.getSignType())
                                .totalAmount(payAmount.setScale(2, RoundingMode.HALF_UP).toPlainString())
                                .subject(payDO.getPaySubject())
                                .body(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .configId(payConfig.getAlipayConfigId())
                                .build();
                        payParams = payProvider.getAlipayPcParams(alipayAppArgs);
                        break;
                    case ALI_H5:
                        payConfigAlipayDO = payConfigAlipayDAO.selectById(payConfig.getAlipayConfigId());
                        alipayAppArgs = AlipayAppArgs.builder()
                                .appId(payConfigAlipayDO.getAppId())
                                .privateKey(payConfigAlipayDO.getPrivateKey())
                                .alipayPublicKey(payConfigAlipayDO.getAlipayPublicKey())
                                .signType(payConfigAlipayDO.getSignType())
                                .totalAmount(payAmount.setScale(2, RoundingMode.HALF_UP).toPlainString())
                                .subject(payDO.getPaySubject())
                                .body(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .configId(payConfig.getAlipayConfigId())
                                .build();
                        payParams = payProvider.getAlipayWapParams(alipayAppArgs);
                        break;
                    case WX_APP:
                        // 获取微信支付配置
                        payConfigWxDO = payConfigWxDAO.selectById(payConfig.getWxPayConfigId());
                        WxpayAppArgs wxpayAppArgs = WxpayAppArgs.builder()
                                .appId(payConfigWxDO.getOpenAppId())
                                .mchId(payConfigWxDO.getOpenMchId())
                                .apiKey(payConfigWxDO.getOpenApiKey())
                                .body(payDO.getPaySubject())
                                .detail(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .totalFee(payAmount.setScale(2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).intValue())
                                .ip(payOrderDO.getCreateIp())
                                .configId(payConfig.getWxPayConfigId())
                                .build();
                        payParams = payProvider.getWxpayAppParams(wxpayAppArgs);
                        break;
                    case WX_H5:
                        // 获取微信 H5 支付配置
                        payConfigWxDO = payConfigWxDAO.selectById(payConfig.getWxPayConfigId());
                        WxpayAppArgs wxpayH5Args = WxpayAppArgs.builder()
                                .appId(payConfigWxDO.getPublicAppId())
                                .mchId(payConfigWxDO.getPublicMchId())
                                .apiKey(payConfigWxDO.getPublicApiKey())
                                .body(payDO.getPaySubject())
                                .detail(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .totalFee(payAmount.setScale(2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).intValue())
                                .ip(payOrderDO.getCreateIp())
                                .configId(payConfig.getWxPayConfigId())
                                .openId(openId)
                                .build();
                        payParams = payProvider.getWxpayH5Params(wxpayH5Args);
                        break;
                    case WX_WAP:
                        // 获取微信 H5 支付配置
                        payConfigWxDO = payConfigWxDAO.selectById(payConfig.getWxPayConfigId());
                        WxpayAppArgs wxpayWapArgs = WxpayAppArgs.builder()
                                .appId(payConfigWxDO.getPublicAppId())
                                .mchId(payConfigWxDO.getPublicMchId())
                                .apiKey(payConfigWxDO.getPublicApiKey())
                                .body(payDO.getPaySubject())
                                .detail(payDO.getPayBody())
                                .outTradeNo(String.valueOf(payDO.getPayId()))
                                .totalFee(payAmount.setScale(2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).intValue())
                                .ip(payOrderDO.getCreateIp())
                                .configId(payConfig.getWxPayConfigId())
                                .openId(openId)
                                .build();
                        payParams = payProvider.getWxPayWapParams(wxpayWapArgs);
                        break;
                    case HUAWEI:
                        String productNo = "1";
                        if (PayTypeEnum.COIN_RECHARGE.getValue() == payOrderDO.getPayType()) {
                            CoinRechargeOrderDO coinRechargeOrderDO = coinRechargeOrderDAO.selectById(orderId);
                            if (coinRechargeOrderDO != null) {
                                productNo = String.valueOf(coinRechargeOrderDO.getItemId());
                            }
                        }
                        // 获取华为支付配置
                        PayConfigHuaweiDO payConfigHuaweiDO = payConfigHuaweiDAO.selectById(payConfig.getHuaweiPayConfigId());
                        HuaweiPayArgs huaweiPayArgs = HuaweiPayArgs.builder()
                                .appId(payConfigHuaweiDO.getAppId())
                                .merchantId(payConfigHuaweiDO.getMerchantId())
                                .merchantName(payConfigHuaweiDO.getMerchantName())
                                .privateKey(payConfigHuaweiDO.getPrivateKey())
                                .productNo(productNo)
                                .productName(payDO.getPaySubject())
                                .productDesc(payDO.getPayBody())
                                .requestId(String.valueOf(payDO.getPayId()))
                                .amount(payAmount.setScale(2, RoundingMode.UP).toPlainString())
                                .configId(payConfig.getHuaweiPayConfigId())
                                .build();
                        payParams = payProvider.getHuaweiPayParams(huaweiPayArgs);
                        break;
                    default:
                        throw new Exception("未知的支付方式");
                }

                return payParams;
            } catch (Exception e) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.error("支付订单创建失败", e);
            }
        }

        return null;
    }

    /**
     * 创建 ApplePay 支付订单
     *
     * @param orderId
     * @param transactionId
     * @param receiptData
     * @param inApprove
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addApplePayOrder(Long orderId, String transactionId, String receiptData, boolean inApprove) {
        String result = PayProvider.getApplePayResult(receiptData, inApprove || SpringContextUtil.isDev());

        // 判空
        if (StringUtils.isBlank(result)) {
            return false;
        }

        JSONObject jsonObject = JSON.parseObject(result);
        String status = jsonObject.getString("status");

        // 状态判断
        if (!ApplePayConstant.STATUS_SUCCESS.equals(status)) {
            return false;
        }
        // 查询支付订单
        PayOrderDO payOrderDO = payOrderDAO.selectById(orderId);
        if (payOrderDO == null) {
            return false;
        }

        String productId;
        JSONObject receipt = jsonObject.getJSONObject("receipt");
        if (receipt.containsKey("in_app")) {
            // 新版
            productId = receipt.getJSONArray("in_app").getJSONObject(0).getString("product_id");
        } else {
            // 旧版
            productId = receipt.getString("product_id");
        }

        if (productId.contains("_")) {
            productId = productId.split("_")[1];
        }

        try {
            Long curTime = System.currentTimeMillis();

            // 创建支付记录
            PayDO payDO = new PayDO();
            payDO.setPayId(identityService.getId());
            payDO.setOrderId(orderId);
            payDO.setPayType(payOrderDO.getPayType());
            payDO.setPayStatus(PayStatusEnum.SUCCESSED.getValue());
            payDO.setPayCreateTime(curTime);
            payDO.setPayWay(payOrderDO.getPayWay());
            payDO.setPayAmount(payOrderDO.getOrderAmount());
            payDO.setPaySubject(payOrderDO.getPaySubject());
            payDO.setPayBody(payOrderDO.getPayBody());
            payDO.setPayNotifyTime(curTime);
            payDO.setTradeId(transactionId);

            payDAO.insert(payDO);

            // 更新支付订单
            if (payOrderDAO.updateAppleTransactionId(orderId, transactionId, PayStatusEnum.SUCCESSED.getValue(), receiptData) == 0) {
                throw new Exception("重复订单");
            }

            if (PayTypeEnum.COIN_RECHARGE.getValue() == payOrderDO.getPayType()) {
                Long itemId = Convert.toLong(productId);
                CoinRechargeOrderDO coinRechargeOrderDO = coinRechargeOrderDAO.selectById(orderId);
                if (coinRechargeOrderDO == null) {
                    throw new RuntimeException("订单不存在");
                }

                if (!coinRechargeOrderDO.getItemId().equals(itemId)) {
                    throw new RuntimeException(String.format("订单不匹配 %d %d", coinRechargeOrderDO.getItemId(), itemId));
                }

                // 增加金币
                walletProvider.addBalance(WalletTypeEnum.STAR_DIAMOND, payOrderDO.getUserId(), coinRechargeOrderDO.getItemCoin(), payOrderDO.getPaySubject(), orderId);

                // 邀请奖励
                rechargeProvider.processInviteRewards(payOrderDO.getChannelId(), payDO.getPayId(), payOrderDO.getUserId(), coinRechargeOrderDO.getItemCoin());

                // 首充判定
                rechargeProvider.saveHasRecharged(payOrderDO.getChannelId(), payOrderDO.getUserId(), payOrderDO.getOrderAmount(), PayWayEnum.APPLE.getValue(), payOrderDO.getOrderId());
            }

            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("ApplePay 支付订单创建失败", e);
        }
        return false;
    }

    @Override
    public boolean updateApplePayInfo(Long orderId, String transactionId, String receiptData) {
        //验证订单失败后存储receipData，便于后续发起查询
        return payAppleReceiptDAO.insertOrUpdate(orderId, transactionId, receiptData, System.currentTimeMillis()) > 0;
    }

    @Override
    public boolean checkApplePayOrder(Long orderId) {
        // 查询订单
        PayAppleReceiptDO payOrderDO = payAppleReceiptDAO.selectById(orderId);
        if (payOrderDO == null) {
            log.info("checkApplePayOrder selectPayOrder:" + orderId);
            return false;
        }

        String appleTransactionId = payOrderDO.getAppleTransactionId();
        String appleReceiptData = payOrderDO.getAppleReceiptData();

        if (StringUtils.isBlank(appleTransactionId) || StringUtils.isBlank(appleReceiptData)) {
            log.info("checkApplePayOrder appleTransactionId  or appleReceiptData");
            return false;
        }

        return addApplePayOrder(orderId, appleTransactionId, appleReceiptData, false);
    }

    /**
     * 查询订单支付状态
     *
     * @param orderId
     * @return
     */
    @Override
    public Integer getPayStatus(Long orderId) {
        return payOrderDAO.selectPayStatusById(orderId);
    }

    /**
     * 查询支付类型
     *
     * @param orderId
     * @return
     */
    @Override
    public Integer getPayType(Long orderId) {
        return payOrderDAO.selectPayTypeById(orderId);
    }

    /**
     * 获取支付方式
     *
     * @param orderId
     * @return
     */
    @Override
    public Integer getPayWay(Long orderId) {
        return payOrderDAO.selectPayWayById(orderId);
    }

    /**
     * 获取 ApplePay 流水号
     *
     * @param orderId
     * @return
     */
    @Override
    public String getAppleTransactionId(Long orderId) {
        return payOrderDAO.selectAppleTransactionIdById(orderId);
    }

    /**
     * 验证支付宝异步通知签名
     *
     * @param configId
     * @param appId
     * @param charset
     * @param signType
     * @param params
     * @return
     */
    @Override
    public boolean checkAlipayNotifySign(Long configId, String appId, String charset, String signType, Map<String, String> params) {
        // 如果没有 configId 尝试通过 appId 查找
        PayConfigAlipayDO payConfigAlipayDO = Optional.ofNullable(payConfigAlipayDAO.selectById(configId)).orElse(payConfigAlipayDAO.selectByAppId(appId));
        if (payConfigAlipayDO == null) {
            log.error("未找到支付宝配置 configId:{} appId: {}", configId, appId);
            return false;
        }
        return payProvider.checkNotifySign(payConfigAlipayDO, appId, charset, signType, params);
    }

    /**
     * 验证微信支付异步通知签名
     *
     * @param xmlData
     * @return
     */
    @Override
    public boolean checkWxpayNotifySign(String xmlData) {
        try {
            WxPayOrderNotifyResult result = WxPayOrderNotifyResult.fromXML(xmlData);
            Long configId = Convert.toLong(result.getAttach());
            PayConfigWxDO payConfigWxDO = payConfigWxDAO.selectById(configId);
            if (payConfigWxDO == null) {
                log.error("未找到微信支付配置:", configId);
                return false;
            }

            String tradeType = result.getTradeType();
            if (tradeType.equals(WxPayConstants.TradeType.APP)) {
                WxPayConfig wxPayConfig = new WxPayConfig();
                wxPayConfig.setAppId(payConfigWxDO.getOpenAppId());
                wxPayConfig.setMchId(payConfigWxDO.getOpenMchId());
                wxPayConfig.setMchKey(payConfigWxDO.getOpenApiKey());

                WxPayService wxPayService = new WxPayServiceImpl();
                wxPayService.setConfig(wxPayConfig);
                result.checkResult(wxPayService, WxPayConstants.SignType.HMAC_SHA256, true);

                return true;
            } else if (tradeType.equals(WxPayConstants.TradeType.JSAPI) || tradeType.equals(WxPayConstants.TradeType.MWEB)) {
                WxPayConfig wxPayConfig = new WxPayConfig();
                wxPayConfig.setAppId(payConfigWxDO.getPublicAppId());
                wxPayConfig.setMchId(payConfigWxDO.getPublicMchId());
                wxPayConfig.setMchKey(payConfigWxDO.getPublicApiKey());

                WxPayService wxPayService = new WxPayServiceImpl();
                wxPayService.setConfig(wxPayConfig);

                result.checkResult(wxPayService, WxPayConstants.SignType.MD5, true);
                return true;
            }
        } catch (Exception e) {
            log.error("微信支付异步通知验签失败", e);
        }
        return false;
    }

    /**
     * 验证华为支付异步通知签名
     *
     * @param configId
     * @param params
     * @return
     */
    @Override
    public boolean checkHuaweiNotifySign(Long configId, Map<String, Object> params) {
        try {
            PayConfigHuaweiDO payConfigHuaweiDO = payConfigHuaweiDAO.selectById(configId);
            if (payConfigHuaweiDO == null) {
                log.error("未找到华为支付配置:", configId);
                return false;
            }

            return HuaweiSignUtil.checkSign(params, payConfigHuaweiDO.getPublicKey());
        } catch (Exception e) {
            log.error("华为支付异步通知验签失败", e);
            return false;
        }
    }

    /**
     * 异步通知处理
     *
     * @param notify
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean handlePayNotify(PayNotify notify) {
        PayDO payDO = payDAO.selectById(notify.getPayId());

        // 判断支付记录是否存在、支付状态是否正确
        if (payDO == null || PayStatusEnum.WAITTING.getValue() != payDO.getPayStatus()) {
            log.info("订单已处理");
            return true;
        }

        Long orderId = payDO.getOrderId();

        // 判断支付金额
        BigDecimal payAmount = SpringContextUtil.isDev() ? new BigDecimal("0.01") : payDO.getPayAmount();
        int compareAmount = notify.getTotalAmount().setScale(2, RoundingMode.HALF_UP).compareTo(payAmount.setScale(2, RoundingMode.HALF_UP));
        if (compareAmount != 0) {
            log.info("订单金额错误 {} {} compare:{}", payAmount.doubleValue(), notify.getTotalAmount().doubleValue());
            return false;
        }

        long curTime = System.currentTimeMillis();

        try {
            // 修改支付单状态
            log.info("修改所有类型支付单状态");
            payDO.setPayStatus(PayStatusEnum.SUCCESSED.getValue());
            payDO.setPayNotifyTime(curTime);
            payDO.setPayBuyerId(notify.getBuyerId());
            payDO.setPayBuyerAccount(notify.getBuyerAccount());
            payDO.setPaySellerId(notify.getSellerId());
            payDO.setPaySellerAccount(notify.getSellerAccount());
            payDO.setTradeId(notify.getNotifyId());
            payDAO.update(payDO);

            // 为渠道充值时
            if (PayTypeEnum.CHANNEL_RECHARGE.getValue() == payDO.getPayType()) {
                // 查询充值订单
                ChannelRechargeHisDO channelRechargeHis = channelRechargeHisDAO.selectById(orderId);
                if (channelRechargeHis == null) {
                    throw new Exception("订单不匹配");
                }
                // 增加余额
                channelWalletProvider.addBalance(channelRechargeHis.getCoinType(), channelRechargeHis.getChannelId(), channelRechargeHis.getCoinAmount(), "渠道充值", orderId);
                // 统计
                statPlatformService.addPlatformStat(channelRechargeHis.getChannelId(), StatPlatformTypeEnum.RECHARGE_CHANNEL_AMOUNT.getValue(), channelRechargeHis.getPayAmount());

            } else {
                log.info("修改金币充值、vip订单状态");
                // 修改联合订单状态
                if (payOrderDAO.updatePayStatus(orderId, PayStatusEnum.SUCCESSED.getValue(), PayStatusEnum.WAITTING.getValue()) == 0) {
                    throw new Exception("重复处理的订单");
                }

                // 查询订单
                PayOrderDO payOrderDO = payOrderDAO.selectById(orderId);
                if (PayTypeEnum.COIN_RECHARGE.getValue() == payOrderDO.getPayType()) {
                    // 金币充值订单
                    CoinRechargeOrderDO coinRechargeOrderDO = coinRechargeOrderDAO.selectById(orderId);
                    if (coinRechargeOrderDO == null) {
                        throw new RuntimeException("订单不匹配");
                    }

                    // 增加金币
                    walletProvider.addBalance(WalletTypeEnum.STAR_DIAMOND, payOrderDO.getUserId(), coinRechargeOrderDO.getItemCoin(), "用户充值", orderId);

                    // 增加星元
                    Long starYuan = BigDecimal.valueOf(coinRechargeOrderDO.getItemCoin()).divide(BigDecimal.valueOf(100), 0, RoundingMode.DOWN).longValue();
                    if (starYuan > 0) {
                        walletProvider.addBalance(WalletTypeEnum.STAR_YUAN, payOrderDO.getUserId(), starYuan, "充值奖励", orderId);
                    }

                    // 邀请奖励
                    rechargeProvider.processInviteRewards(payOrderDO.getChannelId(), payDO.getPayId(), payOrderDO.getUserId(), coinRechargeOrderDO.getItemCoin());

                    // 首充判定
                    rechargeProvider.saveHasRecharged(payOrderDO.getChannelId(), payOrderDO.getUserId(), payOrderDO.getOrderAmount(), payOrderDO.getPayWay(), payOrderDO.getOrderId());

                    sendRechargeFinish(payOrderDO.getUserId(), payOrderDO.getOrderId(), payOrderDO.getOrderAmount());

                } else if (PayTypeEnum.VIP.getValue() == payOrderDO.getPayType()) {
                    // 会员订单
                    VipOrderDO vipOrder = vipOrderDAO.selectById(orderId);
                    if (vipOrder == null) {
                        throw new RuntimeException("订单不匹配");
                    }

                    // 会员处理
                    vipProvider.handlePay(payOrderDO.getUserId(), vipOrder);

                    // 刷新直播间资料
                    userService.refreshLiveUserProfile(vipOrder.getUserId());
                    userService.refreshChatUserProfile(vipOrder.getUserId());
                }

                PayResultNotifyMsg notifyMsg = new PayResultNotifyMsg();
                notifyMsg.setUserId(payOrderDO.getUserId());
                notifyMsg.setPayType(payOrderDO.getPayType());
                notifyMsg.setPayStatus(payOrderDO.getPayStatus());
                payProvider.notifyClientAsync(notifyMsg);
            }

            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("订单支付通知处理失败", e);
        }
        return false;
    }


    /**
     * 充值成功
     */
    private void sendRechargeFinish(Long userId, Long orderPayId, BigDecimal amount) {
        UserRechargeFinishMsg rejectMsg = new UserRechargeFinishMsg();
        rejectMsg.setUserId(userId);
        rejectMsg.setOrderPayId(orderPayId);
        rejectMsg.setAmount(amount);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.USER_RECHARGE_FINISH, rejectMsg.serialize());
    }

    @Override
    public void testSendRechargeFinish(Long userId, Long orderPayId, BigDecimal amount) {
        sendRechargeFinish(userId, orderPayId, amount);
    }

    /**
     * 模拟充值回调
     *
     * @param orderId       订单id
     * @param buyerId       买家ID
     * @param buyerAccount  买家账号
     * @param sellerId      卖家ID
     * @param sellerAccount 卖家账号
     * @param totalFee      总额
     * @param transcationId 交易ID(第三方的ID号)
     * @return
     */
    @Override
    public boolean imitateRechargeCallback(Long orderId, String buyerId, String buyerAccount, String sellerId, String sellerAccount, BigDecimal totalFee, String transcationId) {
        try {
            PayNotify payNotify = new PayNotify();
            payNotify.setPayId(orderId);
            payNotify.setBuyerId(buyerId);
            payNotify.setBuyerAccount(buyerAccount);
            payNotify.setSellerId(sellerId);
            payNotify.setSellerAccount(sellerAccount);
            payNotify.setTotalAmount(totalFee.divide(new BigDecimal(100)));
            payNotify.setNotifyId(transcationId);

            return handlePayNotify(payNotify);
        } catch (Exception e) {
            log.error("模拟订单支付回调失败：" + e);
        }

        return false;
    }


    /**
     * 微信异步通知处理
     *
     * @param xmlData
     * @return
     */
    @Override
    public boolean handleWxpayNotify(String xmlData) {
        try {
            WxPayOrderNotifyResult result = WxPayOrderNotifyResult.fromXML(xmlData);
            PayNotify payNotify = new PayNotify();
            payNotify.setPayId(Convert.toLong(result.getOutTradeNo()));
            payNotify.setBuyerId(result.getOpenid());
            payNotify.setBuyerAccount(result.getOpenid());
            payNotify.setSellerId(result.getMchId());
            payNotify.setSellerAccount(result.getMchId());
            payNotify.setTotalAmount(new BigDecimal(result.getTotalFee()).divide(new BigDecimal(100)));
            payNotify.setNotifyId(result.getTransactionId());

            return handlePayNotify(payNotify);
        } catch (Exception e) {
            log.error("微信异步通知处理失败", e);
        }
        return false;
    }

    /**
     * 获取价格配置
     *
     * @return
     */
    @Override
    public PriceConfig getEnabledPriceConfig() {
        PriceConfigDO priceConfigDO = priceConfigDAO.selectLastEnabled();
        if (priceConfigDO != null) {
            PriceConfig priceConfig = new PriceConfig();
            BeanUtils.copyProperties(priceConfigDO, priceConfig);
            return priceConfig;
        }
        return null;
    }

    /**
     * 支付礼物
     *
     * @param receiverUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Override
    public Long payForGift(Long receiverUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId) {
        return payForGift(receiverUserId, senderUserId, giftId, quantity, channelId, GiftSceneTypeEnum.Live, null);
    }

    /**
     * 支付礼物
     *
     * @param receiverUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param giftSceneType
     * @param channelName
     * @return
     */
    @Override
    public Long payForGift(Long receiverUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId, GiftSceneTypeEnum giftSceneType, String channelName) {
        Long orderId = identityService.getId();

        // 读取主播信息
        UserCardVO receiver = userService.getCard(receiverUserId);

        // 礼物信息
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);

        if (receiver != null && giftPayItem != null) {
            // 计算价格
            Long amount = giftPayItem.getGiftPrice() * quantity;
            Long anchorAmount = Long.valueOf(amount);

            LuckyGiftLotteryConfig lotteryConfig = luckyGiftProvider.getConfig();
            if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                if (lotteryConfig == null) {
                    log.error("幸运礼物未配置");
                    return null;
                }

                anchorAmount = BigDecimal.valueOf(amount).multiply(new BigDecimal(String.valueOf(lotteryConfig.getAnchorRatio()))).setScale(0, RoundingMode.DOWN).longValue();
            }

            // 扣费
            Long balance = walletProvider.reduceBalance(giftPayItem.getGiftPriceType(), senderUserId, -amount, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverUserId, quantity, giftPayItem.getGiftName()), orderId);

            if (balance == null || balance < 0) {
                return balance;
            }

            // 异步主播结算 抽奖礼物主播是没钱的
            if (!giftPayItem.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
                settleProvider.settleByGift(giftPayItem, receiverUserId, senderUserId, quantity, anchorAmount, orderId);
                if (GiftSceneTypeEnum.Live == giftSceneType) {
                    statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.STAR_COIN_ADD_LIVE.getValue(), BigDecimal.valueOf(anchorAmount));
                } else if (GiftSceneTypeEnum.SingleChat == giftSceneType) {
                    statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.STAR_COIN_ADD_SINGLE_CHAT.getValue(), BigDecimal.valueOf(anchorAmount));
                } else if (GiftSceneTypeEnum.VoiceChat == giftSceneType) {
                    statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.STAR_COIN_ADD_VOICE_CHAT.getValue(), BigDecimal.valueOf(anchorAmount));
                }
            }

            QueueGiftPaidMsg queueGiftPaidMsg = new QueueGiftPaidMsg();
            queueGiftPaidMsg.setOrderId(orderId);
            queueGiftPaidMsg.setGiftId(giftId);
            queueGiftPaidMsg.setGiftType(giftPayItem.getGiftType());
            queueGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
            queueGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
            queueGiftPaidMsg.setQuantity(quantity);
            queueGiftPaidMsg.setAnchorAmount(anchorAmount);
            queueGiftPaidMsg.setSenderUserId(senderUserId);
            queueGiftPaidMsg.setReceiverUserId(receiverUserId);
            queueGiftPaidMsg.setFromBag(false);
            queueGiftPaidMsg.setChannelId(channelId);
            queueGiftPaidMsg.setGiftSceneType(giftSceneType.getValue());
            queueGiftPaidMsg.setChannelName(channelName);
            queueGiftPaidMsg.setSourceType(GiftSourceTypeEnum.PAY.getValue());

            // 幸运礼物异步抽奖
            if (WalletTypeEnum.STAR_DIAMOND.getValue() == giftPayItem.getGiftPriceType() && GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                luckyGiftProvider.asyncLottery(queueGiftPaidMsg);
            } else {
                // 丢出礼物赠送消息到消息队列
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.GIFT_PAID, queueGiftPaidMsg.serialize());
            }

            return balance;
        }
        return null;
    }

    /**
     * 支付粉丝团费用
     *
     * @param anchorUserId
     * @param payUserId
     * @return
     */
    @Override
    public Long payForFans(Long anchorUserId, Long payUserId) {
        // 读取主播信息
        UserCardVO anchor = userService.getCard(anchorUserId);

        // 读取用户信息
        UserCardVO user = userService.getCard(payUserId);

        if (anchor != null && user != null) {
            Long orderId = identityService.getId();

            // 粉丝团价格
            PriceConfig priceConfig = settleProvider.getPriceConfig();

            // 扣费
            String userRemarks = String.format("加入[%s](%d)的粉丝团", anchor.getUserName(), anchorUserId);
            Long balance = walletProvider.reduceBalance(WalletTypeEnum.STAR_DIAMOND, payUserId, priceConfig.getFansTeamPrice(), userRemarks, orderId);

            // 增加财富值
            scoreService.addScore(payUserId, ScoreTypeEnum.WEALTH.getValue(), priceConfig.getFansTeamWealth(), userRemarks, orderId, anchorUserId);

            if (balance != null && balance > 0) {
                // 给主播结算
                String anchorRemarks = String.format("[%s](%d)加入粉丝团", user.getUserName(), payUserId);
                walletProvider.addBalance(WalletTypeEnum.STAR_COIN, anchorUserId, priceConfig.getFansTeamPrice(), anchorRemarks, orderId);

                // 直播增加星币收入
                Long coinIncome = cacheService.hashIncreaseLong(RedisLiveKeys.LIVE_ANCHOR_COIN, String.valueOf(anchorUserId), priceConfig.getFansTeamPrice());

                // 增加魅力值
                scoreService.addScore(anchorUserId, ScoreTypeEnum.CHARM.getValue(), priceConfig.getFansTeamCharm(), anchorRemarks, orderId, anchorUserId);

                // 给主播发送结算通知
                settleProvider.sendAnchorIncomeChangeMsg(WalletTypeEnum.STAR_COIN, coinIncome, anchorUserId);

                // 给直播间发送加入粉丝团消息
                LiveUserFansMsg liveUserFansMsg = new LiveUserFansMsg();
                imService.sendLiveRoomMsg(ImTargetType.GROUP, payUserId, anchorUserId, liveUserFansMsg);
            }

            return balance;
        }

        return null;
    }

    /**
     * 启动单聊支付
     *
     * @param hisId
     * @param times
     * @param timestamp
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long payForSingleChat(String hisId, int times, long timestamp) {
        try {
            SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
            if (singleChatHis == null || ChatStatusEnum.Closed.getValue() == singleChatHis.getHisStatus()) {
                return null;
            }

            if (times == 1) {
                if (!cacheService.hashPutIfAbsent(RedisFinanceKeys.SINGLE_CHAT_CHARGING, hisId, "0")) {
                    return 0L;
                }
            }

            Integer curTimes = Convert.toInt(cacheService.hashGet(RedisFinanceKeys.SINGLE_CHAT_CHARGING, hisId));
            if (curTimes == null) {
                return null;
            }

            if (curTimes >= times) {
                // 该次已计费
                return 0L;
            }

            // 扣费
            UserCardVO userCard = userService.getCard(singleChatHis.getTargetUserId());
            String remarks = String.format("与[%s](%d)视频聊天1分钟结算", userCard.getUserName(), userCard.getUserId());

            // 优先扣除时间
            Integer minutes = chatTimeWalletProvider.reduceMinutes(singleChatHis.getUserId(), ChatTimeWalletType.SINGLE_CHAT, 1, remarks, hisId);
            Long balance = walletProvider.getBalance(singleChatHis.getUserId(), singleChatHis.getHisPriceType());
            if (minutes >= 0) {
                // 增加计费次数
                cacheService.hashIncreaseLong(RedisFinanceKeys.SINGLE_CHAT_CHARGING, hisId, 1);
                cacheService.hashIncreaseLong(RedisFinanceKeys.SINGLE_CHAT_CHARGING_TIME, hisId, 1);
            } else {
                balance = walletProvider.reduceBalance(singleChatHis.getHisPriceType(), singleChatHis.getUserId(), singleChatHis.getHisPrice(), remarks, singleChatHis.getTargetUserId());
                if (balance < 0) {
                    if (times == 1) {
                        cacheService.hashDel(RedisFinanceKeys.SINGLE_CHAT_CHARGING, hisId);
                    }
                    return balance;
                }

                // 异步主播结算
                settleProvider.settleBySingleChat(singleChatHis);

                // 增加计费次数
                cacheService.hashIncreaseLong(RedisFinanceKeys.SINGLE_CHAT_CHARGING, hisId, 1);

                statPlatformService.addPlatformStat(singleChatHis.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_AMOUNT, BigDecimal.valueOf(singleChatHis.getHisPrice()));
            }

            // 统计
            statPlatformService.addPlatformStat(singleChatHis.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_MINUTES, BigDecimal.ONE);

            return balance;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("单聊支付失败", e);
        }
        return null;
    }

    /**
     * 转账给用户
     *
     * @param type
     * @param userId
     * @param amount
     * @param bizId
     * @param title
     * @return
     */
    @Override
    public String transferToUser(int type, Long userId, BigDecimal amount, String bizId, String title) {
        if (TransferType.ALIPAY == type) {
            // 获取该用户的绑定支付宝信息
            UserAlipayVO userAlipay = thirdAuthService.getAlipay(userId);
            if (userAlipay == null) {
                return "该用户未绑定支付宝";
            }

            UserAuthDO userAuth = userService.getUserAuth(userId);
            if (userAuth == null) {
                return "该用户未实名认证";
            }

            Long hisId = identityService.getId();

            if (SpringContextUtil.isDev()) {
                // 测试环境 1 毛钱
                amount = new BigDecimal("0.1");
            }

            // 发起请求
            AlipayFundTransUniTransferModel model = new AlipayFundTransUniTransferModel();
            model.setOutBizNo(String.valueOf(hisId));
            model.setTransAmount(amount.setScale(2, RoundingMode.DOWN).toPlainString());
            model.setProductCode("TRANS_ACCOUNT_NO_PWD");
            model.setBizScene("DIRECT_TRANSFER");
            model.setOrderTitle(title);

            Participant payeeInfo = new Participant();
            payeeInfo.setIdentity(userAlipay.getAlipayUserId());
            payeeInfo.setIdentityType("ALIPAY_USER_ID");
            payeeInfo.setName(userAuth.getRealName());
            model.setPayeeInfo(payeeInfo);

            model.setRemark(String.format("关联业务编号：%s", bizId));

            AlipayFundTransUniTransferRequest request = new AlipayFundTransUniTransferRequest();
            request.setBizModel(model);

            try {
                // 初始化 SDK
                AlipayFundTransUniTransferResponse response = payProvider.getDefaultClient().certificateExecute(request);

                try {
                    // 创建请求记录
                    TransferHisDO transferHisDO = new TransferHisDO();
                    transferHisDO.setHisId(hisId);
                    transferHisDO.setUserId(userId);
                    transferHisDO.setHisType(type);
                    transferHisDO.setHisAmount(amount);
                    transferHisDO.setBizId(bizId);
                    transferHisDO.setUserAccount(userAlipay.getAlipayUserId());
                    transferHisDO.setHisCreateTime(System.currentTimeMillis());
                    transferHisDO.setPayOrderId(response.getOrderId());
                    transferHisDO.setPayFundOrderId(response.getPayFundOrderId());
                    transferHisDO.setPayStatus(response.getStatus());
                    transferHisDO.setPayCode(response.getCode());
                    transferHisDO.setPayMsg(response.getMsg());
                    transferHisDO.setPaySubCode(response.getSubCode());
                    transferHisDO.setPaySubMsg(response.getSubMsg());
                    transferHisDAO.insert(transferHisDO);
                } catch (Exception e) {
                    log.error("转账记录写入失败", e);
                }

                if ("SUCCESS".equalsIgnoreCase(response.getStatus())) {
                    return null;
                }
                return response.getSubMsg();

            } catch (AlipayApiException e) {
                log.error("转账用户失败", e);
                return e.getErrMsg();
            }
        }
        return "不支持的第三方类型";
    }

    /**
     * 支付 RTC 礼物
     *
     * @param roomId
     * @param targetUserSeats
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Override
    public SendGiftOrder payForRtcGift(Long roomId, Map<Long, Integer> targetUserSeats, Long senderUserId, Long giftId, Integer quantity, Long channelId) {
        Long orderId = identityService.getId();
        boolean allSeat = targetUserSeats.size() > 7 && targetUserSeats.size() == rtcSeatService.getSeatUserCount(roomId, false, senderUserId);

        // 礼物信息 目前仅支持普通星钻礼物
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);
        if (giftPayItem == null || WalletTypeEnum.STAR_DIAMOND.getValue() != giftPayItem.getGiftPriceType()) {
            return null;
        }

        GiftTypeEnum giftType = GiftTypeEnum.toEnum(giftPayItem.getGiftType());
        if (GiftTypeEnum.COMMON != giftType && GiftTypeEnum.LOTTERY != giftType) {
            return null;
        }

        boolean isLotteryGift = GiftTypeEnum.LOTTERY.getValue() == giftPayItem.getGiftType();

        if (CollectionUtil.isNotEmpty(targetUserSeats)) {
            // 单个用户价格
            Long singleAmount = giftPayItem.getGiftPrice() * quantity;

            // 获取用户信息
            Set<Long> targetUserIds = targetUserSeats.keySet();
            Map<Long, UserCardVO> userCardMap = userService.getNameAndAvatarByIds(new ArrayList<>(targetUserIds)).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
            UserCardVO sender = userService.getCard(senderUserId);
            // 用户家族map
            Map<Long, Long> userFamilyMap = familyService.getFamilyIds(targetUserIds);
            log.info("userFamilyMap={} targetUserIds={}", userFamilyMap, targetUserIds);

            RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(roomId);
            // 尝试扣费
            Long balance = null;
            int roomFamilyUserCount = 0;
            TransactionStatus transactionStatus = transactionManager.getTransaction(transactionDefinition);
            try {
                // 房间家族
//                Long roomFamilyId = rtcRoomService.getFamilyId(roomId);
                Long roomFamilyId = rtcRatioProvider.getFamilyId(rtcRoomRatio);
                List<Long> receiverUserIds = new ArrayList<>(targetUserSeats.size());
                List<String> receiverUserNames = new ArrayList<>(targetUserSeats.size());
                Long totalPay = 0L;
                for (Long receiverUserId : targetUserIds) {
                    UserCardVO receiver = userCardMap.get(receiverUserId);
                    balance = walletProvider.reduceBalance(giftPayItem.getGiftPriceType(), senderUserId, -singleAmount,
                            String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverUserId, quantity, giftPayItem.getGiftName()), orderId);
                    if (balance == null || balance < 0) {
                        throw new RuntimeException("余额不足");
                    }
                    receiverUserIds.add(receiverUserId);
                    receiverUserNames.add(receiver.getUserName());
                    totalPay += singleAmount;

                    // 用户结算
                    if (!isLotteryGift) {
                        /*BigDecimal ratio = null;
                        // 用户家族
                        Long familyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                        // 是否设置分成比例
                        Double userRatio = rtcRoomService.getUserRatio(roomId);
                        if (userRatio != null) {
                            ratio = Convert.toBigDecimal(userRatio);
                        } else {
                            ratio = Convert.toBigDecimal(giftNoFamilyUserRatio);
                            // 用户家族
                            if (roomFamilyId != null && roomFamilyId.equals(familyId)) {
                                ratio = Convert.toBigDecimal(giftFamilyUserRatio);
                                roomFamilyUserCount++;
                            }
                        }*/
                        // 用户家族
                        Long userFamilyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                        if (roomFamilyId != null && roomFamilyId.equals(userFamilyId)) {
                            roomFamilyUserCount++;
                        }
                        // 用户实际分成比例
                        BigDecimal userRatio = rtcRatioProvider.getUserRatio(userFamilyId, rtcRoomRatio);
                        rtcRatioProvider.checkRatio(userRatio);
                        log.info("roomFamilyId {} familyId {} ratio {} receiverUserId {} roomFamilyUserCount{}",
                                roomFamilyId, userFamilyId, userRatio, receiverUserId, roomFamilyUserCount);
                        // 接收者
                        Long dustAmount = BigDecimal.valueOf(singleAmount).multiply(userRatio).setScale(0, RoundingMode.DOWN).longValue();
                        String remark = String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName());
                        walletProvider.addBalance(WalletTypeEnum.STAR_DUST, receiverUserId, dustAmount, remark, orderId);

                        //增加礼物记录
                        RtcRoomGiftHis rmh = new RtcRoomGiftHis();
                        rmh.setId(identityService.getId());
                        rmh.setUserId(senderUserId);
                        rmh.setTargetUserId(receiverUserId);
                        rmh.setGiftId(giftPayItem.getGiftId());
                        rmh.setGiftType(StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue());
                        rmh.setTargetUserfamilyId(userFamilyId == null ? 0L : userFamilyId);

                        Long roomfid = roomFamilyId == null ? 0L : roomFamilyId;
                        log.info("roomFamilyId-------{}", roomfid);
                        rmh.setRoomFamilyId(roomfid);
                        rmh.setRoomId(roomId);
                        rmh.setOrderId(orderId);
                        rmh.setPrice(dustAmount);
                        rmh.setRemark(remark);
                        setInsertTime(rmh);

                        rtcRoomGiftHisService.insertRtcRoomGiftHis(rmh);


                        // 是否家族房间
                        if (roomFamilyId != null) {
//                            BigDecimal giftFamilyRatio = cacheProvider.familyRatio(roomFamilyId);
                            BigDecimal giftFamilyRatio = rtcRatioProvider.getFamilyRatio(rtcRoomRatio);
                            rtcRatioProvider.checkRatio(userRatio, giftFamilyRatio);
                            BigDecimal presidentAmount = BigDecimal.valueOf(singleAmount)
                                    .multiply(giftFamilyRatio)
                                    .setScale(2, RoundingMode.DOWN);
                            // 返到家族钱包
                            familyWalletProvider.addBalance(roomFamilyId, presidentAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName()), orderId);
                            log.info(LogUtil.devLog("支付语聊房礼物-familyIncome-{} Gift{id={} price={} quantity={}} {} * {}=[{}]"),
                                    roomFamilyId, giftPayItem.getGiftId(), giftPayItem.getGiftPrice(), quantity, singleAmount, giftFamilyRatio, presidentAmount);
                        }
                    }
                }

                SendGiftSnapshotDO sendGiftSnapshot = new SendGiftSnapshotDO();
                sendGiftSnapshot.setOrderId(orderId);
                sendGiftSnapshot.setSenderUserId(senderUserId);
                sendGiftSnapshot.setSenderUserName(sender.getUserName());
                sendGiftSnapshot.setReceiverUserIds(StringUtils.join(receiverUserIds, ","));
                sendGiftSnapshot.setReceiverUserNames(StringUtils.join(receiverUserNames, ","));
                sendGiftSnapshot.setGiftId(giftId);
                sendGiftSnapshot.setGiftName(giftPayItem.getGiftName());
                sendGiftSnapshot.setGiftType(giftPayItem.getGiftType());
                sendGiftSnapshot.setGiftPrice(giftPayItem.getGiftPrice());
                sendGiftSnapshot.setGiftPriceType(giftPayItem.getGiftPriceType());
                sendGiftSnapshot.setQuantity(quantity);
                sendGiftSnapshot.setIsFromBag(BoolIntVal.False);
                sendGiftSnapshot.setTotalPay(totalPay);
                sendGiftSnapshot.setRoomId(roomId);
                sendGiftSnapshot.setRoomFamilyId(roomFamilyId);
                if (rtcRoomRatio != null && rtcRoomRatio.getBusinessType() != null) {
                    sendGiftSnapshot.setRoomBusinessType(rtcRoomRatio.getBusinessType().getValue());
                }
                SendGiftSnapshotDO snapshotDO = sendGiftSnapshotProvider.create(sendGiftSnapshot);
                sendGiftSnapshotMsgService.sendMQMsg(snapshotDO);

                transactionManager.commit(transactionStatus);
            } catch (Exception e) {
                if (!"余额不足".equals(e.getMessage())) {
                    log.error("礼物赠送失败", e);
                }
                transactionManager.rollback(transactionStatus);
                log.info("jsonrtcroom11111:");
                return null;
            }

            QueueRtcGiftPaidMsg rtcGiftPaidMsg = new QueueRtcGiftPaidMsg();
            rtcGiftPaidMsg.setOrderId(orderId);
            rtcGiftPaidMsg.setGiftId(giftId);
            rtcGiftPaidMsg.setGiftType(giftPayItem.getGiftType());
            rtcGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
            rtcGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
            rtcGiftPaidMsg.setQuantity(quantity);
            rtcGiftPaidMsg.setSenderUserId(senderUserId);
            rtcGiftPaidMsg.setTargetUserSeat(targetUserSeats);
            rtcGiftPaidMsg.setFromBag(false);
            rtcGiftPaidMsg.setChannelId(channelId);
            rtcGiftPaidMsg.setRoomId(roomId);
            rtcGiftPaidMsg.setAllSeat(BoolIntVal.fromBool(allSeat));
            rtcGiftPaidMsg.setRoomFamilyUserCount(roomFamilyUserCount);
            rtcGiftPaidMsg.setRoomBusinessType(rtcRoomRatio.getBusinessType());

            // 丢出礼物赠送消息到消息队列
            boolean send = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_GIFT_PAID, rtcGiftPaidMsg.serialize());
            if (Boolean.FALSE.equals(send)) {
                String errMsg = "丢出礼物赠送消息到消息队列失败 msg=";
                if (isDev()) {
                    log.error(LogUtil.devLog(errMsg + JSON.toJSONString(rtcGiftPaidMsg, true)));
                } else {
                    log.error(errMsg + rtcGiftPaidMsg);
                }
            }

            SendGiftOrder order = new SendGiftOrder();
            order.setOrderId(orderId);
            order.setBalance(balance);
            order.setWalletType(WalletTypeEnum.toEnum(giftPayItem.getGiftPriceType()));
            return order;
        }

        return null;
    }

    /**
     * 支付 语音 礼物
     *
     * @param hisId
     * @param receiveUserId
     * @param senderUserId
     * @param giftId
     * @param quantity
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long payForVoiceGift(Long hisId, Long receiveUserId, Long senderUserId, Long giftId, Integer quantity, Long channelId) {

        Long orderId = identityService.getId();

        // 礼物信息 目前仅支持普通星钻礼物
        GiftPayItem giftPayItem = giftService.getPayItem(giftId);
        if (giftPayItem == null || WalletTypeEnum.STAR_DIAMOND.getValue() != giftPayItem.getGiftPriceType()) {
            return null;
        }

        GiftTypeEnum giftType = GiftTypeEnum.toEnum(giftPayItem.getGiftType());
        if (GiftTypeEnum.COMMON != giftType && GiftTypeEnum.LOTTERY != giftType) {
            return null;
        }

        boolean isLotteryGift = GiftTypeEnum.LOTTERY.getValue() == giftPayItem.getGiftType();
        Long totalAmount = giftPayItem.getGiftPrice() * quantity;

        // 获取用户信息
        UserCardVO sender = userService.getCard(senderUserId);
        UserCardVO receiver = userService.getCard(receiveUserId);

        // 尝试扣费
        Long balance = null;
        try {
            balance = walletProvider.reduceBalance(giftPayItem.getGiftPriceType(), senderUserId, -totalAmount, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiveUserId, quantity, giftPayItem.getGiftName()), orderId);
            if (balance == null || balance < 0) {
                throw new RuntimeException("余额不足");
            }

            // 用户结算
            if (!isLotteryGift) {

                BigDecimal ratio = Convert.toBigDecimal(giftFamilyUserRatio);
                // 用户结算
                if (familyService.getFamilyId(receiveUserId) == null) {
                    ratio = Convert.toBigDecimal(giftNoFamilyUserRatio);
                }

                Long dustAmount = BigDecimal.valueOf(totalAmount).multiply(ratio).setScale(0, RoundingMode.DOWN).longValue();
                walletProvider.addBalance(WalletTypeEnum.STAR_DUST, receiveUserId, dustAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftPayItem.getGiftName()), orderId);
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return balance;
        }

        QueueVoiceGiftPaidMsg voiceGiftPaidMsg = new QueueVoiceGiftPaidMsg();
        voiceGiftPaidMsg.setOrderId(orderId);
        voiceGiftPaidMsg.setHisId(hisId);
        voiceGiftPaidMsg.setGiftId(giftId);
        voiceGiftPaidMsg.setGiftType(giftPayItem.getGiftType());
        voiceGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
        voiceGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
        voiceGiftPaidMsg.setQuantity(quantity);
        voiceGiftPaidMsg.setSenderUserId(senderUserId);
        voiceGiftPaidMsg.setReceiveUserId(receiveUserId);
        voiceGiftPaidMsg.setFromBag(false);
        voiceGiftPaidMsg.setChannelId(channelId);

        // 丢出礼物赠送消息到消息队列
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.VOICE_GIFT_PAID, voiceGiftPaidMsg.serialize());

        return balance;

    }

    /**
     * 创建渠道支付订单
     *
     * @param orderId
     * @param channelId
     * @param payAmount
     * @param payWay
     * @return
     */
    @Override
    public ChannelPayOrder addChannelPayOrder(Long orderId, Long channelId, BigDecimal payAmount, int payWay) {
        Long payId = identityService.getId();
        PayDO payDO = new PayDO();
        payDO.setPayId(payId);
        payDO.setOrderId(orderId);
        payDO.setPayType(PayTypeEnum.CHANNEL_RECHARGE.getValue());
        payDO.setPayStatus(PayStatusEnum.WAITTING.getValue());
        payDO.setPayCreateTime(System.currentTimeMillis());
        payDO.setPayWay(payWay);
        payDO.setPayAmount(payAmount);
        payDO.setPaySubject("渠道充值");
        payDO.setPayBody("渠道充值");

        try {
            // 持久化支付记录
            payDAO.insert(payDO);

            // 生成支付参数
            ChannelPayConfig payConfig = channelService.getPayConfig(0L);
            PayWayEnum payWayEnum = PayWayEnum.toEnum(payDO.getPayWay());
            payAmount = SpringContextUtil.isDev() ? new BigDecimal("0.01") : payDO.getPayAmount();
            ChannelPayOrder channelPayOrder = new ChannelPayOrder();
            channelPayOrder.setPayId(payId);
            switch (payWayEnum) {
                case ALI_QR:
                    // 获取支付宝配置
                    PayConfigAlipayDO payConfigAlipayDO = payConfigAlipayDAO.selectById(payConfig.getAlipayConfigId());
                    AlipayAppArgs alipayAppArgs = AlipayAppArgs.builder()
                            .appId(payConfigAlipayDO.getAppId())
                            .privateKey(payConfigAlipayDO.getPrivateKey())
                            .alipayPublicKey(payConfigAlipayDO.getAlipayPublicKey())
                            .signType(payConfigAlipayDO.getSignType())
                            .totalAmount(payAmount.setScale(2, RoundingMode.HALF_UP).toPlainString())
                            .subject(payDO.getPaySubject())
                            .body(payDO.getPayBody())
                            .outTradeNo(String.valueOf(payDO.getPayId()))
                            .configId(payConfig.getAlipayConfigId())
                            .channelId(channelId)
                            .build();
                    channelPayOrder.setQrCode(payProvider.getAlipayQRParams(alipayAppArgs));
                    break;
                default:
                    throw new Exception("未知的支付方式");
            }

            return channelPayOrder;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("支付订单创建失败", e);
        }
        return null;
    }

    /**
     * 创建技能订单
     *
     * @param orderDO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addSkillOrder(SkillOrderDO orderDO) {
        if (skillOrderDAO.insert(orderDO) > 0) {

            // 添加陪玩头条（陪玩 改 开黑）
            PlayTopDO topDO = new PlayTopDO();
            topDO.setTopId(identityService.getId());
            topDO.setTopType(PlayTopTypeEnum.PLAY_ORDERS.getCode());
            topDO.setUserId(orderDO.getUserId());
            topDO.setSkillId(orderDO.getSkillId());
            topDO.setOrderId(orderDO.getOrderId());
            topDO.setCreateTime(System.currentTimeMillis());
            dataService.addPlayTop(topDO);

            // 订单创建
            QueueSkillOrderCreateMsg skillOrderCreateMsg = new QueueSkillOrderCreateMsg();
            skillOrderCreateMsg.setOrderId(orderDO.getOrderId());
            skillOrderCreateMsg.setUserId(orderDO.getUserId());
            skillOrderCreateMsg.setReceiveUserId(orderDO.getReceiveUserId());
            skillOrderCreateMsg.setRequireTime(orderDO.getRequireTime());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SKILL_ORDER_CREATE, skillOrderCreateMsg.serialize());

            // 超时检测
            QueueSkillOrderPayTimeoutMsg skillOrderPayTimeoutMsg = new QueueSkillOrderPayTimeoutMsg();
            skillOrderPayTimeoutMsg.setOrderId(orderDO.getOrderId());

            // 十分钟
            Long deliverTime = orderDO.getCreateTime() + 10 * 60 * 1000L;
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SKILL_ORDER_PAY_TIMEOUT, skillOrderPayTimeoutMsg.serialize(), deliverTime);
            return true;
        }
        return false;
    }

    /**
     * 获取技能订单
     *
     * @param orderId
     * @return
     */
    @Override
    public SkillOrderDO getSkillOrder(Long orderId) {
        return skillOrderDAO.selectById(orderId);
    }

    /**
     * 支付技能
     *
     * @param orderId
     * @param channelId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean payForSkill(Long orderId, Long channelId) {
        SkillOrderDO orderDO = skillOrderDAO.selectById(orderId);
        if (orderDO == null) {
            return false;
        }
        Long userId = orderDO.getUserId();
        Long receiveUserId = orderDO.getReceiveUserId();
        Long receiveFamilyId = orderDO.getReceiveFamilyId();
        Long payAmount = orderDO.getPayAmount();
        Long requireTime = orderDO.getRequireTime();
        Integer quantity = orderDO.getQuantity();
        Long skillId = orderDO.getSkillId();

        if (payAmount == null || payAmount < 0) {
            log.warn("金额非法");
            return false;
        }

        long curTime = System.currentTimeMillis();
        String key = RedisChatKeys.getSkillOrderPayKey(orderId);
        String value = UUID.randomUUID().toString();
        try {
            boolean lock = redisService.tryLock(key, value, 10, TimeUnit.SECONDS);
            if (lock) {
                String hikRemarks = String.format("[%s]下单", skillService.getSkillName(skillId));
                Long balance = walletProvider.reduceBalance(WalletTypeEnum.STAR_DIAMOND, userId, -payAmount, hikRemarks, orderId);

                if (balance == null || balance < 0) {
                    log.info("余额不足");
                    return false;
                }

                // 订单支付
                if (!skillOrderProvider.doPay(orderId, curTime)) {
                    log.info("支付失败");
                    return false;
                }

                // 支付成功mq消息
                QueueSkillOrderPaidMsg skillPaidMsg = new QueueSkillOrderPaidMsg();
                skillPaidMsg.setOrderId(orderId);
                skillPaidMsg.setSkillId(skillId);
                skillPaidMsg.setPayAmount(payAmount);
                skillPaidMsg.setQuantity(quantity);
                skillPaidMsg.setUserId(userId);
                skillPaidMsg.setReceiveFamilyId(receiveFamilyId);
                skillPaidMsg.setReceiveUserId(receiveUserId);
                skillPaidMsg.setChannelId(channelId);
                skillPaidMsg.setRequireTime(requireTime);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SKILL_ORDER_PAID, skillPaidMsg.serialize());
                return true;
            }
        } catch (Exception e) {
            log.error("出现错误", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        } finally {
            redisService.releaseLock(key, value);
        }
        return false;
    }

    /**
     * 获取技能服务订单详情
     *
     * @param orderId
     * @return
     */
    @Override
    public SkillOrderVO getOrderDetail(Long orderId, Long curUserId) {
        SkillOrderVO orderVO = skillOrderDAO.selectDetail(orderId);
        if (orderVO != null) {
            orderVO.setUserFollowStatus(socialRelationService.isFollower(curUserId, orderVO.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
            orderVO.setReceiveUserFollowStatus(socialRelationService.isFollower(curUserId, orderVO.getReceiveUserId()) ? BoolIntVal.True : BoolIntVal.False);
        }
        return orderVO;
    }

    /**
     * 获取接单记录
     *
     * @param pageIndex
     * @param pageSize
     * @param orderStatus
     * @param receiveUserId
     * @return
     */
    @Override
    public PageList<SkillOrderVO> getOrderReceiveList(int pageIndex, int pageSize, Integer orderStatus, Long receiveUserId) {
        PageList<SkillOrderVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(skillOrderDAO.selectOrderList(page.getPageCursor(), pageSize, orderStatus, PayStatusEnum.SUCCESSED.getValue(), receiveUserId, null));
        page.setTotal(skillOrderDAO.countOrderList(orderStatus, PayStatusEnum.SUCCESSED.getValue(), receiveUserId, null));
        return page;
    }

    /**
     * 获取点单记录
     *
     * @param pageIndex
     * @param pageSize
     * @param orderStatus
     * @param payStatus
     * @param userId
     * @return
     */
    @Override
    public PageList<SkillOrderVO> getOrderPayList(int pageIndex, int pageSize, Integer orderStatus, Integer payStatus, Long userId) {
        PageList<SkillOrderVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(skillOrderDAO.selectOrderList(page.getPageCursor(), pageSize, orderStatus, payStatus, null, userId));
        page.setTotal(skillOrderDAO.countOrderList(orderStatus, payStatus, null, userId));
        return page;
    }

    /**
     * 是否等待接单
     *
     * @param orderId
     * @return
     */
    @Override
    public boolean isWaitReceive(Long orderId) {
        return skillOrderProvider.isWaitReceive(orderId);
    }

    /**
     * 接收订单
     *
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean receiveOrder(Long orderId) {
        SkillOrderDO orderDO = skillOrderDAO.selectById(orderId);
        if (orderDO == null) {
            return false;
        }
        if (skillOrderProvider.doReceiveOrder(orderId)) {

            Long userId = orderDO.getUserId();
            Long receiveUserId = orderDO.getReceiveUserId();
            Long requireTime = orderDO.getRequireTime();
            Integer quantity = orderDO.getQuantity();
            Long payAmount = orderDO.getPayAmount();
            long curTime = System.currentTimeMillis();

            // 系统消息
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setTitle("订单已接单");
            String body = String.format("你有一条【%s*%d】的订单已被接取，将按订单计划于%s开始为你服务，请耐心等待！", skillService.getSkillName(orderDO.getSkillId()), orderDO.getQuantity(), DateUtil.format(new Date(orderDO.getRequireTime()), "yyyy-MM-dd HH:mm"));
            systemMsg.setBody(body);
            systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
            systemMsg.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
            systemMsg.setModuleId(Convert.toStr(orderId));
            imService.sendCustomImMsg(null, orderDO.getUserId(), systemMsg, null);

            // 推送通知
            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(identityService.getId());
            taskDO.setTaskName("系统消息-技能服务");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(curTime);
            taskDO.setTaskType(TaskTypeEnum.SINGLE_PUSH.getValue());
            taskDO.setTargetValue(Convert.toStr(userId));

            taskDO.setAdminId(0L);
            taskDO.setRemark("系统自动推送");

            String title = String.format("%s 已接单", userService.getUserName(receiveUserId));
            taskDO.setPushTitle(title);
            taskDO.setPushBody("戳>>>");

            PushMsgParam pushParam = new PushMsgParam();
            pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
            pushParam.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
            pushParam.setModuleId(Convert.toStr(orderId));
            taskDO.setExtParameters(JSON.toJSONString(pushParam));
            pushService.addPushTask(taskDO);


            // 如果时间已经过了开始时间 => 立即开始服务
            if (curTime >= requireTime) {
                startService(orderId);
            } else {
                // 开始等待服务mq消息
                QueueSkillStartMsg skillStartMsg = new QueueSkillStartMsg();
                skillStartMsg.setOrderId(orderId);
                skillStartMsg.setUserId(userId);
                skillStartMsg.setReceiveUserId(receiveUserId);
                skillStartMsg.setRequireTime(requireTime);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SKILL_SERVICE_WAIT_START, skillStartMsg.serialize(), requireTime);
            }

            // 统计
            statPlatformService.addPlatformStat(userService.getChannelId(userId), StatPlatformTypeEnum.SKILL_ORDER_NUM, Convert.toBigDecimal(quantity));
            statPlatformService.addPlatformStat(userService.getChannelId(userId), StatPlatformTypeEnum.SKILL_ORDER_STAR_DIAMOND, Convert.toBigDecimal(payAmount));

            return true;
        }
        return false;
    }

    /**
     * 拒绝订单
     *
     * @param orderId
     * @param rejectReason
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean rejectOrder(Long orderId, String rejectReason) {

        boolean result = skillOrderProvider.doRejectOrder(orderId, rejectReason);
        if (result) {
            SkillOrderDO orderDO = skillOrderDAO.selectById(orderId);

            long curTime = System.currentTimeMillis();
            Long userId = orderDO.getUserId();
            Long receiveUserId = orderDO.getReceiveUserId();

            // 推送通知
            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(identityService.getId());
            taskDO.setTaskName("系统消息-技能服务");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(curTime);
            taskDO.setTaskType(TaskTypeEnum.SINGLE_PUSH.getValue());
            taskDO.setTargetValue(Convert.toStr(userId));

            taskDO.setAdminId(0L);
            taskDO.setRemark("系统自动推送");

            String title = String.format("%s 比较忙，已拒绝您的订单", userService.getUserName(receiveUserId));
            taskDO.setPushTitle(title);
            taskDO.setPushBody("戳>>>");

            PushMsgParam pushParam = new PushMsgParam();
            pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
            pushParam.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
            pushParam.setModuleId(Convert.toStr(userId));
            taskDO.setExtParameters(JSON.toJSONString(pushParam));
            pushService.addPushTask(taskDO);

            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setTitle("订单已拒绝");
            String body = String.format("你有一条【%s*%d】的订单已被拒绝，已给您退单！详情请私信接单人或更换为其他接单人重新下单。", skillService.getSkillName(orderDO.getSkillId()), orderDO.getQuantity());
            systemMsg.setBody(body);
            systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
            systemMsg.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
            systemMsg.setModuleId(Convert.toStr(orderId));
            imService.sendCustomImMsg(null, orderDO.getUserId(), systemMsg, null);
            return true;
        }
        return false;
    }

    /**
     * 开始服务
     *
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean startService(Long orderId) {
        return skillOrderProvider.doStartService(orderId, System.currentTimeMillis());
    }

    /**
     * 确认完成
     *
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result confirmService(Long orderId) {
        return skillOrderProvider.doConfirmService(orderId, System.currentTimeMillis());
    }

    /**
     * 判断技能订单是否存在
     *
     * @param orderId
     * @return
     */
    @Override
    public boolean isSkillOrderExist(Long orderId) {
        return skillOrderDAO.countById(orderId) > 0;
    }

    /**
     * 是否等待支付
     *
     * @param orderId
     * @return
     */
    @Override
    public boolean isWaitPay(Long orderId) {
        return skillOrderProvider.isWaitPay(orderId);
    }

    /**
     * 取消支付
     *
     * @param orderId
     * @return
     */
    @Override
    public boolean cancelPay(Long orderId) {
        return skillOrderProvider.cancelOrder(orderId);
    }

    /**
     * 支付语音速配
     *
     * @param hisId
     * @param userId
     * @param targetUserId
     * @param channelId
     * @param hisPrice
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long payForVoiceChat(Long hisId, Long userId, Long targetUserId, Long channelId, Long hisPrice) {
        log.info("支付语音速配 hisId {}", hisId);

        // 结算
        UserCardVO targetUserCard = userService.getCard(targetUserId);
        String remarks = String.format("与[%s](%d)语音速配聊天1分钟结算", targetUserCard.getUserName(), targetUserCard.getUserId());

        // 扣费
        Long balance = walletProvider.reduceBalance(WalletTypeEnum.STAR_DIAMOND, userId, hisPrice, remarks, targetUserId);
        if (balance < 0) {
            log.info("余额不足,记录id{},用户id{},余额{}", hisId, userId, balance);
            return balance;
        }

        // 异步结算
        settleProvider.settleByVoiceChat(hisId, userId, targetUserId, hisPrice);

        // 增加计费次数
        redisService.hashIncreaseLong(RedisFinanceKeys.VOICE_CHAT_CHARGING, Convert.toStr(hisId), 1);
        // 统计语音速配消耗金币
        statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.VOICE_CHAT_AMOUNT, BigDecimal.valueOf(hisPrice));
        // 统计语音速配分钟数
        statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.VOICE_CHAT_MINUTES, BigDecimal.ONE);

        return balance;
    }

    /**
     * 获取接单人用户id
     *
     * @param orderId
     * @return
     */
    @Override
    public Long getSkillOrderReceiveUserId(Long orderId) {
        return skillOrderDAO.selectReceiveUserId(orderId);
    }

    /**
     * 获取下单人用户id
     *
     * @param orderId
     * @return
     */
    @Override
    public Long getSkillOrderUserId(Long orderId) {
        return skillOrderDAO.selectUserId(orderId);
    }

    private void setInsertTime(RtcRoomGiftHis rmh) {
        long timestamp = System.currentTimeMillis();
        rmh.setTimeY(getTimestamp(timestamp, 4L));
        rmh.setTimeM(getTimestamp(timestamp, 3L));
        rmh.setTimeD(getTimestamp(timestamp, 2L));
        rmh.setTimeH(getTimestamp(timestamp, 1L));
        rmh.setCreatetime(timestamp);
    }


}
