package com.starbuds.server.service.finance.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.constant.cache.RedisRiskKeys;
import com.starbuds.server.common.constant.finance.PayStatusEnum;
import com.starbuds.server.common.constant.finance.PayTypeEnum;
import com.starbuds.server.common.constant.finance.PayWayEnum;
import com.starbuds.server.common.pojo.api.RankScore;
import com.starbuds.server.common.pojo.db.CoinRechargeItemDO;
import com.starbuds.server.common.pojo.db.CoinRechargeOrderDO;
import com.starbuds.server.common.pojo.db.PayOrderDO;
import com.starbuds.server.common.pojo.finance.CoinRecharge;
import com.starbuds.server.common.pojo.finance.CoinRechargeItem;
import com.starbuds.server.common.pojo.finance.CoinRechargeOrderAddDTO;
import com.starbuds.server.common.pojo.finance.CoinRechargeOrderVO;
import com.starbuds.server.common.pojo.user.ChannelPayWayConfig;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.RechargeService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.service.finance.dao.CoinRechargeItemDAO;
import com.starbuds.server.service.finance.dao.CoinRechargeOrderDAO;
import com.starbuds.server.service.finance.dao.PayOrderDAO;
import com.starbuds.server.service.finance.provider.RechargeProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;

/**
 * 充值服务实现
 *
 * @author Lay
 * @date 2019/12/17
 */
@Service(version = "1.0.0")
@Slf4j
public class RechargeServiceImpl implements RechargeService {

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Resource
    CoinRechargeItemDAO coinRechargeItemDAO;

    @Resource
    CoinRechargeOrderDAO coinRechargeOrderDAO;

    @Resource
    PayOrderDAO payOrderDAO;

    @Resource
    RechargeProvider rechargeProvider;

    @Value("${name.diamond}")
    private String nameDiamond;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 根据渠道获取金币充值项目
     *
     * @param channelId
     * @return
     */
    @Override
    public List<CoinRecharge> getCoinRechargeByChannelId(Long channelId) {
        // 根据渠道获取支付方式
        ChannelPayWayConfig payWayConfig = channelService.getPayWayConfig(channelId);
        if (payWayConfig != null && payWayConfig.getPayWaySize() > 0) {
            List<CoinRechargeItemDO> itemDOList = coinRechargeItemDAO.selectItemsByPayWays(payWayConfig.getPayWays());

            // 支付方式 充值项映射
            Map<Integer, CoinRecharge> coinRechargeMap = new HashMap<>();

            // 默认充值比例 100%
            if (payWayConfig.getRechargeRatio() == null) {
                payWayConfig.setRechargeRatio(1d);
            }

            //支付活动标签处理
            String AliPayTag = redisService.get(RedisFinanceKeys.getPayTypeAlipayTag());
            String WXPayTag = redisService.get(RedisFinanceKeys.getPayTypeWXTag());
            String aliStartTime = redisService.get(RedisFinanceKeys.getPayTypeALipayTimeStart());
            String aliEndTime = redisService.get(RedisFinanceKeys.getPayTypeALipayTimeEnd());
            String wxStartTime = redisService.get(RedisFinanceKeys.getPayTypeWXTimeStart());
            String wxEndTime = redisService.get(RedisFinanceKeys.getPayTypeWXTimeEnd());
            boolean aliTagShow = false;
            boolean wxTagShow = false;
            long timeNow = System.currentTimeMillis();
            if (StringUtils.isNotEmpty(AliPayTag) && StringUtils.isNotEmpty(aliStartTime) && StringUtils.isNotEmpty(aliEndTime)) {
                if (timeNow >= Long.parseLong(aliStartTime) && timeNow <= Long.parseLong(aliEndTime)) {
                    aliTagShow = true;
                }
            }
            if (StringUtils.isNotEmpty(WXPayTag) && StringUtils.isNotEmpty(wxStartTime) && StringUtils.isNotEmpty(wxEndTime)) {
                if (timeNow >= Long.parseLong(wxStartTime) && timeNow <= Long.parseLong(wxEndTime)) {
                    wxTagShow = true;
                }
            }

            // 遍历
            for (CoinRechargeItemDO item : itemDOList) {
                if (!coinRechargeMap.containsKey(item.getPayWay())) {
                    PayWayEnum payWayEnum = PayWayEnum.toEnum(item.getPayWay());
                    CoinRecharge coinRecharge = new CoinRecharge();
                    coinRecharge.setPayWay(item.getPayWay());
                    coinRecharge.setPayWayName(payWayEnum.getName());
                    coinRecharge.setPayWayProvider(getPayWayProvider(payWayEnum));
                    coinRecharge.setItems(new ArrayList<>());

                    if (payWayEnum.getName().contains("支付宝")) {
                        if (aliTagShow) {
                            log.info("paytypetagname1:" + "true" + "," + payWayEnum.getName() + "," + AliPayTag);
                            coinRecharge.setTag(AliPayTag);
                        }
                    }

                    if (payWayEnum.getName().contains("微信")) {
                        if (wxTagShow) {
                            coinRecharge.setTag(WXPayTag);
                        }
                    }

                    coinRechargeMap.put(item.getPayWay(), coinRecharge);
                }

                CoinRechargeItem coinRechargeItem = new CoinRechargeItem();
                coinRechargeItem.setItemId(item.getItemId());
                // 显示的名称，替换成乘以充值比例之后的
                coinRechargeItem.setItemCoin(Convert.toLong(item.getItemCoin() * payWayConfig.getRechargeRatio()));
                coinRechargeItem.setItemName(item.getItemName().replace(Convert.toStr(item.getItemCoin()), Convert.toStr(item.getItemCoin() * payWayConfig.getRechargeRatio())));
                coinRechargeItem.setItemDesc(item.getItemDesc().replace(Convert.toStr(item.getItemCoin()), Convert.toStr(item.getItemCoin() * payWayConfig.getRechargeRatio())));
                coinRechargeItem.setItemPrice(item.getItemPrice());
                coinRechargeMap.get(item.getPayWay()).getItems().add(coinRechargeItem);
            }
            return new ArrayList<>(coinRechargeMap.values());
        }
        return Collections.emptyList();
    }

    /**
     * 判断金币充值项目是否存在
     *
     * @param itemId
     * @return
     */
    @Override
    public boolean isCoinRechargeItemExists(Long itemId) {
        if (itemId != null && itemId > 0) {
            return coinRechargeItemDAO.countById(itemId) > 0;
        }
        return false;
    }

    /**
     * 获取金币充值项目
     *
     * @param itemId
     * @return
     */
    @Override
    public CoinRechargeItem getCoinRechargeItem(Long itemId) {
        CoinRechargeItemDO coinRechargeItemDO = coinRechargeItemDAO.selectById(itemId);
        if (coinRechargeItemDO != null) {
            CoinRechargeItem coinRechargeItem = new CoinRechargeItem();
            BeanUtils.copyProperties(coinRechargeItemDO, coinRechargeItem);
            return coinRechargeItem;
        }
        return null;
    }

    /**
     * 创建金币充值订单
     *
     * @param coinRechargeOrderAddDTO
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public CoinRechargeOrderVO addCoinRechargeOrder(CoinRechargeOrderAddDTO coinRechargeOrderAddDTO) {
        // 查询充值项目
        CoinRechargeItemDO coinRechargeItemDO = coinRechargeItemDAO.selectById(coinRechargeOrderAddDTO.getItemId());

        // 判断充值项目是否存在
        if (coinRechargeItemDO != null) {
            try {
                Long curTime = System.currentTimeMillis();

                // 创建金币充值订单
                Long orderId = identityService.getId();
                CoinRechargeOrderDO coinRechargeOrderDO = new CoinRechargeOrderDO();
                coinRechargeOrderDO.setOrderId(orderId);
                coinRechargeOrderDO.setItemId(coinRechargeItemDO.getItemId());
                coinRechargeOrderDO.setUserId(coinRechargeOrderAddDTO.getUserId());
                coinRechargeOrderDO.setItemPrice(coinRechargeItemDO.getItemPrice());

                ChannelPayWayConfig payWayConfig = channelService.getPayWayConfig(coinRechargeOrderAddDTO.getChannelId());
                if (payWayConfig != null && payWayConfig.getPayWaySize() > 0) {
                    if (payWayConfig.getRechargeRatio() == null) {
                        payWayConfig.setRechargeRatio(1d);
                    }
                    coinRechargeOrderDO.setItemName(coinRechargeItemDO.getItemName().replace(Convert.toStr(coinRechargeItemDO.getItemCoin()), Convert.toStr(coinRechargeItemDO.getItemCoin() * payWayConfig.getRechargeRatio())));
                    coinRechargeOrderDO.setItemCoin(Convert.toLong(coinRechargeItemDO.getItemCoin() * payWayConfig.getRechargeRatio()));
                } else {
                    coinRechargeOrderDO.setItemName(coinRechargeItemDO.getItemName());
                    coinRechargeOrderDO.setItemCoin(coinRechargeItemDO.getItemCoin());
                }

                coinRechargeOrderDO.setOrderCreateTime(curTime);
                coinRechargeOrderDO.setOrderUpdateTime(curTime);
                coinRechargeOrderDAO.insert(coinRechargeOrderDO);

                // 创建支付订单
                String subject = String.format("%.2f元%d%s",
                        coinRechargeOrderDO.getItemPrice(), coinRechargeOrderDO.getItemCoin(), nameDiamond);
//                String subject = "购买商品";
                PayOrderDO payOrderDO = new PayOrderDO();
                payOrderDO.setOrderId(orderId);
                payOrderDO.setUserId(coinRechargeOrderAddDTO.getUserId());
                payOrderDO.setChannelId(coinRechargeOrderAddDTO.getChannelId());
                payOrderDO.setPayType(PayTypeEnum.COIN_RECHARGE.getValue());
                payOrderDO.setPayWay(coinRechargeItemDO.getPayWay());
                payOrderDO.setOrderAmount(coinRechargeItemDO.getItemPrice());
                payOrderDO.setPayStatus(PayStatusEnum.WAITTING.getValue());
                payOrderDO.setPayOrderCreateTime(curTime);
                payOrderDO.setCreateIp(coinRechargeOrderAddDTO.getClientIp());
                payOrderDO.setPaySubject(subject);
                payOrderDO.setPayBody(subject);
                payOrderDAO.insert(payOrderDO);

                return CoinRechargeOrderVO.builder().itemId(coinRechargeItemDO.getItemId()).orderId(orderId).payWay(payOrderDO.getPayWay()).paySubject(subject).build();
            } catch (Exception e) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.error("金币充值订单创建失败", e);
            }
        }

        return null;
    }

    /**
     * 获取今日充值方式
     *
     * @param userId
     * @return
     */
    @Override
    public RankScore getTodayRecharge(Long userId) {
        String key = RedisFinanceKeys.getRechargeDaily(rechargeProvider.getToday());
        Double score = cacheService.zScore(key, String.valueOf(userId));
        if (score != null) {
            RankScore rankScore = new RankScore();
            rankScore.setScore(score);
            rankScore.setRankNo(cacheService.zRevRank(key, String.valueOf(userId)));
            return rankScore;
        }
        return null;
    }

    private static String getPayWayProvider(PayWayEnum payWay) {
        switch (payWay) {
            case APPLE:
                return "ApplePay";
            case ALI_APP:
            case ALI_H5:
                return "支付宝";
            case WX_APP:
            case WX_H5:
                return "微信";
            default:
                return "其他";
        }
    }
}
