package com.starbuds.server.service.finance.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.constant.finance.WithdrawPayTypeEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.UserIdCardDO;
import com.starbuds.server.common.pojo.finance.ThirdPartyWithdrawHisDO;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestBO;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestRecordBO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.SailtobService;
import com.starbuds.server.service.finance.dao.UserIDCardDAO;
import com.starbuds.server.service.finance.dao.UserSignContractDAO;
import com.starbuds.server.service.finance.impl.api.SailtobApiService;
import com.starbuds.server.service.finance.pojo.db.UserSignContractDO;
import com.starbuds.server.service.finance.pojo.sailtob.SailtobTax;
import com.starbuds.server.service.finance.provider.ThirdPartyWithdrawHisProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Map;

import static com.starbuds.server.common.constant.finance.ThirdPartyWithdrawStatusEnum.HAS_BEEN_SENT;

/***
 * @author qinjiang
 * @description 智企行服务实现
 * @date 2021/10/26
 */
@Slf4j
@Service(version = "1.0.0")
public class SailtobServiceImpl implements SailtobService {

    @Resource
    private SailtobApiService sailtobApiService;

    @Resource
    UserIDCardDAO userIDCardDAO;

    @Resource
    UserSignContractDAO userSignContractDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    ThirdPartyWithdrawHisProvider thirdPartyWithdrawHisProvider;

    /**
     * 执行提现请求并返回相关记录
     *
     * @param wr
     * @return 提现请求发送记录
     */
    @Override
    public WithdrawRequestRecordBO doWithdraw(WithdrawRequestBO wr) {
        WithdrawRequestRecordBO wrd = new WithdrawRequestRecordBO();

        WithdrawPayTypeEnum payTypeEnum = wr.getPayTypeEnum();
        BigDecimal balance = this.getBalance(Long.valueOf(payTypeEnum.getValue()));
        if (balance == null) {
            wrd.setErrMsg(payTypeEnum.getName() + "查询余额失败");
            return wrd;
        }
        if (wr.getAmount().compareTo(balance) == 1) {
            wrd.setErrMsg(payTypeEnum.getName() + "余额不足！余额:" + balance + "元 提现金额:" + wr.getAmount() + "元");
            return wrd;
        }

        wrd = sailtobApiService.doWithdraw(wr);

        ThirdPartyWithdrawHisDO po = new ThirdPartyWithdrawHisDO();
        Long uwwHisId = wr.getUserWalletWithdrawHisId();
        po.setUserWalletWithdrawHisId(uwwHisId);
        po.setUserId(wr.getUserId());
        po.setAmount(wr.getAmount());
        po.setAccountType(wr.getAccountTypeEnum().getValue());
        po.setReceivingAccount(wr.getReceivingAccount());
        po.setCompanyId(ThirdPartyCompanyEnum.SAILTOB.getId());
        po.setRequestTimes(1);
        po.setRequestData(wrd.getRequest());
        po.setRequestUrl(wrd.getUrl());
        po.setResponseData(wrd.getResponse());
        po.setResponseIsSuccess(wrd.getIsSuccess() ? BoolIntVal.True : BoolIntVal.False);
        po.setResponseErrMsg(wrd.getErrMsg());
        po.setSpendTime(wrd.getSpendTime().intValue());
        po.setWithdrawStatus(HAS_BEEN_SENT.getValue());

        if (thirdPartyWithdrawHisProvider.exist(uwwHisId)) {
            ThirdPartyWithdrawHisDO tpwPO = thirdPartyWithdrawHisProvider.getByUserWalletWithdrawHisId(uwwHisId);
            Integer requestTimes = tpwPO.getRequestTimes();

            po.setRequestTimes(requestTimes + 1);
            po.setUpdateTime(System.currentTimeMillis());

            QueryWrapper qw = new QueryWrapper();
            qw.eq("userWalletWithdrawHisId", uwwHisId);
            qw.eq("updateTime", tpwPO.getUpdateTime());
            UpdateWrapper uw = new UpdateWrapper();
            uw.setQw(qw);
            uw.setPo(po);
            thirdPartyWithdrawHisProvider.update(uw);
            log.info("提现记录更新 {}", JSON.toJSONString(po, true));
        } else {
            thirdPartyWithdrawHisProvider.create(po);
            log.info("提现记录新增 {}", JSON.toJSONString(po, true));
        }

        return wrd;
    }

    /**
     * 获得优税地余额
     *
     * @param bussinessId
     * @return
     */
    @Override
    public BigDecimal getBalance(Long bussinessId) {
        return sailtobApiService.getBalance(bussinessId);
    }

    /**
     * 用户签约
     *
     * @param userId
     * @return
     */
    @Override
    public Boolean sign(Long userId, String name, String mobile, String cardNo, String idCard, Long bussinessId) {
        UserIdCardDO userIdCardDO = userIDCardDAO.getUserIDCardByUserId(userId);
        if (userIdCardDO == null) {
            log.info("智企行：提现请求用户签约查找不成功");
            return Boolean.valueOf(false);
        }
        //获取用户身份证
        String idCardFront = userIdCardDO.getFaceImg();
        String idCardBack = userIdCardDO.getNationImg();
        SailtobTax sailtobTax = sailtobApiService.getMatchSailtobTax(bussinessId);
        if (sailtobTax == null) {
            log.info("智企行：签约查找配置不成功");
            return Boolean.valueOf(false);
        }

        boolean isSign = sailtobApiService.signSoho(name, cardNo, mobile, idCard, idCardFront, idCardBack, userId + "", bussinessId);
        if (isSign) {
            UserSignContractDO signContractDO = new UserSignContractDO();
            signContractDO.setHisId(identityService.getId());
            signContractDO.setUserId(userId);
            signContractDO.setPlatfrom(ThirdPartyCompanyEnum.SAILTOB.getId());
            signContractDO.setFlagId(Long.valueOf(sailtobTax.getLevyCode()));
            signContractDO.setName(name);
            signContractDO.setBankCardNo(cardNo);
            signContractDO.setIdCardNo(idCard);
            signContractDO.setIdCardFront(idCardFront);
            signContractDO.setIdCardBack(idCardBack);
            signContractDO.setPhone(mobile);
            signContractDO.setStatus(1);
            signContractDO.setExpireTimestamp(0L);
            signContractDO.setTimestamp(System.currentTimeMillis());

            userSignContractDAO.createUserSign(signContractDO);
        }

        return Boolean.valueOf(isSign);
    }

    @Override
    public Boolean verify(Map<String, String> map, String strSign) {
        boolean isVerify = sailtobApiService.verify(map, strSign);
        return Boolean.valueOf(isVerify);
    }

    /**
     * 用户是否已签约
     *
     * @param userId      用户ID
     * @param bussinessId 平台标识ID(前端传递过来)
     * @return
     */
    @Override
    public Boolean isUserSign(Long userId, Long bussinessId) {
        boolean isSign = sailtobApiService.isUserSign(userId, bussinessId);
        return Boolean.valueOf(isSign);
    }
}
