package com.starbuds.server.service.finance.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.SendGiftSnapshotDO;
import com.starbuds.server.common.pojo.mq.QueueSendGiftSnapShotMsg;
import com.starbuds.server.common.service.finance.SendGiftSnapshotMsgService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;

import javax.annotation.Resource;

import static com.starbuds.server.service.finance.util.SpringContextUtil.isDev;

/**
 * 送礼物快照
 */
@Service(version = "1.0.0")
@Slf4j
public class SendGiftSnapshotMsgServiceImpl implements SendGiftSnapshotMsgService {

    @Resource
    RocketProducer rocketProducer;

    @Override
    public void sendMQMsg(SendGiftSnapshotDO snapshotDO) {
        log.info("送礼快照生产mq:{}", JSON.toJSONString(snapshotDO));
        if (snapshotDO != null) {
            QueueSendGiftSnapShotMsg msg = new QueueSendGiftSnapShotMsg();
            BeanUtils.copyProperties(snapshotDO, msg);
            // 礼物赠送快照消息到消息队列
            boolean send = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SEND_GIFT_SNAPSHOT, msg.serialize());
            if (Boolean.FALSE.equals(send)) {
                String errMsg = "礼物赠送快照消息到消息队列失败 msg=";
                if (isDev()) {
                    log.error(LogUtil.devLog(errMsg + JSON.toJSONString(msg, true)));
                } else {
                    log.error(errMsg + msg);
                }
            }
        }
    }
}