package com.starbuds.server.service.finance.impl;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.SendGiftSnapshotDO;
import com.starbuds.server.common.pojo.finance.qo.SendGiftSnapshotPageListQO;
import com.starbuds.server.common.service.finance.SendGiftSnapshotService;
import com.starbuds.server.service.finance.dao.SendGiftSnapshotDAO;
import com.starbuds.server.service.finance.provider.SendGiftSnapshotProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 送礼物快照
 */
@Service(version = "1.0.0")
@Slf4j
public class SendGiftSnapshotServiceImpl implements SendGiftSnapshotService {

    @Resource
    private SendGiftSnapshotProvider sendGiftSnapshotProvider;

    @Resource
    private SendGiftSnapshotDAO sendGiftSnapshotDAO;

    @Override
    public SendGiftSnapshotDO getById(Long id) {
        return sendGiftSnapshotProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return sendGiftSnapshotProvider.exist(id);
    }

    @Override
    public PageList<SendGiftSnapshotDO> getPageList(SendGiftSnapshotPageListQO qo) {
        return sendGiftSnapshotProvider.getPageList(qo);
    }

    @Override
    public Map<Long, SendGiftSnapshotDO> getMapByIds(Collection ids) {
        return sendGiftSnapshotProvider.getMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获取个播场次总流水
     *
     * @param roomId
     * @param createTime
     * @return
     */
    @Override
    public Long getSingleRtcRoomRoundFlow(Long roomId, Long createTime) {
        return sendGiftSnapshotDAO.selectSingleRtcRoomRoundFlow(roomId, createTime);
    }

    /**
     * 获取个播场次直刷总流水
     *
     * @param roomId
     * @param createTime
     * @return
     */
    @Override
    public Long getSingleRtcRoomRoundDirectFlow(Long roomId, Long createTime) {
        return sendGiftSnapshotDAO.selectSingleRtcRoomRoundDirectFlow(roomId, createTime);
    }

    /**
     * 获取个播场次送礼人数(去重)
     *
     * @param roomId
     * @param createTime
     * @return
     */
    @Override
    public Long getSingleRtcRoomRoundSendGiftPeople(Long roomId, Long createTime) {
        return sendGiftSnapshotDAO.selectSingleRtcRoomRoundSendGiftPeople(roomId, createTime);
    }


    /**
     * 获取送礼人数
     *
     * @param beginTime
     * @param endTime
     * @param businessType
     * @return
     */
    @Override
    public Integer getSendGiftPeople(Long beginTime, Long endTime, Integer businessType) {
        return sendGiftSnapshotDAO.getSendGiftPeople(beginTime, endTime, businessType);
    }

    /**
     * 获取个播场次送礼列表
     *
     * @param roomId
     * @param createTime
     * @return
     */
    @Override
    public List<SendGiftSnapshotDO> getSingleRtcRoomRoundList(Long roomId, Long createTime, Integer isFromBag) {
        return sendGiftSnapshotDAO.selectSingleRtcRoomRoundList(roomId, createTime, isFromBag);
    }
}