package com.starbuds.server.service.finance.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestBO;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestRecordBO;
import com.starbuds.server.common.service.finance.WithdrawRequestService;
import com.starbuds.server.service.finance.impl.api.YunxiaokeApiService;
import com.starbuds.server.service.finance.pojo.api.ApiTrace;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 云销客（薪八达旗下品牌）服务
 *
 * @author aoe
 * @date 2021/5/31
 */
@Slf4j
@Service(version = "1.0.0")
public class YunxiaokeServiceImpl implements WithdrawRequestService {

    @Resource
    private YunxiaokeApiService yunxiaokeApiService;

    @Override
    public WithdrawRequestRecordBO doWithdraw(WithdrawRequestBO wr) {
        long startTime = System.currentTimeMillis();
        ApiTrace trace = yunxiaokeApiService.doWithdraw(wr);

        WithdrawRequestRecordBO wrd = new WithdrawRequestRecordBO();
        wrd.setUrl(trace.getUrl());
        wrd.setRequest(trace.getRequest());
        wrd.setSpendTime(startTime);
        wrd.setIsSuccess(false);

        String resStr = trace.getResponse();
        wrd.setResponse(resStr);
        JSONObject res = JSON.parseObject(resStr);
        if (res == null) {
            log.error("云销客返回结果不是JSON格式 res = " + resStr);
            return wrd;
        }
        JSONObject data = res.getJSONObject("data");
        if (data == null) {
            log.error("云销客返回结果data不是JSON格式 res = " + resStr);
            return wrd;
        }

        String outId = data.getString("order_uuid");
        if (StringUtils.isBlank(outId)) {
            log.error("云销客返回结果不包含 order_uuid");
            return wrd;
        }

        wrd.setOutId(outId);
        wrd.setIsSuccess(true);
        return wrd;
    }

    @Override
    public BigDecimal getBalance() {
        return yunxiaokeApiService.getBalance();
    }
}
