package com.starbuds.server.service.finance.impl.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.constant.finance.WithdrawPayTypeEnum;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestBO;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestRecordBO;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.service.finance.dao.UserIDCardDAO;
import com.starbuds.server.service.finance.dao.UserSignContractDAO;
import com.starbuds.server.service.finance.pojo.db.UserSignContractDO;
import com.starbuds.server.service.finance.pojo.sailtob.SailtobConfig;
import com.starbuds.server.service.finance.pojo.sailtob.SailtobTax;
import com.starbuds.server.service.finance.pojo.sailtob.SailtobWithholdingAccount;
import com.starbuds.server.service.finance.pojo.api.ApiTrace;
import com.starbuds.server.service.finance.provider.ThirdPartyWithdrawHisProvider;
import com.starbuds.server.service.finance.util.HttpUtil;
import com.starbuds.server.service.finance.util.SailtobSignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;


/***
 * @author qinjiang
 * @description 智企行打款服务
 * @date 2021/10/22
 */
@Service
@Slf4j
public class SailtobApiService {
    static String signType = "RSA";
    static String encType = "DES";
    static String charset = "UTF-8";

    //    static SailtobConfig sailtobConfig = null;
    //税优地余额,使用的时候按照 多-->少 的顺序排列
    static Map<String, SailtobWithholdingAccount> sailtobWithholdingMap = new HashMap<>(5);

    @Resource
    UserIDCardDAO userIDCardDAO;

    @Resource
    UserSignContractDAO userSignContractDAO;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @Resource
    private ThirdPartyWithdrawHisProvider thirdPartyWithdrawHisProvider;

    public WithdrawRequestRecordBO doWithdraw(WithdrawRequestBO wr) {
        SailtobConfig sailtobConfig = getSailtobConfig();

        String orderID = wr.getUserWalletWithdrawHisId().toString();
        Long userID = wr.getUserId();
        BigDecimal amt = wr.getAmount();
        String payeeName = wr.getRealName();
        String payeeAcc = wr.getReceivingAccount();
        String identifier = wr.getIdCard();
        String mobile = wr.getPhone();
        String remark = wr.getRemark();
        WithdrawPayTypeEnum payTypeEnum = wr.getPayTypeEnum();

        WithdrawRequestRecordBO wrd = new WithdrawRequestRecordBO();
        if (sailtobConfig == null) {
            wrd.setErrMsg(payTypeEnum.getName() + ":没有生效配置");
            return wrd;
        }

        long startTime = System.currentTimeMillis();
        Map<String, Object> againPaymentResult = payment(orderID, amt, payeeName, payeeAcc, identifier, mobile, Long.valueOf(payTypeEnum.getValue()));
        String strStatus = againPaymentResult.get("status").toString();
        String response = againPaymentResult.get("res").toString();
        String rspMsg = againPaymentResult.get("msg").toString();

        wrd.setUrl(againPaymentResult.get("url").toString());
        wrd.setRequest(againPaymentResult.get("req").toString());
        wrd.setSpendTime(startTime);
        wrd.setIsSuccess(false);

        if (!"1".equals(strStatus)) {
            wrd.setErrMsg(payTypeEnum.getName() + ":" + rspMsg);
            if ("6025".equals(strStatus)) {
                wrd.setErrMsg(payTypeEnum.getName() + " 签约失败，稍候再试！");
            }

            return wrd;
        }

        if (StringUtils.isEmpty(response)) {
            log.error("智企行打款结果为空");
            return wrd;
        }

        wrd.setResponse(response);
        JSONObject res = JSON.parseObject(response);
        if (res == null) {
            log.error("智企行返回结果：" + response);
            return wrd;
        }
        if (!"0000".equals(res.getString("resCode"))) {
            log.error("智企行 打款结果不成功" + response);
            return wrd;
        }
        wrd.setIsSuccess(true);

        return wrd;
    }

//    /**
//     * 提现请求
//     * @param wr
//     * @return
//     */
//    public ApiTrace doWithdraw1(WithdrawRequestBO wr) {
//        initSailtobConfig();
//
//        String orderID = wr.getUserWalletWithdrawHisId().toString();
//        Long userID = wr.getUserId();
//        BigDecimal amt = wr.getAmount();
//        String payeeName = wr.getRealName();
//        String payeeAcc = wr.getReceivingAccount();
//        String identifier = wr.getIdCard();
//        String mobile = wr.getPhone();
//        String remark = wr.getRemark();
//
//        //查询到的税优地 选择打款余额最多的税优地
//        List<Map.Entry<String,SailtobWithholdingAccount>> allWithholdingAccount = new ArrayList<>(sailtobWithholdingMap.entrySet());
//        Collections.sort(allWithholdingAccount, new Comparator<Map.Entry<String, SailtobWithholdingAccount>>() {
//            @Override
//            public int compare(Map.Entry<String, SailtobWithholdingAccount> o1, Map.Entry<String, SailtobWithholdingAccount> o2) {
//                return new BigDecimal(o2.getValue().getAmt()).compareTo(new BigDecimal(o1.getValue().getAmt()));
//            }
//        });
//        Map.Entry<String,SailtobWithholdingAccount> accountEntry = allWithholdingAccount.get(0);
//
//        SailtobTax tax = new SailtobTax();
//        tax.setLevyCode(accountEntry.getValue().getLevyCode());
//        tax.setBankCode(accountEntry.getValue().getBankCode());
//
//        String paymentURL = "";
//        String paymentReq = "";
//        String paymentRes = "";
//
//        try {
//            /**
//             * Map<string,Object>
//             * status: -1 检测数值类错误 、 1 成功 、 xxxx 其它数值为【智企行】响应码
//             * msg: 可以提示的消息
//             * res：请求网络后的响应信息
//             */
//            Map<String,Object> paymentResult = payment(orderID,amt,payeeName,payeeAcc,identifier,mobile,sailtobConfig,tax);
//            paymentURL = paymentResult.get("url").toString();
//            paymentReq = paymentResult.get("req").toString();
//            paymentRes = paymentResult.get("res").toString();
//
//            Integer status = Integer.valueOf(paymentResult.get("status").toString());
//            if (status.compareTo(1) == 0) {
//                return structTraceInstance(paymentURL,paymentReq,paymentRes);
//            }
//            else if (status.compareTo(6025) == 0) { //未签约
//                UserIdCardDO userIdCardDO = userIDCardDAO.getUserIDCardByUserId(userID);
//                if(userIdCardDO == null) {
//                    log.info("智企行：提现请求用户签约查找不成功");
//                    return structTraceInstance(paymentResult.get("url").toString(),paymentResult.get("req").toString(),"");
//                }
//                //获取用户身份证
//                String idCardFront = userIdCardDO.getFaceImg();
//                String idCardBack = userIdCardDO.getNationImg();
//                //先签约
//                boolean signSoho =  false;//signSoho(payeeName,payeeAcc,identifier,mobile,idCardFront,idCardBack,remark,sailtobConfig,tax);
//                if (signSoho) { //签约失败，尝试其它账号去打款
//                    Map<String,Object> againPaymentResult = payment(orderID,amt,payeeName,payeeAcc,identifier,mobile,sailtobConfig,tax);
//                    paymentURL = againPaymentResult.get("url").toString();
//                    paymentReq = againPaymentResult.get("req").toString();
//                    paymentRes = againPaymentResult.get("res").toString();
//                    Integer againStatus = Integer.valueOf(againPaymentResult.get("status").toString());
//                    if (againStatus.compareTo(1) == 0) {
//                        return structTraceInstance(againPaymentResult.get("url").toString(),againPaymentResult.get("req").toString(),againPaymentResult.get("res").toString());
//                    }
//                }
//            }
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//
//        //循环打款<看上面打款流程效果，可以循环寻找到打款成功>
////        for (Map.Entry<String,SailtobWithholdingAccount> element : allWithholdingAccount) { }
//
//        return structTraceInstance(paymentURL,paymentReq,paymentRes);
//    }

    /**
     * 获取公司账户余额
     *
     * @return 如果账号下有多个税优地，选择金额最大的一个
     */
    public BigDecimal getBalance(Long bussinessId) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        SailtobTax sailtobTax = getMatchSailtobTax(bussinessId);
        if (sailtobTax == null) {
            return BigDecimal.ZERO;
        }

        Map<String, Object> param = new HashMap();
        param.put("bankCode", sailtobTax.getBankCode());
        param.put("levyCode", sailtobTax.getLevyCode());

        String strHost = String.format("%s/account/balance/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());

        String resData = sendPOST(strHost, param, null, sailtobConfig);
        if (resData == null) {
            log.info("智企行：获取数据失败：：" + resData);
            return BigDecimal.ZERO;
        }

        JSONObject jsonObject = JSON.parseObject(resData);
        if (!jsonObject.containsKey("resCode")) {
            log.info("智企行：获取数据错误：：" + resData);
            return BigDecimal.ZERO;
        }

        Map<String, String> signMap = new HashMap();
        signMap.put("data", jsonObject.getString("data"));
        signMap.put("reqId", jsonObject.getString("reqId"));

        String strSign = SailtobSignUtil.sortKey(signMap);
        boolean isCorrect = SailtobSignUtil.verify(strSign, jsonObject.getString("sign"), sailtobConfig.getPublicKey());
        if (!isCorrect) {
            log.info("智企行：校验数据不通过::" + jsonObject.getString("sign"));
            return BigDecimal.ZERO;
        }

        String data = jsonObject.getString("data");
        String dataString = "";
        try {
            dataString = new String(SailtobSignUtil.decode(data), charset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        log.info("智企行：获取优税地余额列表::" + dataString);
        BigDecimal balance = BigDecimal.ZERO;
        List<SailtobWithholdingAccount> accounts = JSON.parseArray(dataString, SailtobWithholdingAccount.class);
        for (SailtobWithholdingAccount account : accounts) {
            BigDecimal accountAmt = new BigDecimal(account.getAmt());
            if (balance.compareTo(accountAmt) == -1) {
                balance = accountAmt;
            }
            sailtobWithholdingMap.put(account.getLevyCode(), account);
        }

        return balance;
    }

    /**
     * 付款交易
     *
     * @param orderID    支付订单号
     * @param amt        金额
     * @param payeeName  收款人姓名
     * @param payeeAcc   收款人账号
     * @param identifier 收款人身份证
     * @param mobile     收款人手机号
     * @throws Exception
     */
    public Map<String, Object> payment(String orderID, BigDecimal amt, String payeeName, String payeeAcc, String identifier, String mobile, Long bussinessID) {
        /**
         * Map<string,Object>
         * status: -1 检测数值类错误 、 1 成功 、 xxxx 其它数值为【智企行】响应码
         * msg: 可以提示的消息
         * res：请求网络后的响应信息
         */
        Map<String, Object> result = new HashMap<>();
        if (amt == null || amt.compareTo(BigDecimal.ZERO) <= 0) {
            result.put("url", "");
            result.put("req", "");
            result.put("status", Integer.valueOf("-1"));
            result.put("msg", "提现金额错误");
            result.put("res", "");
            return result;
        }

        SailtobConfig sailtobConfig = getSailtobConfig();
        SailtobTax sailtobTax = getMatchSailtobTax(bussinessID);
        if (sailtobTax == null) {
            result.put("status", Integer.valueOf("-1"));
            result.put("msg", "付款配置获取错误");
            return result;
        }

        JSONObject payItem = new JSONObject();
        payItem.put("merOrderId", orderID);
        payItem.put("amt", amt);
        payItem.put("payeeName", payeeName);
        payItem.put("payeeAcc", payeeAcc);
        payItem.put("idCard", identifier);
        payItem.put("mobile", mobile);
        payItem.put("accType", "PRI");
        payItem.put("levyCode", sailtobTax.getLevyCode());
        payItem.put("bankCode", sailtobTax.getBankCode());
        payItem.put("inType", "API");
        payItem.put("paymentType", "CARD");

        JSONArray payItemArray = new JSONArray();
        payItemArray.add(payItem);
        String encodePayItem = SailtobSignUtil.encode(JSON.toJSONString(payItemArray).getBytes());

        Map<String, Object> param = new HashMap();
        param.put("totalCount", "1");
        param.put("totalAmt", amt.toPlainString());
        param.put("merBatchId", String.valueOf(System.currentTimeMillis()));
        param.put("payItems", encodePayItem);

        String strHost = String.format("%s/payment/pay/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());
        String resData = sendPOST(strHost, param, null, sailtobConfig);

        //{"reqId":"1635317989958","resCode":"6003","resMsg":"企业信息不存在","sign":null,"ip":"125.119.235.186","data":null,"successNum":0,"failureNum":0,"merBatchId":null}

        result.put("url", strHost);
        result.put("req", JSON.toJSONString(param));

        if (resData == null) {
            log.info("智企行：：获取支付数据失败");
            result.put("status", Integer.valueOf("-1"));
            result.put("msg", "获取支付数据失败");
            result.put("res", "");
            return result;
        }

        JSONObject jsonObject = JSON.parseObject(resData);
        if (!jsonObject.containsKey("resCode")) {
            log.info("智企行：：获取支付数据格式错误：" + resData);
            result.put("status", Integer.valueOf("-1"));
            result.put("msg", "获取支付数据格式错误");
            result.put("res", resData);
            return result;
        }
        String resCode = jsonObject.getString("resCode");
        if (!resCode.equals("0000")) {
            log.info("智企行：：支付状态响应：" + resData);
            result.put("status", Integer.valueOf(resCode));
            result.put("msg", "支付状态响应未成功");
            result.put("res", resData);
            return result;
        }

        Map<String, String> signMap = new HashMap<>();
        signMap.put("successNum", jsonObject.getString("successNum"));
        signMap.put("failureNum", jsonObject.getString("failureNum"));
        signMap.put("merBatchId", jsonObject.getString("merBatchId"));
        signMap.put("data", jsonObject.getString("data"));
        signMap.put("reqId", jsonObject.getString("reqId"));
        String signString = SailtobSignUtil.sortKey(signMap);
        boolean sign = SailtobSignUtil.verify(signString, jsonObject.getString("sign"), sailtobConfig.getPublicKey());
        if (!sign) {
            log.info("智企行：打款响应数据验签不通过");
            result.put("status", Integer.valueOf("-1"));
            result.put("msg", "打款响应数据验签不通过");
            result.put("res", "");
            return result;
        }
        String data = jsonObject.getString("data");

        String dataString = "";
        try {
            dataString = new String(SailtobSignUtil.decode(data), charset);
        } catch (Exception e) {
            log.info("智企行：付款转换文本问题");
        }

//[{"amt":0.01,"merOrderId":"1635319576847","payItemId":"1453261855718899712","resCode":"0000","resMsg":"成功","splitFlag":"NOSPLIT"}]

        JSONArray dataArray = JSON.parseArray(dataString);
        JSONObject payJSONObject = dataArray.getJSONObject(0);

        log.info("智企行：解析响应文本" + payJSONObject.toJSONString());
        //解签后的数据进行组织存储
        jsonObject.put("data", dataArray);
        String payResCode = payJSONObject.getString("resCode");
        String payMessage = payJSONObject.getString("resMsg");

        result.put("status", payResCode.equals("0000") ? 1 : Integer.valueOf(payResCode));
        result.put("msg", payMessage);
        result.put("res", jsonObject.toJSONString());

        return result;
    }

    /**
     * 签约人才
     *
     * @param name
     * @param cardNo
     * @param idCard
     * @param mobile
     * @param idCardFront
     * @param idCardBack
     * @param remark
     * @param bussinessID
     * @return
     */
    public boolean signSoho(String name, String cardNo, String mobile, String idCard, String idCardFront, String idCardBack, String remark, Long bussinessID) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        SailtobTax sailtobTax = getMatchSailtobTax(bussinessID);
        if (sailtobTax == null) {
            return false;
        }

        byte[] idCardFrontStream = getFileStream(idCardFront);
        byte[] idCardBackStream = getFileStream(idCardBack);

        String desKey = sailtobConfig.getDesKey();
        Map<String, Object> params = new HashMap<>();
        params.put("name", SailtobSignUtil.encrypt(name, desKey));
        params.put("cardNo", SailtobSignUtil.encrypt(cardNo, desKey));
        params.put("idCard", SailtobSignUtil.encrypt(idCard, desKey));
        params.put("mobile", SailtobSignUtil.encrypt(mobile, desKey));
        params.put("levyCode", sailtobTax.getLevyCode());
        params.put("idCardType", "A");
        params.put("remark1", remark);

        Map<String, Object> attachParam = new HashMap<>();
        attachParam.put("idCardFront", bytesToHexString(idCardFrontStream));
        attachParam.put("idCardBack", bytesToHexString(idCardBackStream));

        String strHost = String.format("%s/soho/singleSoho/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());
        String resData = sendPOST(strHost, params, attachParam, sailtobConfig);
        log.info("智企云：人才签约：：请求地址：" + strHost);
        log.info("智企云：人才签约：：请求参数：" + JSON.toJSONString(params));
        log.info("智企云：人才签约：：响应文本：" + resData);
        if (StringUtils.isEmpty(resData)) {
            log.info("智企云：人才签约响应文本错误");
            return false;
        }

        // {"reqId":"1635246211091","resCode":"6016","resMsg":"该用户信息已经做过签约","sign":"lWWv4Wu+CGHpqSpPNc37Z8Zv9mfmwJjW8gneOQTDBS0vyMbJPeUgiHL508QPLPw/1eQWHy/KaYNhUDALiw+VFRDbyfgmMEa7LOiMU4E44YSj6/AtY7Jtul52qExWtPlNc2jl7A60NOdsucwqVKrEOYOCpeMcOZuAznX5jQefiEM=","ip":"125.119.235.186","remark1":"我是标签","remark2":null}

        //{"ip":"47.99.137.164","sign":"a5ZwoqqitaKNlezd6EfntR3N05HqQrSfCwEqN4kzWpCtGSSNFFYHK5cmnJ4E5dX498+qQCg8r3QbdomHXoQ8LQagQKJaeGTwpe1LQ6TCNkQACR5vNzvU1ZSKwulVNOyKh1yaX4nJnqcBUKB5QLjJLt6YoUk5/DN+mB8BJ9y2ttw=","resCode":"0000","resMsg":"成功","remark1":"9294409891907","reqId":"1636430715914"}
        JSONObject jsonObject = JSON.parseObject(resData);
        log.info("智企云：人才签约：解析文本：" + jsonObject.toJSONString());
        if (jsonObject.containsKey("resCode")) {
            String resCode = jsonObject.getString("resCode");
            // 6016:用户信息已经做过签约
            if ("6016".equals(resCode)) {
                return true;
            }
            if (!"0000".equals(resCode)) {
                return false;
            }
        }

        Map<String, String> signMap = new HashMap();
        signMap.put("remark1", jsonObject.getString("remark1"));
        signMap.put("reqId", jsonObject.getString("reqId"));

        String strSign = SailtobSignUtil.sortKey(signMap);
        boolean isCorrect = SailtobSignUtil.verify(strSign, jsonObject.getString("sign"), sailtobConfig.getPublicKey());
        if (!isCorrect) {
            log.info("智企云：人才签约数据校验失败");
            return false;
        }

//        String data = jsonObject.getString("data");
//        String dataString = new String(SailtobSignUtil.decode(data),charset);

        return true;
    }

    //查询人才
    public boolean querySoho(String name, String cardNo, String idCard, String mobile, String remark, SailtobConfig sailtobConfig, SailtobTax sailtobTax) throws Exception {
        Map<String, Object> params = new HashMap<>();
        params.put("name", SailtobSignUtil.encrypt(name, sailtobConfig.getDesKey()));
        params.put("cardNo", SailtobSignUtil.encrypt(cardNo, sailtobConfig.getDesKey()));
        params.put("idCard", SailtobSignUtil.encrypt(idCard, sailtobConfig.getDesKey()));
        params.put("mobile", SailtobSignUtil.encrypt(mobile, sailtobConfig.getDesKey()));
//        params.put("levyCode", sailtobConfig.levyCode);
        params.put("remark1", remark);

        String strHost = String.format("%s/soho/querySoho/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());
        String resData = sendPOST(strHost, params, null, sailtobConfig);
        if (resData == null) {
            return false;
        }

        JSONObject jsonObject = JSON.parseObject(resData);
        if (jsonObject.containsKey("resCode")) {
            String resCode = jsonObject.getString("resCode");
            if (!"0000".equals(resCode)) {
                return false;
            }
        }

//        {"reqId":"1635400226070","resCode":"0000","resMsg":"成功","sign":"H9jXIEYuNdpUEHUIq71+7yeOkX1tYbHz+ZAm3z8UI7tUV3PdoD3mAJSl8HAkUsjW93g7gqo5tfVwbSCBv+WZiyd2kQSBlUI994KPmqwXg0EDTqe2+zknoHsxHlydLn+w/Gjhz7VUCXWlCpWvigGgrIMx+Xn6ZoZ25GNcLJQ4bRo=","ip":null,"name":"kC4qh/Dh6alS6nDCWtMpbQ==","idCard":"W5rfPy1qMRgA4t7N8nVJSi6M3gT5ehbl","bankCard":"0G2xrdGYg6V5YJ0DyvfQ49BgItMGy3M4","mobile":"pJ3VTR5pElbsR0nPcQlTkA==","levyCode":"1000030412","state":"SIGN"}

        String sign = jsonObject.getString("sign");
        jsonObject.remove("sign");
        jsonObject.remove("resCode");
        jsonObject.remove("resMsg");
        jsonObject.remove("ip");

        Map<String, String> signMap = JSON.parseObject(jsonObject.toJSONString(), Map.class);
        String strSign = SailtobSignUtil.sortKey(signMap);
        boolean isCorrect = verify(signMap, sign);
        if (!isCorrect) {
            return false;
        }

        //state  SIGN:已签  NOSIGN:未签
        return "sign".equals(jsonObject.getString("state"));
    }

    /**
     * 发送请求
     *
     * @param strURL
     * @param param       传递参数(会签名)
     * @param attachParam 附加参数(不参与签名)
     * @return
     * @throws Exception
     */
    private String sendPOST(String strURL, Map<String, Object> param, Map<String, Object> attachParam, SailtobConfig sailtobConfig) {
        String reqId = String.valueOf(System.currentTimeMillis());

        Map<String, Object> commonParam = new TreeMap();
        commonParam.put("version", sailtobConfig.getVersion());
        commonParam.put("reqId", reqId);
        commonParam.put("merId", sailtobConfig.getMerId());
        commonParam.putAll(param);

        String linkString = SailtobSignUtil.createLinkString(commonParam, false);
        String signString = SailtobSignUtil.sign(linkString, sailtobConfig.getPrivateKey());
        commonParam.put("sign", signString);
        commonParam.put("signType", signType);
        commonParam.put("encType", encType);

        if (attachParam != null && attachParam.keySet().size() > 0) {
            commonParam.putAll(attachParam);
        }

        String strJSON = JSON.toJSONString(commonParam);
        log.info("智企行 {}  参数：{}", strURL, strJSON);

        String res = null;
        try {
            res = HttpUtil.post(strURL, strJSON);
            log.info("智企行 {} 请求成功:{}", strURL, res);
        } catch (Exception e) {
            log.error("智企行 {} 请求失败:{}", strURL, e.toString());
        }

        return res;
    }

    /**
     * 初始智企行配置
     *
     * @throws Exception
     */
    private SailtobConfig getSailtobConfig() {
        String strSailtobConfig = releaseConfigService.getCachReleaseConfig("sailtob");
        if (StringUtils.isEmpty(strSailtobConfig)) {
            log.info("获取在线打款智企行配置失败");
            return null;
        }

        return JSON.parseObject(strSailtobConfig, SailtobConfig.class);
    }

    /**
     * 查询用户是否签名
     *
     * @param userId
     * @param bussinessId
     * @return
     */
    public boolean isUserSign(Long userId, Long bussinessId) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        SailtobTax sailtobTax = getMatchSailtobTax(bussinessId);
        if (sailtobTax == null) {
            return false;
        }

        UserSignContractDO userSignContractDO = userSignContractDAO.selectUserSignRecord(userId, ThirdPartyCompanyEnum.SAILTOB.getId(), Long.valueOf(sailtobTax.getLevyCode()));
        if (userSignContractDO == null) {
            return false;
        }

        return true;
    }

    /**
     * 验签
     *
     * @param signMap map
     * @param sign    sign字符串
     * @return
     */
    public boolean verify(Map<String, String> signMap, String sign) {
        if (signMap == null) {
            return false;
        }

        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        String strSign = SailtobSignUtil.sortKey(signMap);
        return SailtobSignUtil.verify(strSign, sign, sailtobConfig.getPublicKey());
    }

    /**
     * 查找配置中的税优地
     *
     * @param bussinessID
     * @return
     */
    public SailtobTax getMatchSailtobTax(Long bussinessID) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return null;
        }

        SailtobTax sailtobTax = null;
        List<SailtobTax> listTax = sailtobConfig.getTax();
        for (SailtobTax tax : listTax) {
            if (tax.getId() == bussinessID.intValue()) {
                sailtobTax = tax;
                break;
            }
        }

        return sailtobTax;
    }

    /**
     * 构造Trace实例
     *
     * @param url      请求URL
     * @param request  请求数据
     * @param response 响应数据
     * @return
     */
    private ApiTrace structTraceInstance(String url, String request, String response) {
        ApiTrace trace = new ApiTrace();
        trace.setUrl(url);
        trace.setRequest(request);
        trace.setResponse(response);

        return trace;
    }

    /**
     * 得到文件流
     *
     * @param url 图片地址
     * @return
     */
    public static byte[] getFileStream(String url) {
        try {
            URL httpUrl = new URL(url);
            HttpURLConnection conn = (HttpURLConnection) httpUrl.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(10 * 1000);
            InputStream inStream = conn.getInputStream();//通过输入流获取图片数据
            byte[] btImg = readInputStream(inStream);//得到图片的二进制数据
            return btImg;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 从输入流中获取数据
     *
     * @param inStream 输入流
     * @return
     * @throws Exception
     */
    public static byte[] readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outStream.write(buffer, 0, len);
        }
        inStream.close();
        return outStream.toByteArray();
    }

    public static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder("");
        if (src == null || src.length <= 0) {
            return null;
        }
        for (byte aSrc : src) {
            int v = aSrc & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }
}
