package com.starbuds.server.service.finance.impl.api;

import cn.hutool.core.lang.UUID;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.finance.bo.WithdrawRequestBO;
import com.starbuds.server.service.finance.pojo.api.ApiTrace;
import com.starbuds.server.service.finance.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * 云销客接口服务
 *
 * @author aoe
 * @date 2021/6/1
 */
@Service
@Slf4j
public class YunxiaokeApiService {

    @Value("${yunxiaoke.baseUrl}")
    private String BASE_URL;

    /**
     * 企业号
     */
    @Value("${yunxiaoke.companyId}")
    private String COMPANY_ID;

    /**
     * 用户UUID
     */
    @Value("${yunxiaoke.xinbadaUserUuid}")
    private String XINBADA_USER_UUID;

    /**
     * 签名密钥
     */
    @Value("${yunxiaoke.secretKey}")
    private String SECRET_KEY;

    @Value("${yunxiaoke.callBack.host}")
    private String CALL_BACK_HOST;

    @Value("${yunxiaoke.callBack.port}")
    private String CALL_BACK_PORT;

    @Value("${yunxiaoke.callBack.url}")
    private String CALL_BACK_URL;

    @Value("${yunxiaoke.callBack.isHttps}")
    private Integer CALL_BACK_IS_HTTPS;

    /**
     * 发起签约
     */
    private String URL_FREEMAN;

    /**
     * 3.2 创建代付订单
     */
    private String URL_ORDERS;

    /**
     * 3.3 查询订单详情
     */
    private String URL_ORDER_INFO;

    /**
     * 查询余额
     */
    private String URL_BALANCE;

    /**
     * 3.1 查询账户信息
     */
    private String URL_ACCOUNTS;

    /**
     * 3.5 供应商信息查询
     */
    private String URL_SERVERS;

    /**
     * 需要调用3.5 供应商信息查询接口获取server_uuid
     */
    private String server_uuid;

    /**
     * 账户uuid
     * 需要调用3.1 查询账户信息接口获取account_uuid
     */
    private String account_uuid;

    @PostConstruct
    private void init() {
        URL_FREEMAN = BASE_URL + "/xinbada/open/apis/contracts/freeman";
        URL_ORDERS = BASE_URL + "/xinbada/open/apis/salary/orders";
        URL_ORDER_INFO = BASE_URL + "/xinbada/open/apis/salary/orders/";
        URL_ACCOUNTS = BASE_URL + "/xinbada/open/apis/accounts/";
        URL_SERVERS = BASE_URL + "/xinbada/open/apis/salary/servers";

        server_uuid = serverUuid();
        if (StringUtils.isBlank(server_uuid)) {
            log.error("云销客获取server_uuid失败");
        } else {
            log.info("云销客 server_uuid {}", server_uuid);
        }

        JSONObject accounts = accounts();
//        if (accounts.getJSONArray("data") == null
//                || accounts.getJSONArray("data").getJSONObject(0) == null) {
//            log.error("云销客获取account_uuid失败 accounts = " + accounts);
//        } else {
//            account_uuid = accounts.getJSONArray("data").getJSONObject(0).getString("account_uuid");
//        }
        account_uuid = "123";
        if (StringUtils.isBlank(account_uuid)) {
            log.error("云销客获取account_uuid失败");
        } else {
            log.info("云销客 account_uuid {}", account_uuid);
        }

        URL_BALANCE = BASE_URL + "/xinbada/open/apis/accounts/" + account_uuid;
    }

    /**
     * 单元测试类使用的初始化方法-测试环境
     */
    public void testInit() {
        BASE_URL = "https://sandbox-open.sinbaad.com";
        COMPANY_ID = "202105280001";
        XINBADA_USER_UUID = "119b0dec-c1d7-11eb-90cd-0242c0a8c00e";
        SECRET_KEY = "Fw2qfGI+WN4261ONpjE6nsrdSlWr65iZzdpLbu9W6r7bhUc+iCHSWJ+fHPojUrBpHEdkHR7mUBp5sFCc8052SdiDiPdkgN0jfmO9ci0C8q8MNW7nhlx9W4qjeIbThiedGWBWFTWXgGD+3g59wq98x4IRCkG/NVPNm950niaXQJo=";

        CALL_BACK_HOST = "api.ti1michat.net";
        CALL_BACK_PORT = "80";
        CALL_BACK_URL = "/api-notify/v1/yunxiaoke/ordersNotify";
        CALL_BACK_IS_HTTPS = 0;

        init();
    }

    /**
     * 单元测试类使用的初始化方法-生产环境
     */
    public void testProdInit() {
        BASE_URL = "https://open.sinbaad.com";
        COMPANY_ID = "202105280001";
        XINBADA_USER_UUID = "f96a0b00-c830-11eb-8b97-02420a0007e4";
        SECRET_KEY = "BK6gXwEAMH6N5aY38XHD2N2/KOMn+7+izYiF3bUSh45ixAtAVUKxcm99oND2cPdjwMssdHlUEkdY23d2giipTtXyMb9thtns+AFEgX+6vMac/GAHg5jUqLDq103g4GOrGgbgzS33FM0+33Bdyg5wG91bPgDRjdDL0dUF5l0FjC8=";

        CALL_BACK_HOST = "api.panygo.com";
        CALL_BACK_PORT = "80";
        CALL_BACK_URL = "/api-notify/v1/yunxiaoke/ordersNotify";
        CALL_BACK_IS_HTTPS = 0;

        init();
    }

    /**
     * 提现请求
     * 3.2 创建代付订单
     *
     * @return
     */
    public ApiTrace doWithdraw(WithdrawRequestBO wr) {
        // 转账对象
        JSONObject target = new JSONObject();
        // 姓名
        target.put("name", wr.getRealName());
        // 身份证
        target.put("social_no", wr.getIdCard());
        target.put("identity_type", "身份证");
        // 账户号
        target.put("card_no", wr.getReceivingAccount());
        target.put("mobile_no", wr.getPhone());
        // 金额(单位：元)
        target.put("amount", wr.getAmount());
        JSONArray task_list = new JSONArray();
        task_list.add(target);

        JSONObject call_back = new JSONObject();
        call_back.put("host", CALL_BACK_HOST);
        call_back.put("port", CALL_BACK_PORT);
        call_back.put("url", CALL_BACK_URL);
        // 是否https请求 1 是 0 否
        call_back.put("is_https", CALL_BACK_IS_HTTPS);

        JSONObject body = new JSONObject();
        // 供应商UUID(需要调用3.5 供应商信息查询接口获取server_uuid)
        body.put("server_user_uuid", server_uuid);
        // 企业账号ID（需要调用3.1 查询账户信息接口获取account_uuid）
        body.put("customer_account_uuid", account_uuid);
        // 预计下发日期 20190606
        body.put("to_launch_date", LocalDate.now().format(DateTimeFormatter.BASIC_ISO_DATE));
        // 下发个人明细
        body.put("task_list", task_list);
        // 回调地址
        body.put("call_back", call_back);
        // 是否同步确认订单金额:1 是 0 否,不传默认为0. 若不确认订单，需调用确认订单金额接口单独确认金额
        body.put("is_to_confirm_order", "1");

        Map<String, String> headerMap = headerMap(wr.getUserWalletWithdrawHisId() + "");
        String bodyStr = body.toJSONString();
        String signature = Base64.encodeBase64String(DigestUtils.getSha512Digest().digest(bodyStr.getBytes(Charset.forName("UTF-8"))));
        headerMap.put("signature", signature);

        String res = doPost(URL_ORDERS, bodyStr, "创建代付订单", headerMap);

        // order_status 只要管状态为2；30；50；-10这4中状态
        // 0:中间状态; 2:账户余额不足; 30:处理中 ; 50:已完成; -10:失败
        // 50/-10 才有回调

        ApiTrace trace = new ApiTrace();
        trace.setUrl(URL_ORDERS);
        JSONObject request = new JSONObject();
        request.put("header", headerMap);
        request.put("body", body);
        trace.setRequest(JSON.toJSONString(request));
        trace.setResponse(res);

        return trace;
    }

    /**
     * 获取公司账户余额
     * 3.6 查询账户余额
     *
     * @return
     */
    public BigDecimal getBalance() {
        String jsonStr = doGet(URL_BALANCE);
        JSONObject res = JSON.parseObject(jsonStr);
        if (res == null) {
            log.error("云销客查询账户余额失败，res = null");
            return BigDecimal.ZERO;
        }

        JSONObject data = res.getJSONObject("data");
        if (res == null) {
            log.error("云销客查询账户余额失败，data = null");
            return BigDecimal.ZERO;
        }

        String balanceAvailable = data.getString("balance_available");
        BigDecimal balance;
        try {
            balance = new BigDecimal(balanceAvailable);
        } catch (NumberFormatException e) {
            log.error("云销客查询账户余额失败，balance_available = " + balanceAvailable);
            return BigDecimal.ZERO;
        }

        return balance;
    }

    /**
     * 3.1 查询账户信息
     *
     * @return
     */
    public JSONObject accounts() {
        return doGet2JSON(URL_ACCOUNTS);
    }

    /**
     * 3.5 供应商信息查询
     *
     * @return
     */
    public JSONObject servers() {
        return doGet2JSON(URL_SERVERS);
    }

    /**
     * 获得服务id
     *
     * @return
     */
    private String serverUuid() {
        JSONObject res = servers();
        if (res == null) {
            return null;
        }
        if (res.getInteger("resp_code") - 200 != 0) {
            log.error("获取服务id失败 3.5 供应商信息查询结果 {}", res);
            return null;
        }
        JSONObject data = res.getJSONObject("data");
        if (data == null) {
            log.error("获取服务id失败! data is null 3.5 供应商信息查询结果 {}", res);
            return null;
        }
        JSONArray servers = data.getJSONArray("servers");
        if (CollectionUtils.isEmpty(servers)) {
            log.error("获取服务id失败! servers is empty 3.5 供应商信息查询结果 {}", res);
            return null;
        }
        JSONObject service = servers.getJSONObject(0);
        if (service == null) {
            log.error("获取服务id失败! service is null 3.5 供应商信息查询结果 {}", res);
            return null;
        }
        return service.getString("server_uuid");
    }

    /**
     * 查询订单详情
     *
     * @param orderUuid
     * @return
     */
    public JSONObject orderInfo(String orderUuid) {
        return doGet2JSON(URL_ORDER_INFO + orderUuid);
    }

    /**
     * 签约
     *
     * @return
     */
    private String freeman() {
        JSONObject body = new JSONObject();
        // 企业UUID
        body.put("ent_user_uuid", COMPANY_ID);
        // 身份证号
        body.put("personal_social_no", "vvv");
        // 姓名
        body.put("personal_user_name", "vvv");
        // 手机号
        body.put("personal_mobile_no", "vvv");
        // cccc
        body.put("kkk", "vvv");
        // cccc
        body.put("kkk", "vvv");
        // cccc
        body.put("kkk", "vvv");
        // cccc
        body.put("kkk", "vvv");
        // cccc
        body.put("kkk", "vvv");

        return null;
    }

    private String doGet(String url) {
        long startTime = System.currentTimeMillis();
        log.info("云销客 开始 {} {}", startTime, url);
        String res = null;
        try {
            //res = HttpUtil.get(url, headerMap(UUID.fastUUID().toString()));
            //res = "{\"123\":\"123\"}";
            log.info("云销客 {} 返回结果 {}", url, res);
        } catch (Exception e) {
            log.error(String.format("云销客 %s %d 请求失败 耗时 %d", url, startTime, System.currentTimeMillis() - startTime), e);
            return String.format("云销客 %s 请求失败", url);
        }
        log.info("云销客 {} {} 结束 耗时 {}", url, startTime, System.currentTimeMillis() - startTime);
        return res;
    }

    private JSONObject doGet2JSON(String url) {
        String res = doGet(url);
        return JSON.parseObject(res);
    }

    /**
     * 执行Post
     *
     * @param url
     * @param json
     * @param bizStr 业务描述
     * @return
     */
    private String doPost(String url, String json, String bizStr, Map<String, String> headerMap) {
        long startTime = System.currentTimeMillis();
        log.info("云销客 {} 开始 {} {} {}", bizStr, startTime, url, json);
        String res;
        try {
            res = HttpUtil.postMaxTimout(url, headerMap, json);
            log.info("云销客 {} 返回结果 {}", bizStr, res);
        } catch (Exception e) {
            log.error(String.format("云销客 %s %d 请求失败 耗时 %d", bizStr, startTime, System.currentTimeMillis() - startTime), e);
            return String.format("云销客 %s 请求失败", bizStr);
        }
        log.info("云销客 {} {} 结束 耗时 {}", bizStr, startTime, System.currentTimeMillis() - startTime);
        return res;
    }

    /**
     * 请求头
     *
     * @param sequenceNo 请求序列号(防止重复提交，天然幂等可用使用 UUID.fastUUID().toString())
     * @return
     */
    private Map<String, String> headerMap(String sequenceNo) {
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("xinbada_user_uuid", XINBADA_USER_UUID);
        headerMap.put("sequence_no", sequenceNo);
        headerMap.put("token", SECRET_KEY);
        return headerMap;
    }
}
