package com.starbuds.server.service.finance.impl.gift;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.finance.gift.SendGiftByFinanceAbstract;
import com.starbuds.server.common.pojo.finance.gift.SendGiftOrder;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.SendGiftService;
import com.starbuds.server.service.finance.provider.GiftProvider;
import com.starbuds.server.service.finance.provider.gift.GiftExecute;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.transaction.annotation.Transactional;

/**
 * 发送礼物
 *
 * @author aoe
 * @date 2021/11/16
 */
@Slf4j
public abstract class SendGiftAbstract implements SendGiftService {

    @Transactional(rollbackFor = Exception.class)
    public SendGiftOrder sendGift(SendGiftByFinanceAbstract sendGift) {
        log.info("接收到发送礼物对象 {}", JSON.toJSONString(sendGift));
        // 校验数据
        String errMsg = checkSendGiftByFinanceBO(sendGift);
        if (errMsg != null) {
            log.error("发送礼物失败 {}", errMsg);
            return null;
        }

        // 根据发送礼物类型建造发送礼物业务对象
        GiftExecute giftExecute = giftProvider().giftProvider(sendGift);
        sendGift = giftExecute.buildSendGift(sendGift);
        sendGift.setOrderId(identityService().getId());
        return sendGift(sendGift, giftExecute);
    }

    @Transactional(rollbackFor = Exception.class)
    public abstract SendGiftOrder sendGift(SendGiftByFinanceAbstract bo, GiftExecute giftExecute);

    protected abstract GiftProvider giftProvider();

    protected abstract IdentityService identityService();


    /**
     * 校验发送礼物数据
     *
     * @param bo
     * @return null:数据正确；错误原因
     */
    private String checkSendGiftByFinanceBO(SendGiftByFinanceAbstract bo) {
        if (bo == null) {
            return "SendGiftByFinanceBO = null";
        }
        if (bo.getSenderUserId() == null) {
            return "发送人id = null";
        }
        if (StringUtils.isBlank(bo.getSenderUserName())) {
            return "发送人姓名为空";
        }
        if (CollectionUtil.isEmpty(bo.getReceiverUsers())) {
            return "接收人集合为空";
        }
        boolean checkReceiver = bo.getReceiverUsers().stream()
                .anyMatch(receiver -> receiver.getUserId() == null || StringUtils.isBlank(receiver.getUserName()));
        if (checkReceiver) {
            return "接收人数据异常 userId 或 userName 有空值";
        }
        if (bo.getGiftId() == null) {
            return "礼物id = null";
        }
        if (bo.getGiftType() == null) {
            return "礼物类型 = null";
        }
        if (StringUtils.isBlank(bo.getGiftName())) {
            return "礼物名称为空值";
        }
        if (bo.getGiftPriceType() == null) {
            return "礼物价格类型 = null";
        }
        if (bo.getGiftPrice() == null) {
            return "礼物价格 = null";
        }
        if (bo.getQuantity() == null) {
            return "礼物数量 = null";
        }
        if (bo.getQuantity() < 1) {
            return "礼物数量小于1";
        }
        if (bo.getChannelId() == null) {
            return "渠道id = null";
        }
        if (bo.getIsFromBag() == null) {
            return "是否从背包发送 = null";
        }
        if (bo.getSendGiftType() == null) {
            return "发送礼物类型 = null";
        }
        return bo.checkParams();
    }
}
