package com.starbuds.server.service.finance.impl.lottery;

import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.service.finance.dao.LotteryGameUserQualificationHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * 抽奖活动中用户获取的抽奖资格
 */
@Slf4j
@Service(version = "1.0.0")
public class LotteryGameQualificationImpl implements LotteryGameQualification {

    public enum StrategiesEnum {
        FIRST_TIME,
        RANDOM_BY_RATE
    }

    private static Map<StrategiesEnum, LotteryGameStrategy> strategies = new HashMap<>();

    static {
        strategies.put(StrategiesEnum.FIRST_TIME, new LotteryGameStrategyFirstTimeImpl());
        strategies.put(StrategiesEnum.RANDOM_BY_RATE, new LotteryGameStrategyRandomByRateImpl());
    }

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    /**
     * 获取抽奖资质
     *
     * @param userId
     * @param summary
     */
    @Override
    public boolean getLotteryResult(Long userId, LotteryGameSummary summary) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("snapId", summary.getSnapId());
        qw.eq("userId", userId);

        // 第一次100%获取抽奖资格
        if (lotteryGameUserQualificationHisDAO.selectCount(qw) == 0) {
            return strategies.get(StrategiesEnum.FIRST_TIME).getResult(summary);
        }

        return strategies.get(StrategiesEnum.RANDOM_BY_RATE).getResult(summary);
    }

}