package com.starbuds.server.service.finance.impl.lottery;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.activity.LotteryGameSnapshotService;
import com.starbuds.server.common.service.activity.LotteryGameUserQualificationHisService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.finance.dao.LotteryGameUserQualificationHisDAO;
import com.starbuds.server.service.finance.provider.lottery.LotteryGameUserJoinHisProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Map;

/**
 * 抽奖活动中用户获取的抽奖资格
 */
@Service(version = "1.0.0")
@Slf4j
public class LotteryGameUserQualificationHisServiceImpl implements LotteryGameUserQualificationHisService {

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    LotteryGameSnapshotService lotteryGameSnapshotService;

    @Reference(version = "1.0.0")
    LotteryGameService lotteryGameService;

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    @Resource
    private LotteryGameQualification lotteryGameQualification;

    @Resource
    private LotteryGameWallet lotteryGameWallet;

    @Resource
    private LotteryGameUserJoinHisProvider lotteryGameUserJoinHisProvider;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Override
    public LotteryGameUserQualificationHisDO create(LotteryGameUserQualificationHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = lotteryGameUserQualificationHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖活动中用户获取的抽奖资格创建成功 {}", json);
            return po;
        }
        log.warn("抽奖活动中用户获取的抽奖资格创建失败 {}", json);
        return null;
    }

    /**
     * 更新数据
     */
    @Override
    public boolean updateById(LotteryGameUserQualificationHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("hisId", po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Override
    public boolean update(UpdateWrapper uw) {
        boolean flag = lotteryGameUserQualificationHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     *
     * @param id
     */
    @Override
    public LotteryGameUserQualificationHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("hisId", id);
        return lotteryGameUserQualificationHisDAO.selectOne(qw);
    }


    /**
     * 判断是否存在
     *
     * @param snapId 主键
     * @return true:存在
     */
    @Override
    public boolean exist(Long snapId, Long userId) {
        if (snapId == null) {
            return false;
        }
        if (userId == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("snapId", snapId);
        qw.eq("userId", userId);
        return lotteryGameUserQualificationHisDAO.selectCount(qw) > 0;
    }

    @Override
    public Map<Long, LotteryGameUserQualificationHisDO> getMapByIds(Collection ids) {
        return lotteryGameUserQualificationHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 检查用户钱包能否支持本次抽奖
     *
     * @param userId
     * @param summary
     */
    @Override
    public boolean checkUserWalletEnough(Long userId, LotteryGameSummary summary) {
        return lotteryGameWallet.checkUserWalletEnough(userId, summary);
    }
}