package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.service.finance.dao.FamilyDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.concurrent.TimeUnit;

/**
 * 缓存工具
 *
 * @author aoe
 * @date 2021/9/25
 */
@Slf4j
@Component
public class CacheProvider {

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Value("${gift.familyRatio}")
    private double giftFamilyRatio;

    @Resource
    private FamilyDAO familyDAO;

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    /**
     * 语聊房收益比率信息
     *
     * @param roomId
     * @return
     */
    public RtcRoomRatio getRtcRoomRatio(Long roomId) {
        RtcRoomRatio rtcRoomRatio;
        String key = RedisFinanceKeys.getRtcRoomRatio(roomId);
        String value = redisService.get(key);
        if (StringUtils.isNotBlank(value)) {
            try {
                rtcRoomRatio = JSON.parseObject(value, RtcRoomRatio.class);
                if (rtcRoomRatio != null) {
                    return rtcRoomRatio;
                }
            } catch (Exception e) {
                log.error(String.format("语聊房收益比率信息Redis存储数据异常，roomId=%s value=%s", roomId, value), e);
            }
        }

        rtcRoomRatio = rtcRoomService.getRtcRoomRatio(roomId);
        if (rtcRoomRatio != null) {
            redisService.set(key, JSON.toJSONString(rtcRoomRatio), 10, TimeUnit.SECONDS);
        }
        return rtcRoomRatio;
    }

    /**
     * 家族分成比例
     *
     * @param familyId
     * @return
     */
    @Deprecated
    public BigDecimal familyRatio(Long familyId) {
        String key = RedisFinanceKeys.getFamilyRatio(familyId);
        String value = redisService.get(key);
        double ratio = NumberUtils.toDouble(value, 0D);
        if (ratio == 0) {
            Double ratioDb = familyDAO.selectFamilyRatio(familyId);
            if (ratioDb == null) {
                log.error(String.format("家族分成比例设置错误 familyId=%s ratioDb=%s;使用默认比例giftFamilyRatio=%s",
                        familyId, ratioDb, giftFamilyRatio));
                ratio = giftFamilyRatio;
            } else {
                ratio = ratioDb;
                redisService.set(key, ratioDb.toString());
                log.info("Redis中设置礼物的家族分成比例 familyId={}, giftFamilyRatio={}", familyId, ratioDb);
            }
        }
        return new BigDecimal(ratio + "");
    }

}
