package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.finance.CaiyuContractDO;
import com.starbuds.server.service.finance.dao.CaiyuContractDAO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.finance.pojo.qo.CaiyuContractPageListQO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.List;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 财鱼用户签约合同
 */
@Component
@Slf4j
public class CaiyuContractProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "caiyuContractId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private CaiyuContractDAO caiyuContractDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public CaiyuContractDO create(CaiyuContractDO po) {
        po.setCaiyuContractId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = caiyuContractDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<CaiyuContractDO> list) {
        boolean flag = caiyuContractDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(CaiyuContractDO po) {
        boolean flag = caiyuContractDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(CaiyuContractDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getCaiyuContractId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = caiyuContractDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public CaiyuContractDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return caiyuContractDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param userId 用户Id
     * @return true:存在
     */
    public boolean exist(Long userId) {
        if (userId == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("userId", userId);
        return caiyuContractDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<CaiyuContractDO> getPageList(CaiyuContractPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<CaiyuContractDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(caiyuContractDAO.selectPageList(qw));
        page.setTotal(caiyuContractDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

}