package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum;
import com.starbuds.server.common.constant.finance.GameCoinOrderStatusEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.finance.GameCoinOrderDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.finance.dao.GameCoinOrderDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 游戏币订单（存取）
 */
@Component
@Slf4j
public class GameCoinOrderProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "orderId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private GameCoinOrderDAO gameCoinOrderDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public GameCoinOrderDO create(GameCoinOrderDO po) {
        po.setOrderId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setVersion(0);
        boolean flag = gameCoinOrderDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = gameCoinOrderDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public GameCoinOrderDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameCoinOrderDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameCoinOrderDAO.selectCount(qw) > 0;
    }

    /**
     * 获得游戏币订单状态
     *
     * @param orderId 订单id
     * @return
     */
    public Integer selectStatus(Long orderId) {
        return gameCoinOrderDAO.selectStatus(orderId);
    }

    /**
     * 转入在途总金额
     *
     * @param userId
     * @return
     */
    public Long selectInOnWayTotal(Long userId) {
        return gameCoinOrderDAO.selectInOnWayTotal(userId,
                GameCoinOrderStatusEnum.PROCESSING.getValue(),
                GameCoinOrderOperateEnum.IN.getValue());
    }
}