package com.starbuds.server.service.finance.provider;

import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.db.UserBagDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.service.finance.pojo.bo.lottery.LotteryGameAwardItemBO;

import java.util.List;

/**
 * 背包服务
 *
 * @author Lay
 * @date 2020/1/18
 */
public interface IBagProvider {
    /**
     * 增加物品数量
     *
     * @param userId
     * @param itemId
     * @param itemType
     * @param quantity
     * @param expireTime -1:永久有效
     * @param hisRemarks
     * @param orderId
     * @return
     */
    boolean addQuantity(Long userId, Long itemId, Integer itemType, Integer quantity, Long expireTime, String hisRemarks, Long orderId);

    /**
     * 减少物品数量
     *
     * @param userId
     * @param itemId
     * @param itemType
     * @param quantity
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Integer reduceQuantity(Long userId, Long itemId, Integer itemType, Integer quantity, String hisRemarks, Long orderId);

    /**
     * 获取用户背包礼物
     *
     * @param userId
     * @return
     */
    List<BagItem<GiftItemVO>> getUserGiftItems(Long userId);

    /**
     * 获取用户背包道具
     *
     * @param userId
     * @param propsType
     * @return
     */
    List<BagItem<PropsVO>> getUserPropsItems(Long userId, Integer propsType);

    /**
     * 获取用户背包道具
     *
     * @param bagId
     * @param userId
     * @return
     */
    BagItem<PropsVO> getProps(String bagId, Long userId);

    /**
     * 获取用户背包坐骑
     *
     * @param userId
     * @return
     */
    List<BagItem<MountsVO>> getUserMountsItems(Long userId);

    /**
     * 获取用户背包勋章
     *
     * @param userId
     * @return
     */
    List<BagItem<MedalVO>> getUserMedalItems(Long userId);

    /**
     * 获取用户背包头饰
     *
     * @param userId
     * @return
     */
    List<BagItem<HeadwearVO>> getUserHeadwearItems(Long userId);

    /**
     * 获取用户背包聊天气泡
     *
     * @param userId
     * @return
     */
    List<BagItem<ChatBubbleVO>> getUserChatBubbleItems(Long userId);

    /**
     * 根据物品 id 获取数量
     *
     * @param userId
     * @param bagItem
     * @param itemId
     * @return
     */
    int getQuantityByItemId(Long userId, BagItemTypeEnum bagItem, Long itemId);

    /**
     * 根据道具类型获取数量
     *
     * @param userId
     * @param propsType
     * @return
     */
    int getPropsQuantityByType(Long userId, PropsTypeEnum propsType);

    /**
     * 根据类型获取数量
     *
     * @param userId
     * @param bagItemType
     * @return
     */
    int getQuantityByType(Long userId, BagItemTypeEnum bagItemType);

    /**
     * 获取用户礼物列表
     *
     * @param userId
     * @return
     */
    List<UserBagDO> getUserBagDoList(Long userId);

    /**
     * 通过礼物ID获取
     *
     * @param giftIds
     * @param userId
     * @return
     */
    List<UserBagDO> getUserBagByGiftIds(List<Long> giftIds, Long userId);
}
