package com.starbuds.server.service.finance.provider;

import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.db.ChannelWalletDO;
import com.starbuds.server.common.pojo.finance.ChannelWalletHisVO;
import com.starbuds.server.common.pojo.finance.ChannelWalletVO;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/17
 */
public interface IChannelWalletProvider {
    /**
     * 增加余额
     *
     * @param walletType
     * @param channelId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long addBalance(WalletTypeEnum walletType, long channelId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 增加余额
     *
     * @param walletType
     * @param channelId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long addBalance(Integer walletType, long channelId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 减少余额
     *
     * @param walletType
     * @param channelId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long reduceBalance(WalletTypeEnum walletType, long channelId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 减少余额
     *
     * @param walletType
     * @param channelId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long reduceBalance(Integer walletType, long channelId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 获取渠道钱包
     *
     * @param channelId
     * @return
     */
    ChannelWalletVO getChannelWallet(Long channelId);

    /**
     * 获取渠道钱包
     *
     * @param walletId
     * @return
     */
    ChannelWalletDO getChannelWallet(String walletId);

    /**
     * 获取余额
     *
     * @param channelId
     * @param walletType
     * @return
     */
    Long getBalance(Long channelId, WalletTypeEnum walletType);

    /**
     * 获取余额
     *
     * @param channelId
     * @param walletType
     * @return
     */
    Long getBalance(Long channelId, Integer walletType);

    /**
     * 创建渠道钱包
     *
     * @param channelId
     * @return
     */
    boolean createChannelWallet(Long channelId);

    /**
     * 获取渠道钱包历史记录
     *
     * @param channelId
     * @param lastId
     * @param pageSize
     * @param walletType
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    List<ChannelWalletHisVO> getChannelWalletHisList(Long channelId, Long lastId, int pageSize, Integer walletType, Integer isIncome, Long beginTime, Long endTime);

    /**
     * 求和
     *
     * @param channelId
     * @param walletType
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    Long sumByTypeAndTime(Long channelId, Integer walletType, Integer isIncome, Long beginTime, Long endTime);
}
