package com.starbuds.server.service.finance.provider;

import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.db.UserWalletDO;
import com.starbuds.server.common.pojo.finance.UserWalletHisVO;
import com.starbuds.server.common.pojo.finance.UserWalletVO;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/17
 */
public interface IWalletProvider {
    /**
     * 增加余额
     *
     * @param walletType
     * @param userId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long addBalance(WalletTypeEnum walletType, long userId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 增加余额
     *
     * @param walletType
     * @param userId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long addBalance(Integer walletType, long userId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 减少余额
     *
     * @param walletType
     * @param userId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long reduceBalance(WalletTypeEnum walletType, long userId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 减少余额
     *
     * @param walletType
     * @param userId
     * @param balanceValue
     * @param hisRemarks
     * @param orderId
     * @return
     */
    Long reduceBalance(Integer walletType, long userId, long balanceValue, String hisRemarks, Long orderId);

    /**
     * 获取用户钱包
     *
     * @param userId
     * @return
     */
    UserWalletVO getUserWallet(Long userId);

    /**
     * 获取用户虚拟币钱包
     *
     * @param userId
     * @return
     */
    UserWalletVO getUserVirtualWallet(Long userId);

    /**
     * 获取用户钱包
     *
     * @param walletId
     * @return
     */
    UserWalletDO getUserWallet(String walletId);

    /**
     * 获取余额
     *
     * @param userId
     * @param walletType
     * @return
     */
    Long getBalance(Long userId, WalletTypeEnum walletType);

    /**
     * 获取余额
     *
     * @param userId
     * @param walletType
     * @return
     */
    Long getBalance(Long userId, Integer walletType);

    /**
     * 创建用户钱包
     *
     * @param userId
     * @return
     */
    boolean createUserWallet(Long userId);

    /**
     * 获取用户钱包历史记录
     *
     * @param userId
     * @param lastId
     * @param pageSize
     * @param walletType
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    List<UserWalletHisVO> getUserWalletHisList(Long userId, Long lastId, int pageSize, Integer walletType, Integer isIncome, Long beginTime, Long endTime);

    /**
     * 求和
     *
     * @param userId
     * @param walletType
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    Long sumByTypeAndTime(Long userId, Integer walletType, Integer isIncome, Long beginTime, Long endTime);
}
