package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.IssueGoldCoinHisDO;
import com.starbuds.server.common.pojo.finance.qo.IssueGoldCoinHisPageListQO;
import com.starbuds.server.service.finance.dao.IssueGoldCoinHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 系统发放金币流水
 */
@Component
@Slf4j
public class IssueGoldCoinHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Resource
    private IssueGoldCoinHisDAO issueGoldCoinHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public IssueGoldCoinHisDO create(IssueGoldCoinHisDO po) {
//        po.setHisId(null);
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        log.info("123123");
        boolean flag = issueGoldCoinHisDAO.insert(po) == 1;
        log.info("456456");
        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("系统发放金币流水创建成功 {}", json);
            return po;
        }
        log.warn("系统发放金币流水创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<IssueGoldCoinHisDO> list) {
        boolean flag = issueGoldCoinHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(IssueGoldCoinHisDO po) {
        boolean flag = issueGoldCoinHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(IssueGoldCoinHisDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = issueGoldCoinHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public IssueGoldCoinHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return issueGoldCoinHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return issueGoldCoinHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<IssueGoldCoinHisDO> getPageList(IssueGoldCoinHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<IssueGoldCoinHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(issueGoldCoinHisDAO.selectPageList(qw));
        page.setTotal(issueGoldCoinHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, IssueGoldCoinHisDO> getMapByIds(Collection ids) {
        return issueGoldCoinHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}