package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import com.starbuds.server.service.finance.dao.LotteryGameWinHisDAO;
import com.starbuds.server.common.pojo.finance.qo.LotteryGameWinHisPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 抽奖游戏的中奖记录
 */
@Component
@Slf4j
public class LotteryGameWinHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private LotteryGameWinHisDAO lotteryGameWinHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameWinHisDO create(LotteryGameWinHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = lotteryGameWinHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖游戏的中奖记录创建成功 {}", json);
            return po;
        }
        log.warn("抽奖游戏的中奖记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<LotteryGameWinHisDO> list) {
        boolean flag = lotteryGameWinHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(LotteryGameWinHisDO po) {
        boolean flag = lotteryGameWinHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(LotteryGameWinHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = lotteryGameWinHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public LotteryGameWinHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameWinHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameWinHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<LotteryGameWinHisDO> getPageList(LotteryGameWinHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<LotteryGameWinHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(lotteryGameWinHisDAO.selectPageList(qw));
        page.setTotal(lotteryGameWinHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, LotteryGameWinHisDO> getMapByIds(Collection ids) {
        return lotteryGameWinHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}