package com.starbuds.server.service.finance.provider;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.*;
import com.alipay.api.domain.AlipayTradeAppPayModel;
import com.alipay.api.domain.AlipayTradePagePayModel;
import com.alipay.api.domain.AlipayTradePrecreateModel;
import com.alipay.api.domain.AlipayTradeWapPayModel;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.alipay.api.request.AlipayTradePrecreateRequest;
import com.alipay.api.request.AlipayTradeWapPayRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.alipay.api.response.AlipayTradePagePayResponse;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import com.alipay.api.response.AlipayTradeWapPayResponse;
import com.github.binarywang.wxpay.bean.order.WxPayAppOrderResult;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.github.binarywang.wxpay.bean.order.WxPayMwebOrderResult;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderRequest;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.service.WxPayService;
import com.github.binarywang.wxpay.service.impl.WxPayServiceImpl;
import com.starbuds.server.common.pojo.db.PayConfigAlipayDO;
import com.starbuds.server.common.pojo.message.im.finance.PayResultNotifyMsg;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.finance.config.ApplePayConfig;
import com.starbuds.server.service.finance.constant.ApplePayConstant;
import com.starbuds.server.service.finance.constant.WxPayConstant;
import com.starbuds.server.service.finance.pojo.*;
import com.starbuds.server.service.finance.util.HuaweiSignUtil;
import com.starbuds.server.service.finance.util.encrypt.DESEncryptUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.File;
import java.util.Locale;
import java.util.Map;

/**
 * 支付
 *
 * @author Lay
 * @date 2019/12/18
 */
@Component
@Slf4j
public class PayProvider {

    /**
     * apple支付配置
     */
    public static ApplePayConfig applePayConfig;

    @Reference(version = "1.0.0")
    ImService imService;

    @Value("${alipay.gateway}")
    String alipayGateway;

    @Value("${alipay.timeoutExpress}")
    String alipayTimeoutExpress;

    @Value("${alipay.notifyUrl}")
    String alipayNotifyUrl;

    @Value("${alipay.returnUrl}")
    String alipayReturnUrl;

    @Value("${wxpay.notifyUrl}")
    String wxpayNotifyUrl;

    @Value("${huawei.notifyUrl}")
    String huaweiNotifyUrl;

    @Value("${alipay.appId}")
    String alipayAppId;

    @Value("${alipay.pid}")
    String alipayPid;

    @Value("${alipay.privateKey}")
    String alipayPrivateKey;

    @Resource
    ApplePayConfig applePayConfigSource;

    @PostConstruct
    public void beforeInit() {
        applePayConfig = applePayConfigSource;
    }

    /**
     * 获取支付宝 APP 支付参数
     *
     * @param alipayAppArgs
     * @return
     */
    public String getAlipayAppParams(AlipayAppArgs alipayAppArgs) {
        // 初始化 SDK
        AlipayClient alipayClient = getAlipayClient(alipayAppArgs.getAppId(), alipayAppArgs.getPrivateKey(), AlipayConstants.FORMAT_JSON, AlipayConstants.CHARSET_UTF8, alipayAppArgs.getAlipayPublicKey(), alipayAppArgs.getSignType());

        // 创建请求
        AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();

        // 请求体
        AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
        model.setSubject(alipayAppArgs.getSubject());
        model.setBody(alipayAppArgs.getBody());
        model.setOutTradeNo(alipayAppArgs.getOutTradeNo());
        model.setTimeoutExpress(alipayTimeoutExpress);
        model.setTotalAmount(alipayAppArgs.getTotalAmount());
        model.setPassbackParams(String.valueOf(alipayAppArgs.getConfigId()));
        request.setBizModel(model);

        // 异步通知地址
        request.setNotifyUrl(alipayNotifyUrl);

        try {
            if (alipayClient == null) {
                throw new Exception("SDK 初始化失败");
            }

            AlipayTradeAppPayResponse response = alipayClient.sdkExecute(request);
            if (response.isSuccess()) {
                return response.getBody();
            }
            throw new Exception(JSON.toJSONString(response));
        } catch (Exception e) {
            log.error("支付宝 APP 支付提交失败", e);
        }

        return null;
    }

    /**
     * 支付宝网页支付
     *
     * @param alipayAppArgs
     * @return
     */
    public String getAlipayPcParams(AlipayAppArgs alipayAppArgs) {
        // 初始化 SDK
        AlipayClient alipayClient = getAlipayClient(alipayAppArgs.getAppId(), alipayAppArgs.getPrivateKey(), AlipayConstants.FORMAT_JSON, AlipayConstants.CHARSET_UTF8, alipayAppArgs.getAlipayPublicKey(), alipayAppArgs.getSignType());

        // 创建请求
        AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();

        // 请求体
        AlipayTradePagePayModel model = new AlipayTradePagePayModel();
        model.setSubject(alipayAppArgs.getSubject());
        model.setBody(alipayAppArgs.getBody());
        model.setOutTradeNo(alipayAppArgs.getOutTradeNo());
        model.setTimeoutExpress(alipayTimeoutExpress);
        model.setTotalAmount(alipayAppArgs.getTotalAmount());
        model.setPassbackParams(String.valueOf(alipayAppArgs.getConfigId()));
        model.setProductCode("FAST_INSTANT_TRADE_PAY");
        request.setBizModel(model);

        // 异步通知地址
        request.setNotifyUrl(alipayNotifyUrl);

        // 同步回调地址
        request.setReturnUrl(alipayReturnUrl);

        try {
            if (alipayClient == null) {
                throw new Exception("SDK 初始化失败");
            }

            AlipayTradePagePayResponse response = alipayClient.pageExecute(request);
            if (response.isSuccess()) {
                return response.getBody();
            }
            throw new Exception(JSON.toJSONString(response));
        } catch (Exception e) {
            log.error("支付宝网页支付提交失败", e);
        }

        return null;
    }

    /**
     * 支付宝手机网站支付
     *
     * @param alipayAppArgs
     * @return
     */
    public String getAlipayWapParams(AlipayAppArgs alipayAppArgs) {
        // 初始化 SDK
        AlipayClient alipayClient = getAlipayClient(alipayAppArgs.getAppId(), alipayAppArgs.getPrivateKey(), AlipayConstants.FORMAT_JSON, AlipayConstants.CHARSET_UTF8, alipayAppArgs.getAlipayPublicKey(), alipayAppArgs.getSignType());

        // 创建请求
        AlipayTradeWapPayRequest request = new AlipayTradeWapPayRequest();

        // 请求体
        AlipayTradeWapPayModel model = new AlipayTradeWapPayModel();
        model.setSubject(alipayAppArgs.getSubject());
        model.setBody(alipayAppArgs.getBody());
        model.setOutTradeNo(alipayAppArgs.getOutTradeNo());
        model.setTimeoutExpress(alipayTimeoutExpress);
        model.setTotalAmount(alipayAppArgs.getTotalAmount());
        model.setPassbackParams(String.valueOf(alipayAppArgs.getConfigId()));
        model.setProductCode("QUICK_WAP_PAY");
        request.setBizModel(model);

        // 异步通知地址
        request.setNotifyUrl(alipayNotifyUrl);

        // 同步回调地址
        request.setReturnUrl(alipayReturnUrl);

        try {
            if (alipayClient == null) {
                throw new Exception("SDK 初始化失败");
            }

            AlipayTradeWapPayResponse response = alipayClient.pageExecute(request);
            if (response.isSuccess()) {
                return response.getBody();
            }
            throw new Exception(JSON.toJSONString(response));
        } catch (Exception e) {
            log.error("支付宝手机网站支付提交失败", e);
        }

        return null;
    }

    public String getAlipayQRParams(AlipayAppArgs alipayAppArgs) {
        // 初始化 SDK
        AlipayClient alipayClient = getAlipayClient(alipayAppArgs.getAppId(), alipayAppArgs.getPrivateKey(), AlipayConstants.FORMAT_JSON, AlipayConstants.CHARSET_UTF8, alipayAppArgs.getAlipayPublicKey(), alipayAppArgs.getSignType());

        // 创建请求
        AlipayTradePrecreateRequest request = new AlipayTradePrecreateRequest();

        // 请求体
        AlipayTradePrecreateModel model = new AlipayTradePrecreateModel();
        model.setSubject(alipayAppArgs.getSubject());
        model.setBody(alipayAppArgs.getBody());
        model.setOutTradeNo(alipayAppArgs.getOutTradeNo());
        model.setTimeoutExpress(alipayTimeoutExpress);
        model.setTotalAmount(alipayAppArgs.getTotalAmount());
        model.setStoreId(String.valueOf(alipayAppArgs.getChannelId()));
        request.setBizModel(model);

        // 异步通知地址
        request.setNotifyUrl(alipayNotifyUrl);

        try {
            if (alipayClient == null) {
                throw new Exception("SDK 初始化失败");
            }

            AlipayTradePrecreateResponse response = alipayClient.certificateExecute(request);
            if (response.isSuccess()) {
                return response.getQrCode();
            }
            throw new Exception(JSON.toJSONString(response));
        } catch (Exception e) {
            log.error("支付宝扫码支付提交失败", e);
        }

        return null;
    }

    /**
     * 获取微信 APP 支付参数
     *
     * @param wxpayAppArgs
     * @return
     */
    public String getWxpayAppParams(WxpayAppArgs wxpayAppArgs) {
        // 随机字符串
        String nonceStr = RandomUtil.randomString(32);

        WxPayUnifiedOrderRequest request = new WxPayUnifiedOrderRequest();
        request.setDeviceInfo(WxPayConstant.DEVICE_INFO_DEFAULT);
        request.setFeeType(WxPayConstant.FEE_TYPE_CNY);
        request.setTradeType(WxPayConstants.TradeType.APP);
        request.setNotifyUrl(wxpayNotifyUrl);
        request.setBody(wxpayAppArgs.getBody());
        request.setDetail(wxpayAppArgs.getDetail());
        request.setNonceStr(nonceStr);
        request.setOutTradeNo(wxpayAppArgs.getOutTradeNo());
        request.setTotalFee(wxpayAppArgs.getTotalFee());
        request.setSignType(WxPayConstants.SignType.HMAC_SHA256);
        request.setSpbillCreateIp(wxpayAppArgs.getIp());
        request.setAttach(String.valueOf(wxpayAppArgs.getConfigId()));

        try {
            WxPayConfig wxPayConfig = new WxPayConfig();
            wxPayConfig.setAppId(wxpayAppArgs.getAppId());
            wxPayConfig.setMchId(wxpayAppArgs.getMchId());
            wxPayConfig.setMchKey(wxpayAppArgs.getApiKey());
            wxPayConfig.setNotifyUrl(wxpayNotifyUrl);

            WxPayService wxPayService = new WxPayServiceImpl();
            wxPayService.setConfig(wxPayConfig);

            WxPayAppOrderResult result = wxPayService.createOrder(request);

            return JSON.toJSONString(result);
        } catch (Exception e) {
            log.error("微信支付创建失败", e);
            return null;
        }
    }

    /**
     * 获取微信 H5 支付参数
     *
     * @param wxpayAppArgs
     * @return
     */
    public String getWxpayH5Params(WxpayAppArgs wxpayAppArgs) {
        // 随机字符串
        String nonceStr = RandomUtil.randomString(32);

        WxPayUnifiedOrderRequest request = new WxPayUnifiedOrderRequest();
        request.setDeviceInfo(WxPayConstant.DEVICE_INFO_DEFAULT);
        request.setFeeType(WxPayConstant.FEE_TYPE_CNY);
        request.setTradeType(WxPayConstants.TradeType.JSAPI);
        request.setNotifyUrl(wxpayNotifyUrl);
        request.setBody(wxpayAppArgs.getBody());
        request.setDetail(wxpayAppArgs.getDetail());
        request.setNonceStr(nonceStr);
        request.setOutTradeNo(wxpayAppArgs.getOutTradeNo());
        request.setTotalFee(wxpayAppArgs.getTotalFee());
        request.setSignType(WxPayConstants.SignType.MD5);
        request.setSpbillCreateIp(wxpayAppArgs.getIp());
        request.setOpenid(wxpayAppArgs.getOpenId());
        request.setAttach(String.valueOf(wxpayAppArgs.getConfigId()));

        try {
            WxPayConfig wxPayConfig = new WxPayConfig();
            wxPayConfig.setAppId(wxpayAppArgs.getAppId());
            wxPayConfig.setMchId(wxpayAppArgs.getMchId());
            wxPayConfig.setMchKey(wxpayAppArgs.getApiKey());
            wxPayConfig.setNotifyUrl(wxpayNotifyUrl);

            WxPayService wxPayService = new WxPayServiceImpl();
            wxPayService.setConfig(wxPayConfig);

            WxPayMpOrderResult result = wxPayService.createOrder(request);

            return JSON.toJSONString(result);
        } catch (Exception e) {
            log.error("微信支付创建失败", e);
            return null;
        }
    }

    public String getWxPayWapParams(WxpayAppArgs wxpayAppArgs) {
        // 随机字符串
        String nonceStr = RandomUtil.randomString(32);

        WxPayUnifiedOrderRequest request = new WxPayUnifiedOrderRequest();
        request.setDeviceInfo(WxPayConstant.DEVICE_INFO_DEFAULT);
        request.setFeeType(WxPayConstant.FEE_TYPE_CNY);
        request.setTradeType(WxPayConstants.TradeType.MWEB);
        request.setNotifyUrl(wxpayNotifyUrl);
        request.setBody(wxpayAppArgs.getBody());
        request.setDetail(wxpayAppArgs.getDetail());
        request.setNonceStr(nonceStr);
        request.setOutTradeNo(wxpayAppArgs.getOutTradeNo());
        request.setTotalFee(wxpayAppArgs.getTotalFee());
        request.setSignType(WxPayConstants.SignType.MD5);
        request.setSpbillCreateIp(wxpayAppArgs.getIp());
        request.setAttach(String.valueOf(wxpayAppArgs.getConfigId()));

        try {
            WxPayConfig wxPayConfig = new WxPayConfig();
            wxPayConfig.setAppId(wxpayAppArgs.getAppId());
            wxPayConfig.setMchId(wxpayAppArgs.getMchId());
            wxPayConfig.setMchKey(wxpayAppArgs.getApiKey());
            wxPayConfig.setNotifyUrl(wxpayNotifyUrl);

            WxPayService wxPayService = new WxPayServiceImpl();
            wxPayService.setConfig(wxPayConfig);

            WxPayMwebOrderResult result = wxPayService.createOrder(request);

            return JSON.toJSONString(result);
        } catch (Exception e) {
            log.error("微信支付创建失败", e);
            return null;
        }
    }

    /**
     * 获取华为支付参数
     *
     * @param args
     * @return
     */
    public String getHuaweiPayParams(HuaweiPayArgs args) {
        HuaweiPayReq payReq = new HuaweiPayReq();
        payReq.setProductName(args.getProductName());
        payReq.setProductDesc((args.getProductDesc()));
        payReq.setApplicationID(args.getAppId());
        payReq.setRequestId(args.getRequestId());
        payReq.setAmount(args.getAmount());
        payReq.setMerchantId(args.getMerchantId());
        payReq.setMerchantName(args.getMerchantName());
        payReq.setServiceCatalog("X5");
        payReq.setSdkChannel("1");
        payReq.setUrl(huaweiNotifyUrl);
        payReq.setExtReserved(String.valueOf(args.getConfigId()));

        // 签名
        payReq.setSign(HuaweiSignUtil.signPayReq(payReq, args.getPrivateKey()));

        return JSON.toJSONString(payReq);
    }

    public AlipayClient getDefaultClient() {
        try {
            CertAlipayRequest certAlipayRequest = new CertAlipayRequest();
            certAlipayRequest.setServerUrl("https://openapi.alipay.com/gateway.do");
            certAlipayRequest.setAppId(alipayAppId);
            certAlipayRequest.setPrivateKey(alipayPrivateKey);
            certAlipayRequest.setFormat(AlipayConstants.FORMAT_JSON);
            certAlipayRequest.setCharset(AlipayConstants.CHARSET_UTF8);
            certAlipayRequest.setSignType(AlipayConstants.SIGN_TYPE_RSA2);
            certAlipayRequest.setCertPath("/data/server/cert/app.crt");
            certAlipayRequest.setAlipayPublicCertPath("/data/server/cert/alipay.crt");
            certAlipayRequest.setRootCertPath("/data/server/cert/root.crt");
            return new DefaultAlipayClient(certAlipayRequest);
        } catch (AlipayApiException e) {
            log.error("创建支付宝 Client 失败", e);
        }
        return null;
    }

    private AlipayClient getAlipayClient(String appId, String privatekey, String format, String charset, String publicKey, String signType) {
        try {
            // 判断是否存在证书
            String certPath = String.format("/data/server/cert/%s", appId);
            File certDir = new File(certPath);
            if (certDir.exists() && certDir.isDirectory()) {
                CertAlipayRequest certAlipayRequest = new CertAlipayRequest();
                certAlipayRequest.setServerUrl("https://openapi.alipay.com/gateway.do");
                certAlipayRequest.setAppId(appId);
                certAlipayRequest.setPrivateKey(privatekey);
                certAlipayRequest.setFormat(format);
                certAlipayRequest.setCharset(charset);
                certAlipayRequest.setSignType(signType);
                certAlipayRequest.setCertPath(String.format("%s%s", certPath, "/app.crt"));
                certAlipayRequest.setAlipayPublicCertPath(String.format("%s%s", certPath, "/alipay.crt"));
                certAlipayRequest.setRootCertPath(String.format("%s%s", certPath, "/root.crt"));
                return new DefaultAlipayClient(certAlipayRequest);
            } else {
                return new DefaultAlipayClient(alipayGateway, appId, privatekey, AlipayConstants.FORMAT_JSON, AlipayConstants.CHARSET_UTF8, publicKey, signType);
            }

        } catch (AlipayApiException e) {
            log.error("创建支付宝 Client 失败", e);
        }
        return null;
    }

    public boolean checkNotifySign(PayConfigAlipayDO payConfigAlipayDO, String appId, String charset, String signType, Map<String, String> params) {
        try {
            params.remove("sign_type");
            // 判断是否存在证书
            String certPath = String.format("/data/server/cert/%s", appId);
            File certDir = new File(certPath);
            if (certDir.exists() && certDir.isDirectory()) {
                return AlipaySignature.rsaCertCheckV2(params, String.format("%s%s", certPath, "/alipay.crt"), charset, signType);
            }
            return AlipaySignature.rsaCheckV2(params, payConfigAlipayDO.getAlipayPublicKey(), charset, signType);
        } catch (Exception e) {
            log.error("支付宝异步通知验签失败", e);
            return false;
        }
    }

    /**
     * 获取 ApplePay 结果
     *
     * @param receiptData
     * @param isSandbox
     * @return
     */
    public static String getApplePayResult(String receiptData, boolean isSandbox) {
        String gateway = isSandbox ? ApplePayConstant.GATEWAY_SANDBOX : ApplePayConstant.GATEWAY;

        ApplePayConfigArgs configArgs = applePayConfig.getArgs();
        if (configArgs == null || configArgs.getEnable() == false) {
            return directApplePayValidate(gateway, receiptData);
        }

        return bridgeApplePayValidate(gateway, receiptData, configArgs);
    }

    /**
     * 中转代理节点调用
     *
     * @param gateway
     * @param receiptData
     * @param configArgs
     * @return
     */
    protected static String bridgeApplePayValidate(String gateway, String receiptData, ApplePayConfigArgs configArgs) {
        try {
            String postReceitData = receiptData.replace("\n", "");
            String strJSONBody = String.format("{\"gateWay\":\"%s\",\"receiptData\":\"%s\"}", gateway, postReceitData);
            String encryptContent = DESEncryptUtil.encrypt(strJSONBody, configArgs.getEncrypt());
            String nodePath = String.format("%s/%s", configArgs.getNode_url(), "apple/validate");

            HttpResponse response = HttpRequest.post(nodePath)
                    .contentType("application/json")
                    .body(encryptContent)
                    .timeout(10 * 1000)
                    .execute();

            if (response.isOk()) {
                log.info("Apple bridge 查询返回: {}", response.body());

                JSONObject jsonObject = JSON.parseObject(response.body());
                int code = jsonObject.getIntValue("code");
                String responseData = jsonObject.getString("data");
                if (StringUtils.isBlank(responseData) || code != 1) {
                    //代理路劲访问如果不成功则直连调用
                    return directApplePayValidate(gateway, receiptData);
                }

                return DESEncryptUtil.decrypt(responseData, configArgs.getEncrypt());
            }

            //代理路劲访问如果不成功则使用直连调用去验证
            return directApplePayValidate(gateway, receiptData);

        } catch (Exception e) {
            log.error("ApplePay bridge 服务器异常", e);
        }
        return null;
    }

    /**
     * 直连applePay进行验证
     *
     * @param gateway
     * @param receiptData
     * @return
     */
    protected static String directApplePayValidate(String gateway, String receiptData) {
        try {
            String str = String.format(Locale.CHINA, "{\"receipt-data\":\"" + receiptData + "\"}");
            HttpResponse response = HttpRequest.post(gateway)
                    .contentType("text/json")
                    .body(str)
                    .execute();

            if (response.isOk()) {
                log.info("Apple 查询返回: {}", response.body());
                return response.body();
            }

            throw new RuntimeException(String.valueOf(response.getStatus()));
        } catch (Exception e) {
            log.error("ApplePay 服务器异常", e);
        }
        return null;
    }

    @Async
    public void notifyClientAsync(PayResultNotifyMsg msg) {
        try {
            imService.sendCustomImMsg(null, msg.getUserId(), msg, null);
        } catch (Exception e) {
            log.error("通知客户端支付结果失败", e);
        }
    }

}
