package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.finance.dao.ThirdPartyCallbackHisDAO;
import com.starbuds.server.service.finance.pojo.qo.ThirdPartyCallbackHisPageListQO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 第三方回调记录
 */
@Component
@Slf4j
public class ThirdPartyCallbackHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ThirdPartyCallbackHisDAO thirdPartyCallbackHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.NOT_SUPPORTED)
    public ThirdPartyCallbackHisDO create(ThirdPartyCallbackHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = thirdPartyCallbackHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ThirdPartyCallbackHisDO> list) {
        boolean flag = thirdPartyCallbackHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ThirdPartyCallbackHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return thirdPartyCallbackHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return thirdPartyCallbackHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ThirdPartyCallbackHisDO> getPageList(ThirdPartyCallbackHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ThirdPartyCallbackHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(thirdPartyCallbackHisDAO.selectPageList(qw));
        page.setTotal(thirdPartyCallbackHisDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

}