package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisDO;
import com.starbuds.server.common.pojo.finance.ThirdPartyWithdrawHisDO;
import com.starbuds.server.service.finance.dao.ThirdPartyWithdrawHisDAO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.finance.pojo.qo.ThirdPartyWithdrawHisPageListQO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.List;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * 第三方提现记录
 */
@Component
@Slf4j
public class ThirdPartyWithdrawHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ThirdPartyWithdrawHisDAO thirdPartyWithdrawHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.NOT_SUPPORTED)
    public ThirdPartyWithdrawHisDO create(ThirdPartyWithdrawHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = thirdPartyWithdrawHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ThirdPartyWithdrawHisDO> list) {
        boolean flag = thirdPartyWithdrawHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ThirdPartyWithdrawHisDO po) {
        boolean flag = thirdPartyWithdrawHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ThirdPartyWithdrawHisDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.NOT_SUPPORTED)
    public boolean update(UpdateWrapper uw) {
        boolean flag = thirdPartyWithdrawHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ThirdPartyWithdrawHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return thirdPartyWithdrawHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param userWalletWithdrawHisId 用户钱包提现记录id
     * @return true:存在
     */
    public boolean exist(Long userWalletWithdrawHisId) {
        if (userWalletWithdrawHisId == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("userWalletWithdrawHisId", userWalletWithdrawHisId);
        return thirdPartyWithdrawHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ThirdPartyWithdrawHisDO> getPageList(ThirdPartyWithdrawHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ThirdPartyWithdrawHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(thirdPartyWithdrawHisDAO.selectPageList(qw));
        page.setTotal(thirdPartyWithdrawHisDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

    /**
     * 根据用户钱包提现记录id查询
     *
     * @param userWalletWithdrawHisId
     * @return
     */
    public ThirdPartyWithdrawHisDO getByUserWalletWithdrawHisId(Long userWalletWithdrawHisId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("userWalletWithdrawHisId", userWalletWithdrawHisId);
        return thirdPartyWithdrawHisDAO.selectOne(qw);
    }

    /**
     * 根据第三方id获取记录（目前只要云销客需要）
     *
     * @param outId
     * @param companyEnum
     * @return
     */
    public ThirdPartyWithdrawHisDO getByOutId(String outId, ThirdPartyCompanyEnum companyEnum) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("outId", outId);
        qw.eq("companyId", companyEnum.getId());
        return thirdPartyWithdrawHisDAO.selectOne(qw);
    }
}