package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.UserBagChangeHisDO;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.service.finance.dao.UserBagChangeHisDAO;
import com.starbuds.server.common.pojo.finance.qo.UserBagChangeHisPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 背包变更记录
 */
@Component
@Slf4j
public class UserBagChangeHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private UserBagChangeHisDAO userBagChangeHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public UserBagChangeHisDO create(UserBagChangeHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = userBagChangeHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("背包变更记录创建成功 {}", json);
            return po;
        }
        log.warn("背包变更记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<UserBagChangeHisDO> list) {
        boolean flag = userBagChangeHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(UserBagChangeHisDO po) {
        boolean flag = userBagChangeHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(UserBagChangeHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = userBagChangeHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public UserBagChangeHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return userBagChangeHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return userBagChangeHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<UserBagChangeHisDO> getPageList(UserBagChangeHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<UserBagChangeHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(userBagChangeHisDAO.selectPageList(qw));
        page.setTotal(userBagChangeHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, UserBagChangeHisDO> getMapByIds(Collection ids) {
        return userBagChangeHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 插入数据
     *
     * @param msg
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public UserBagChangeHisDO create(QueueAddItemToBagMsg msg) {
        UserBagChangeHisDO po = new UserBagChangeHisDO();
        po.setOrderId(msg.getOrderId());
        po.setSource(msg.getSource().getValue());
        po.setItemId(msg.getItemId());
        po.setItemName(msg.getItemName());
        po.setItemType(msg.getItemType().getValue());
        po.setQuantity(msg.getQuantity());
        po.setUserId(msg.getUserId());
        po.setHisRemarks(msg.getRemark());
        return create(po);
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(Collection<QueueAddItemToBagMsg> msgList) {
        long createTime = System.currentTimeMillis();
        List<UserBagChangeHisDO> list = msgList.stream()
                .map(msg -> {
                    UserBagChangeHisDO po = new UserBagChangeHisDO();
                    po.setHisId(identityService.getId());
                    po.setOrderId(msg.getOrderId());
                    po.setSource(msg.getSource().getValue());
                    po.setItemId(msg.getItemId());
                    po.setItemName(msg.getItemName());
                    po.setItemType(msg.getItemType().getValue());
                    po.setQuantity(msg.getQuantity());
                    po.setUserId(msg.getUserId());
                    po.setHisRemarks(msg.getRemark());
                    po.setCreateTime(createTime);
                    return po;
                })
                .collect(Collectors.toList());
        return createBatch(list);
    }
}