package com.starbuds.server.service.finance.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.finance.WalletTransDustToPinkHisDO;
import com.starbuds.server.service.finance.dao.WalletTransDustToPinkHisDAO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.finance.pojo.qo.WalletTransDustToPinkHisPageListQO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.List;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 星尘转粉钻记录
 */
@Component
@Slf4j
public class WalletTransDustToPinkHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private WalletTransDustToPinkHisDAO walletTransDustToPinkHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public WalletTransDustToPinkHisDO create(WalletTransDustToPinkHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = walletTransDustToPinkHisDAO.insert(po) == 1;

        if (flag) {
            return po;
        }
        String json = JSON.toJSONString(po);
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<WalletTransDustToPinkHisDO> list) {
        boolean flag = walletTransDustToPinkHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(WalletTransDustToPinkHisDO po) {
        boolean flag = walletTransDustToPinkHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(WalletTransDustToPinkHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = walletTransDustToPinkHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public WalletTransDustToPinkHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return walletTransDustToPinkHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param transDate 兑换结算日期
     * @param userId
     * @return true:存在
     */
    public boolean exist(Integer transDate, Long userId) {
        if (transDate == null || userId == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("transDate", transDate);
        qw.eq("userId", userId);
        return walletTransDustToPinkHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<WalletTransDustToPinkHisDO> getPageList(WalletTransDustToPinkHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<WalletTransDustToPinkHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(walletTransDustToPinkHisDAO.selectPageList(qw));
        page.setTotal(walletTransDustToPinkHisDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

}