package com.starbuds.server.service.finance.provider.gift;

import com.starbuds.server.common.pojo.finance.gift.SendGiftByFinanceAbstract;
import com.starbuds.server.common.pojo.finance.gift.SendGiftResult;
import com.starbuds.server.service.finance.impl.gift.GiftRate;

import java.math.BigDecimal;
import java.util.List;

/**
 * 礼物执行器
 *
 * @author aoe
 * @date 2020/12/17
 */
public interface GiftExecute {

    /**
     * 礼物基准费率
     *
     * @return
     */
    BigDecimal baseRate();

    /**
     * 计算礼物费率
     *
     * @return
     */
    GiftRate giftRate();

    /**
     * 根据发送礼物类型建造发送礼物业务对象
     *
     * @param sendGift
     * @return 具体的发送礼物对象
     */
    SendGiftByFinanceAbstract buildSendGift(SendGiftByFinanceAbstract sendGift);

    /**
     * 执行从背包发送礼物完成后的任务
     *
     * @param sendGift
     * @param results  发送礼物结果
     */
    void doAfterSendByBag(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results);

    /**
     * 执行消耗钻石发送礼物完成后的任务
     *
     * @param sendGift
     * @param results  发送礼物结果
     */
    void doAfterSendByDiamond(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results);

    /**
     * 发送消息
     *
     * @param sendGift
     * @param results  发送礼物结果-从背包赠送礼物集合
     */
    void sendMsg(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results);
}
