package com.starbuds.server.service.finance.provider.gift;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.finance.gift.SendGiftByFinanceAbstract;
import com.starbuds.server.common.pojo.finance.gift.SendGiftPrivateChatBO;
import com.starbuds.server.common.pojo.finance.gift.SendGiftResult;
import com.starbuds.server.common.pojo.mq.QueueBaseGiftPaidMsg;
import com.starbuds.server.service.finance.impl.gift.GiftRate;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 私聊送礼物
 *
 * @author aoe
 * @date 2020/12/17
 */
@Component
@Slf4j
public class SendGiftPrivateChatExecute implements GiftExecute {

    @Resource
    private RocketProducer rocketProducer;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Override
    public BigDecimal baseRate() {
        return Convert.toBigDecimal(giftNoFamilyUserRatio);
    }

    @Override
    public GiftRate giftRate() {
        return (sendGift, receiver) -> baseRate();
    }

    @Override
    public SendGiftByFinanceAbstract buildSendGift(SendGiftByFinanceAbstract sendGift) {
        SendGiftPrivateChatBO bo = (SendGiftPrivateChatBO) sendGift;
        return bo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doAfterSendByBag(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {

    }

    @Override
    public void doAfterSendByDiamond(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {

    }

    @Override
    public void sendMsg(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        SendGiftPrivateChatBO bo = (SendGiftPrivateChatBO) sendGift;
        QueueBaseGiftPaidMsg giftPaidMsg = new QueueBaseGiftPaidMsg();
        giftPaidMsg.setOrderId(bo.getOrderId());
        giftPaidMsg.setGiftId(bo.getGiftId());
        giftPaidMsg.setGiftType(bo.getGiftType());
        giftPaidMsg.setGiftPrice(bo.getGiftPrice());
        giftPaidMsg.setGiftPriceType(bo.getGiftPriceType());
        giftPaidMsg.setQuantity(bo.getQuantity());
        giftPaidMsg.setSenderUserId(bo.getSenderUserId());
        giftPaidMsg.setFromBag(bo.getIsFromBag());
        giftPaidMsg.setChannelId(bo.getChannelId());
        giftPaidMsg.setReward(false);
        bo.getReceiverUsers().forEach(receiver -> {
            giftPaidMsg.setReceiveUserId(receiver.getUserId());
        });
        // 丢出礼物赠送消息到消息队列
        boolean flag = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.PRIVATE_CHAT_GIFT_PAID, giftPaidMsg.serialize());
        log.info("私聊送礼物发送消息结果 {} {}", flag, JSON.toJSONString(giftPaidMsg));
    }
}
