package com.starbuds.server.service.finance.provider.gift;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.SendGiftRewardFeedSnapshotDO;
import com.starbuds.server.common.pojo.finance.gift.*;
import com.starbuds.server.common.pojo.mq.QueueBaseGiftPaidMsg;
import com.starbuds.server.common.pojo.mq.QueueFeedSendGiftMsg;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.impl.gift.GiftRate;
import com.starbuds.server.service.finance.provider.SendGiftRewardFeedSnapshotProvider;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * 打赏动态送礼物
 *
 * @author aoe
 * @date 2020/12/17
 */
@Component
@Slf4j
public class SendGiftRewardFeedExecute implements GiftExecute {

    @Resource
    private RocketProducer rocketProducer;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Resource
    private SendGiftRewardFeedSnapshotProvider sendGiftRewardFeedSnapshotProvider;

    @Override
    public BigDecimal baseRate() {
        return Convert.toBigDecimal(giftNoFamilyUserRatio);
    }

    @Override
    public GiftRate giftRate() {
        return (sendGift, receiver) -> baseRate();
    }

    @Override
    public SendGiftByFinanceAbstract buildSendGift(SendGiftByFinanceAbstract sendGift) {
        SendGiftRewardFeedBO bo = (SendGiftRewardFeedBO) sendGift;
        return bo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doAfterSendByBag(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doAfterSendByDiamond(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        SendGiftRewardFeedBO bo = (SendGiftRewardFeedBO) sendGift;
        log.info(LogUtil.devLog("打赏了 -------------------- \n{}"), JSON.toJSONString(bo, true));
        SendGiftRewardFeedSnapshotDO snap = new SendGiftRewardFeedSnapshotDO();
        snap.setFeedId(bo.getFeedId());
        snap.setOrderId(bo.getOrderId());
        snap.setSenderUserId(bo.getSenderUserId());
        snap.setSenderUserName(bo.getSenderUserName());
        // 受赠人
        Set<GiftReceiver> receiverUsers = bo.getReceiverUsers();
        if (CollectionUtils.isNotEmpty(receiverUsers)) {
            Optional<GiftReceiver> receiver = receiverUsers.stream().findAny();
            if (receiver.isPresent()) {
                snap.setReceiverUserId(receiver.get().getUserId());
                snap.setReceiverUserName(receiver.get().getUserName());
            }
        }
        snap.setGiftId(bo.getGiftId());
        snap.setGiftName(bo.getGiftName());
        snap.setGiftType(bo.getGiftType());
        snap.setGiftPrice(bo.getGiftPrice());
        snap.setGiftPriceType(bo.getGiftPriceType());
        snap.setQuantity(bo.getQuantity());
        snap.setIsFromBag(bo.getIsFromBag() ? 1 : 0);
        snap.setTotalPay(bo.getGiftPrice() * bo.getQuantity());
        SendGiftRewardFeedSnapshotDO create = sendGiftRewardFeedSnapshotProvider.create(snap);
        if (create != null) {
            log.info("保存送礼物-打赏动态快照成功 {}", JSON.toJSONString(create));
        } else {
            log.error("保存送礼物-打赏动态快照失败 {}", JSON.toJSONString(snap));
        }
    }

    @Override
    public void sendMsg(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        SendGiftRewardFeedBO rewardFeedBO = (SendGiftRewardFeedBO) sendGift;
        // 保持私聊对象
        SendGiftPrivateChatBO bo = new SendGiftPrivateChatBO();
        BeanUtils.copyProperties(sendGift, bo);
        QueueBaseGiftPaidMsg giftPaidMsg = new QueueBaseGiftPaidMsg();
        giftPaidMsg.setOrderId(bo.getOrderId());
        giftPaidMsg.setGiftId(bo.getGiftId());
        giftPaidMsg.setGiftType(bo.getGiftType());
        giftPaidMsg.setGiftPrice(bo.getGiftPrice());
        giftPaidMsg.setGiftPriceType(bo.getGiftPriceType());
        giftPaidMsg.setQuantity(bo.getQuantity());
        giftPaidMsg.setSenderUserId(bo.getSenderUserId());
        giftPaidMsg.setFromBag(bo.getIsFromBag());
        giftPaidMsg.setChannelId(bo.getChannelId());
        giftPaidMsg.setReward(true);
        giftPaidMsg.setFeedId(rewardFeedBO.getFeedId());
        bo.getReceiverUsers().forEach(receiver -> {
            giftPaidMsg.setReceiveUserId(receiver.getUserId());
        });
        // 丢出礼物赠送消息到消息队列
        boolean flag = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.PRIVATE_CHAT_GIFT_PAID, giftPaidMsg.serialize());
        log.info("打赏动态送礼物发送消息结果 {} {}", flag, JSON.toJSONString(giftPaidMsg));

        // 发送动态打赏消息
        QueueFeedSendGiftMsg feedSendGiftMsg = new QueueFeedSendGiftMsg();
        feedSendGiftMsg.setFeedId(rewardFeedBO.getFeedId());
        feedSendGiftMsg.setOperatorUserId(rewardFeedBO.getSenderUserId());
        Set<GiftReceiver> receiverUsers = rewardFeedBO.getReceiverUsers();
        Optional<GiftReceiver> first = receiverUsers.stream().findFirst();
        feedSendGiftMsg.setGiftPrice(rewardFeedBO.getGiftPrice());
        feedSendGiftMsg.setQuantity(rewardFeedBO.getQuantity());
        if (first.isPresent()) {
            feedSendGiftMsg.setUserId(first.get().getUserId());
            boolean sendFeedSendGiftMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FEED_SEND_GIFT, feedSendGiftMsg.serialize());
            log.info("动态打赏发送消息结果 {} {}", sendFeedSendGiftMsg, JSON.toJSONString(feedSendGiftMsg));
        }
    }
}
