package com.starbuds.server.service.finance.provider.gift;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.finance.gift.SendGiftByFinanceAbstract;
import com.starbuds.server.common.pojo.finance.gift.SendGiftResult;
import com.starbuds.server.common.pojo.finance.gift.SendGiftRtcUnderMaiBO;
import com.starbuds.server.common.pojo.mq.QueueRtcGiftUnderMaiPaidMsg;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.impl.gift.GiftRate;
import com.starbuds.server.service.finance.provider.CacheProvider;
import com.starbuds.server.service.finance.provider.FamilyWalletProvider;
import com.starbuds.server.service.finance.provider.ratio.RtcRatioProvider;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 语音房麦下送礼物
 *
 * @author aoe
 * @date 2020/12/17
 */
@Component
@Slf4j
public class SendGiftRtcUnderMaiExecute implements GiftExecute {

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    private FamilyService familyService;

    @Resource
    private FamilyWalletProvider familyWalletProvider;

    @Resource
    private RocketProducer rocketProducer;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Value("${gift.familyUserRatio}")
    private double giftFamilyUserRatio;

    @Resource
    private CacheProvider cacheProvider;

    @Resource
    private RtcRatioProvider rtcRatioProvider;

    @Override
    public BigDecimal baseRate() {
        return Convert.toBigDecimal(giftNoFamilyUserRatio);
    }

    @Override
    public GiftRate giftRate() {
        return (sendGift, receiver) -> {
            // 用户家族
            SendGiftRtcUnderMaiBO gift = (SendGiftRtcUnderMaiBO) sendGift;
            Long userFamilyId = Optional.ofNullable(gift.getUserFamilyMap()).map(e -> e.get(receiver.getUserId())).orElse(null);
            /*// 用户家族
            Long familyId = Optional.ofNullable(gift.getUserFamilyMap()).map(e -> e.get(receiver.getUserId())).orElse(null);
            // 是否设置分成比例
            Double userRatio = rtcRoomService.getUserRatio(gift.getRoomId());
            if (userRatio != null) {
                return Convert.toBigDecimal(userRatio);
            }
            if (gift.getRoomFamilyId() != null && gift.getRoomFamilyId().equals(familyId)) {
                return Convert.toBigDecimal(giftFamilyUserRatio);
            }
            return baseRate();*/
            RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(gift.getRoomId());
            BigDecimal userRatio = rtcRatioProvider.getUserRatio(userFamilyId, rtcRoomRatio);
            rtcRatioProvider.checkRatio(userRatio);
            return userRatio;
        };
    }

    @Override
    public SendGiftByFinanceAbstract buildSendGift(SendGiftByFinanceAbstract sendGift) {
        SendGiftRtcUnderMaiBO bo = (SendGiftRtcUnderMaiBO) sendGift;
        bo.setRoomFamilyId(rtcRoomService.getFamilyId(bo.getRoomId()));
        Set<Long> userIds = bo.getReceiverUsers().stream()
                .map(GiftReceiver::getUserId)
                .collect(Collectors.toSet());
        bo.setUserFamilyMap(familyService.getFamilyIds(userIds));
        return bo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doAfterSendByBag(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        doFamilyIncome(sendGift, results);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void doAfterSendByDiamond(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        doFamilyIncome(sendGift, results);
    }

    @Override
    public void sendMsg(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        SendGiftRtcUnderMaiBO bo = (SendGiftRtcUnderMaiBO) sendGift;
        RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(bo.getRoomId());
        int roomFamilyUserCount = 0;
        for (GiftReceiver receiver : sendGift.getReceiverUsers()) {
            Long familyId = Optional.ofNullable(bo.getUserFamilyMap()).map(e -> e.get(receiver.getUserId())).orElse(null);
            if (bo.getRoomFamilyId() != null && bo.getRoomFamilyId().equals(familyId)) {
                roomFamilyUserCount++;
            }
        }

        QueueRtcGiftUnderMaiPaidMsg giftPaidMsg = new QueueRtcGiftUnderMaiPaidMsg();
        giftPaidMsg.setOrderId(bo.getOrderId());
        giftPaidMsg.setGiftId(bo.getGiftId());
        giftPaidMsg.setGiftType(bo.getGiftType());
        giftPaidMsg.setGiftPrice(bo.getGiftPrice());
        giftPaidMsg.setGiftPriceType(bo.getGiftPriceType());
        giftPaidMsg.setQuantity(bo.getQuantity());
        giftPaidMsg.setSenderUserId(bo.getSenderUserId());
        giftPaidMsg.setSenderUserName(bo.getSenderUserName());
        giftPaidMsg.setSenderUserAvatar(bo.getSenderUserAvatar());
        giftPaidMsg.setFromBag(bo.getIsFromBag());
        giftPaidMsg.setChannelId(bo.getChannelId());
        giftPaidMsg.setRoomId(bo.getRoomId());
        giftPaidMsg.setReceiverUsers(bo.getReceiverUsers());
        giftPaidMsg.setRoomFamilyUserCount(roomFamilyUserCount);
        giftPaidMsg.setAppVer(sendGift.getAppVer());
        giftPaidMsg.setRoomBusinessType(rtcRoomRatio.getBusinessType());
        // 丢出礼物赠送消息到消息队列
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_UNDER_MAI_GIFT_PAID, giftPaidMsg.serialize());
    }

    /**
     * 计算家族收益
     *
     * @param sendGift
     * @param results
     */
    @Transactional(rollbackFor = Exception.class)
    public void doFamilyIncome(SendGiftByFinanceAbstract sendGift, List<SendGiftResult> results) {
        boolean isLotteryGift = GiftTypeEnum.LOTTERY.getValue() == sendGift.getGiftType();
        SendGiftRtcUnderMaiBO bo = (SendGiftRtcUnderMaiBO) sendGift;
        Long roomFamilyId = bo.getRoomFamilyId();
        RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(bo.getRoomId());
        BigDecimal giftFamilyRatio = rtcRatioProvider.getFamilyRatio(rtcRoomRatio);
        results.stream().forEach(result -> {
            if (isLotteryGift) {
                return;
            }

            // 判断房间是否有所属家族
            if (roomFamilyId == null) {
                return;
            }
//            BigDecimal giftFamilyRatio = cacheProvider.familyRatio(roomFamilyId);
            rtcRatioProvider.checkRatio(result.getRate(), giftFamilyRatio);
            BigDecimal receiveAmount = BigDecimal.valueOf(result.getUnitPrice())
                    .multiply(giftFamilyRatio)
                    .setScale(2, RoundingMode.DOWN);
            // 返到家族钱包
            BigDecimal balance = familyWalletProvider.addBalance(roomFamilyId, receiveAmount,
                    String.format("[%s](%d)赠送%d个%s", result.getSenderUserName(), result.getSenderUserId(), result.getQuantity(), result.getGiftName()),
                    bo.getOrderId());
            log.info(LogUtil.devLog("语音房麦下送礼物-familyIncome-{} Gift{id={} price={} quantity={}} {} * {}=[{}] newBalance={}"),
                    roomFamilyId, result.getGiftId(), result.getUnitPrice(), result.getQuantity(), result.getUnitPrice(), giftFamilyRatio, receiveAmount, balance);
            if (balance == null) {
                throw new RuntimeException(StrUtil.format("发送礼物失败，返到家族钱包操作失败 {} {}",
                        JSON.toJSONString(bo),
                        JSON.toJSONString(result)));
            }
        });
    }
}
