package com.starbuds.server.service.finance.provider.lottery;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableList;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.activity.bo.DivideCakeWinInfo;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.dao.LotteryGameUserQualificationHisDAO;
import com.starbuds.server.service.finance.pojo.bo.lottery.LotteryGameAwardItemBO;
import com.starbuds.server.service.finance.provider.LotteryGameWinHisProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.service.finance.util.SpringContextUtil.isDev;

/**
 * 抽奖游戏-中奖结果-瓜分蛋糕
 *
 * @author aoe
 * @date 2021/11/27
 */
@Slf4j
@Component
public class DivideCakeWinResultProvider implements LotteryGameWinResult {

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    @Reference(version = "1.0.0")
    private LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Resource
    private LotteryGameWinHisProvider lotteryGameWinHisProvider;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<LotteryGameAwardItemBO> doLottery(long snapId) {
        if (isDev()) {
            log.info("抽奖游戏-中奖结果-瓜分蛋糕 开始抽奖 snapId={}", snapId);
        }
        List<LotteryGameUserQualificationHisDO> hisList = lotteryGameUserQualificationHisDAO.selectListBySnapId(snapId, BoolIntVal.True);
        if (isDev()) {
            log.info("抽奖游戏-中奖结果-瓜分蛋糕 抽奖结果 snapId={} hisList={}", snapId, LogUtil.obj2JsonObj(hisList));
        }
        if (CollectionUtils.isEmpty(hisList)) {
            log.info("抽奖游戏-中奖结果-瓜分蛋糕-没有人中奖：没有人获得抽奖资格 snapId={}", snapId);
            return Collections.emptyList();
        }

        long createTime = System.currentTimeMillis();
        List<LotteryGameWinHisDO> winHisList = hisList.stream()
                .filter(Objects::nonNull)
                .map(his -> {
                    Long userId = his.getUserId();
                    return getWinHis(snapId, his.getHisId(), userId, createTime);
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        if (isDev()) {
            log.info("抽奖游戏的中奖记录-1 snapId={} {}", snapId, LogUtil.obj2JsonObj(winHisList));
        }
        if (CollectionUtils.isEmpty(winHisList)) {
            log.info("没有人中奖 snapId={} 参与抽奖人数={}", snapId, hisList.size());
            return Collections.emptyList();
        }

        winHisList = mergeWinHisList(winHisList);
        final Map<Long, String> giftNameMap = getGiftNameMap(getItemIds(winHisList, LotteryGameRewardTypeEnum.GIFT));
        final Map<Long, String> propsNameMap = getPropsNameMap(getItemIds(winHisList, LotteryGameRewardTypeEnum.PROPS));
        winHisList = winHisList.stream()
                .map(win -> addRewardName(win, giftNameMap, propsNameMap))
                .collect(Collectors.toList());
        if (isDev()) {
            log.info("抽奖游戏-瓜分蛋糕的中奖记录-2 snapId={} {}", snapId, LogUtil.obj2JsonObj(winHisList));
        }
        boolean createOk = lotteryGameWinHisProvider.createBatch(winHisList);
        if (!createOk) {
            log.error("抽奖游戏-中奖结果-瓜分蛋糕-保存中奖信息失败 snapId=" + snapId + " winHisList=" + JSON.toJSONString(winHisList));
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }

        List<LotteryGameAwardItemBO> results = winHisList.stream()
                .map(winHis -> {
                    LotteryGameAwardItemBO bo = new LotteryGameAwardItemBO();
                    BeanUtils.copyProperties(winHis, bo);
                    bo.setOrderId(winHis.getHisId());
                    return bo;
                }).collect(Collectors.toList());


        if (isDev()) {
            List<Long> userIds = ImmutableList.of(141075591200768L);
            new Thread(() -> {
                for (int i = 0; i < 10; i++) {
                    try {
                        Thread.sleep(1000L);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    List<DivideCakeWinInfo> result = userIds.stream()
                            .map(userId -> lotteryGameService.getResultByDivideCake(userId, snapId))
                            .flatMap(win -> win.getWinInfos().stream())
                            .collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(result)) {
                        continue;
                    }
                    log.info("很久以后瓜分蛋糕中奖结果aaaaaa={} {}秒后 {}", snapId, (i + 1), LogUtil.devLog(LogUtil.obj2JsonObj(result)));
                    break;
                }
            }).start();
        }
        return results;
    }

    private List<LotteryGameWinHisDO> mergeWinHisList(List<LotteryGameWinHisDO> winHisList) {
        Map<String, LotteryGameWinHisDO> map = new HashMap<>();
        winHisList.stream()
                .forEach(po -> {
                    String key = getMergeKey(po);
                    LotteryGameWinHisDO mergePo = map.get(key);
                    if (mergePo != null) {
                        mergePo.setRewardNum(mergePo.getRewardNum() + po.getRewardNum());
                        mergePo.setRewardTotalPrice(mergePo.getRewardTotalPrice() + po.getRewardTotalPrice());
                    } else {
                        mergePo = po;
                    }
                    map.put(key, mergePo);
                });
        return map.values()
                .stream()
                .collect(Collectors.toList());
    }

    private String getMergeKey(LotteryGameWinHisDO po) {
        // snapId-playType-userId-rewardId-rewardType
        return String.format("%s-%s-%s-%s-%s", po.getSnapId(), po.getPlayType(), po.getUserId(), po.getRewardId(), po.getRewardType());
    }

    /**
     * 中奖记录
     *
     * @param snapId
     * @param hisId
     * @param userId
     * @return
     */
    private LotteryGameWinHisDO getWinHis(long snapId, Long hisId, Long userId, long createTime) {
        LotteryGameWinItemBO item = lotteryGameService.lottery(snapId);
        if (item == null) {
            return null;
        }
        LotteryGameWinHisDO his = new LotteryGameWinHisDO();
        his.setPlayType(LotteryGamePlayTypeEnum.DIVIDE_CAKE.value);
        his.setHisId(hisId);
        his.setUserId(userId);
        his.setSnapId(snapId);
        his.setRewardId(item.getRewardId());
        Long rewardPrice = Optional.ofNullable(item.getRewardPrice()).orElse(0L);
        his.setRewardPrice(rewardPrice);
        his.setRewardNum(item.getRewardNum());
        his.setRewardTotalPrice(rewardPrice * item.getRewardNum());
        his.setRewardType(item.getRewardType());
        his.setCreateTime(createTime);
        return his;
    }

    /**
     * 添加奖品名称
     *
     * @param winHis
     * @param giftNameMap
     * @param propsNameMap
     * @return
     */
    private LotteryGameWinHisDO addRewardName(LotteryGameWinHisDO winHis, Map<Long, String> giftNameMap,
                                              Map<Long, String> propsNameMap) {
        LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(winHis.getRewardType());
        Long rewardId = winHis.getRewardId();
        String rewardName = null;
        switch (type) {
            case GIFT:
                rewardName = giftNameMap.get(rewardId);
                break;
            case PROPS:
                rewardName = propsNameMap.get(rewardId);
                break;
        }
        rewardName = rewardName == null ? "" : rewardName;
        winHis.setRewardName(rewardName);
        return winHis;
    }

    /**
     * 礼物名称Map
     *
     * @param ids
     * @return
     */
    private Map<Long, String> getGiftNameMap(List<Long> ids) {
        Map<Long, String> result = new HashMap<>();
        Map<Long, GiftDO> map = giftService.getGiftMapByGiftIds(ids);
        map.forEach((k, v) -> {
            GiftDO gift = map.get(k);
            if (gift == null) {
                return;
            }
            result.put(k, Optional.ofNullable(gift.getGiftName()).orElse(""));
        });
        return result;
    }

    /**
     * 礼物名称Map
     *
     * @param ids
     * @return
     */
    private Map<Long, String> getPropsNameMap(List<Long> ids) {
        List<PropsVO> propsList = propsService.getPropsByIds(ids);
        if (CollectionUtils.isEmpty(propsList)) {
            return Collections.emptyMap();
        }
        return propsList.stream()
                .filter(Objects::nonNull)
                .filter(props -> props.getPropsId() != null)
                .filter(props -> StringUtils.isNotBlank(props.getPropsName()))
                .collect(Collectors.toMap(PropsVO::getPropsId, PropsVO::getPropsName));
    }

    /**
     * 获得奖品id
     *
     * @param winItems
     * @param typeEnum
     * @return
     */
    private List<Long> getItemIds(List<LotteryGameWinHisDO> winItems, LotteryGameRewardTypeEnum typeEnum) {
        return winItems.stream()
                .filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == typeEnum)
                .map(item -> item.getRewardId())
                .distinct()
                .collect(Collectors.toList());
    }
}
