package com.starbuds.server.service.finance.provider.lottery;

import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.pojo.bo.lottery.LotteryGameAwardItemBO;
import com.starbuds.server.service.finance.provider.BagProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;

/**
 * 抽奖游戏-发放奖品-礼物
 *
 * @author aoe
 * @date 2021/11/27
 */
@Slf4j
@Component
public class LotteryGameAwardGiftProvider implements LotteryGameAward {

    @Resource
    private BagProvider bagProvider;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int awardItems(final Collection<LotteryGameAwardItemBO> items, final LotteryGamePlayTypeEnum playType) {
        if (CollectionUtils.isEmpty(items)) {
            log.info("抽奖游戏-发放奖品-礼物 items is empty");
            return 0;
        }

        Collection<QueueAddItemToBagMsg> msgList = lotteryGameProvider.toQueueAddItemToBagMsgList(items, playType, -1);
        if (CollectionUtils.isEmpty(items)) {
            log.info("抽奖游戏-发放奖品-礼物 msgList is empty items=" + LogUtil.obj2JsonObj(items));
            return 0;
        }

        bagProvider.addItemsToBagThenSendImMsgStatQueueMsg(msgList);
        return (int) msgList.stream()
                .count();
    }
}