package com.starbuds.server.service.finance.provider.lottery;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGameCostTypeEnum;
import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatCommonBizSourceEnum;
import com.starbuds.server.common.pojo.activity.params.LotteryGameCostItemParam;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeCostForkMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeGetForkMsg;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Objects;

import static com.starbuds.server.common.constant.statistics.StatCommonBizSourceEnum.ANNIVERSARY_DIVIDE_CAKE_GET_FORK;

/**
 * 抽奖礼物统计数据
 *
 * @author aoe
 * @date 2021/12/24
 */
@Slf4j
@Component
public class LotteryGameStatisticsProvider {

    @Resource
    private RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    public void sendGetForkMessages(Long userId, Long itemId, Integer itemType, Integer quantity, Long orderId) {
        BagItemTypeEnum bagItemTypeEnum = BagItemTypeEnum.toEnum(itemType);
        if (BagItemTypeEnum.PROPS != bagItemTypeEnum) {
            return;
        }
        if (!isForkId(itemId)) {
            return;
        }
        QueueDivideCakeGetForkMsg msg = new QueueDivideCakeGetForkMsg();
        msg.setUserId(userId);
        msg.setCount(quantity);
        msg.setSource(ANNIVERSARY_DIVIDE_CAKE_GET_FORK);
        msg.setOrderId(orderId);
        sendDivideCakeGetForkMsg(msg);
    }

    /**
     * 发送周年庆-瓜分蛋糕-用户获得餐叉消息
     *
     * @param addItemToBagMessages
     */
    @Deprecated
    public void sendGetForkMessages(Collection<QueueAddItemToBagMsg> addItemToBagMessages) {
        if (CollectionUtils.isEmpty(addItemToBagMessages)) {
            return;
        }
        addItemToBagMessages.stream()
                .filter(Objects::nonNull)
                .filter(addItemToBagMsg -> isForkId(addItemToBagMsg.getItemId()))
                .map(this::toGetForkMsg)
                .filter(Objects::nonNull)
                .forEach(this::sendDivideCakeGetForkMsg);
    }

    /**
     * 发送用户消耗餐叉的消息
     *
     * @param costItemParam
     */
    public void sendCostForkMsg(LotteryGameCostTypeEnum costType, LotteryGameCostItemParam costItemParam) {
        if (LotteryGameCostTypeEnum.PROPS != costType) {
            return;
        }
        Long itemId = costItemParam.getItemId();
        if (!isForkId(itemId)) {
            return;
        }
        QueueDivideCakeCostForkMsg msg = new QueueDivideCakeCostForkMsg();
        msg.setUserId(costItemParam.getUserId());
        msg.setCount(costItemParam.getQuantity());
        msg.setOrderId(costItemParam.getOrderId());
        msg.setSource(StatCommonBizSourceEnum.ANNIVERSARY_DIVIDE_CAKE_ONE_USER_COST_FORK_COUNT);
        boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.DIVIDE_CAKE_COST_FORK, msg.serialize());
        String msgStr = JSON.toJSONString(msg);
        if (sendMsg) {
            log.info("发送用户消耗餐叉的消息成功 {}", msgStr);
            return;
        }
        log.error("发送用户消耗餐叉的消息失败 " + msgStr);
    }

    private void sendDivideCakeGetForkMsg(QueueDivideCakeGetForkMsg msg) {
        boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.DIVIDE_CAKE_GET_FORK, msg.serialize());
        String msgStr = JSON.toJSONString(msg);
        if (sendMsg) {
            log.info("发送用户获得餐叉的消息成功 {}", msgStr);
            return;
        }
        log.error("发送用户消获得叉的消息失败 " + msgStr);
    }

    private QueueDivideCakeGetForkMsg toGetForkMsg(QueueAddItemToBagMsg addItemToBagMsg) {
        StatCommonBizSourceEnum statCommonBizSourceEnum = ANNIVERSARY_DIVIDE_CAKE_GET_FORK;
        if (statCommonBizSourceEnum == null) {
            return null;
        }
        QueueDivideCakeGetForkMsg msg = new QueueDivideCakeGetForkMsg();
        msg.setUserId(addItemToBagMsg.getUserId());
        msg.setCount(addItemToBagMsg.getQuantity());
        msg.setOrderId(addItemToBagMsg.getOrderId());
        msg.setSource(statCommonBizSourceEnum);
        return msg;
    }

    /*private StatCommonBizSourceEnum toStatCommonBizSourceEnum(AddItemToBagSourceEnum source) {
        StatCommonBizSourceEnum statSource = null;
        switch (source) {
            case RED_BAG:
                statSource = StatCommonBizSourceEnum.ANNIVERSARY_DIVIDE_CAKE_ONE_USER_GET_FORK_COUNT;
                break;
            case DAILY_RECHARGE_TOTAL_TO_1000:
                statSource = StatCommonBizSourceEnum.DAILY_RECHARGE_TOTAL_TO_1000;
                break;
            case DAILY_RECHARGE_98_MUST_GET:
                statSource = StatCommonBizSourceEnum.DAILY_RECHARGE_98_MUST_GET;
                break;
        }
        return statSource;
    }*/

    /**
     * 是否餐叉id
     *
     * @param itemId
     * @return true:餐叉id
     */
    private boolean isForkId(Long itemId) {
        if (itemId == null) {
            return false;
        }
        String forkIdStr = releaseConfigService.getCachReleaseConfig("forkId");
        if (StringUtils.isBlank(forkIdStr)) {
            return false;
        }
        long forkId = NumberUtils.toLong(forkIdStr);
        if (forkId - itemId != 0) {
            return false;
        }
        return true;
    }
}
