package com.starbuds.server.service.finance.provider.lottery.cost;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.activity.params.LotteryGameCostItemParam;
import com.starbuds.server.service.finance.provider.IBagProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 抽奖游戏消耗物品-道具
 *
 * @author aoe
 * @date 2021/12/7
 */
@Component
@Slf4j
public class LotteryGameCostProps implements LotteryGameCostItem {

    @Resource
    private IBagProvider bagProvider;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean costOneItem(LotteryGameCostItemParam costItem) {
        checkCostItem(costItem);
        Integer quantity = costItem.getQuantity();
        Integer reduce = bagProvider.reduceQuantity(costItem.getUserId(), costItem.getItemId(),
                BagItemTypeEnum.PROPS.getValue(),
                quantity,
                String.format("参与%s消耗%s个", costItem.getLotteryGamePlayType().desc, quantity),
                costItem.getOrderId());
        if (reduce == null || reduce < 0) {
            log.warn("扣除道具失败 reduce={} costItem={}", reduce, JSON.toJSON(costItem));
            return false;
        }
        return true;
    }

    /**
     * 检查参数
     *
     * @param costItem
     */
    private void checkCostItem(LotteryGameCostItemParam costItem) {
        Preconditions.checkNotNull(costItem);
        Preconditions.checkNotNull(costItem.getItemId());
        Preconditions.checkNotNull(costItem.getLotteryGamePlayType());
        Preconditions.checkNotNull(costItem.getQuantity());
        Preconditions.checkNotNull(costItem.getUserId());
        Preconditions.checkNotNull(costItem.getOrderId());
    }
}
