package com.starbuds.server.service.finance.provider.lottery.cost;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.finance.dao.LotteryGameUserQualificationHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 抽奖活动中用户获取的抽奖资格
 */
@Component
@Slf4j
public class LotteryGameUserQualificationHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameUserQualificationHisDO create(LotteryGameUserQualificationHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = lotteryGameUserQualificationHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖活动中用户获取的抽奖资格创建成功 {}", json);
            return po;
        }
        log.warn("抽奖活动中用户获取的抽奖资格创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<LotteryGameUserQualificationHisDO> list) {
        boolean flag = lotteryGameUserQualificationHisDAO.insertBatch(list) > 0;
        return flag;
    }

    public int getCount(long snapId, long userId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("snapId", snapId);
        qw.eq("userId", userId);
        return lotteryGameUserQualificationHisDAO.selectCount(qw);
    }
}
