package com.starbuds.server.service.finance.rocket;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.PropertyValueConst;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.service.finance.rocket.listener.DelayMsgListener;
import com.starbuds.server.service.finance.rocket.listener.NormalMsgListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Class
 *
 * @author Lay
 * @date 2019/05/30
 */
@Component
@Slf4j
public class RocketConsumer implements DisposableBean {
    @Resource
    RocketConfig rocketConfig;

    @Resource
    NormalMsgListener normalMsgListener;

    @Resource
    DelayMsgListener delayMsgListener;

    private Consumer consumer;

    @Override
    public void destroy() throws Exception {
        if (consumer != null) {
            consumer.shutdown();
        }
    }

    public void start() {
        log.info("RocketMQ 消费者初始化");
        Properties properties = rocketConfig.getProps();

        // 集群模式订阅，共同分担消息处理
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.CLUSTERING);
        consumer = ONSFactory.createConsumer(properties);

        log.info("RocketMQ 消费者启动");
        consumer.subscribe(RocketTopic.TPC_NORMAL, getSubscribeTags(Arrays.asList(
                RocketTag.LOTTERY_GIFT_REWARDS,
                RocketTag.LOTTERY_GIFT_REWARDS_RTC,
                RocketTag.LOTTERY_GIFT_REWARDS_VOICE_CHAT,
                RocketTag.LOTTERY_GIFT_REWARDS_PRIVATE_CHAT,
                RocketTag.ANCHOR_WITHDRAW,
                RocketTag.USER_WITHDRAW,
                RocketTag.TRANSFER_TO_USER,
                RocketTag.SKILL_ORDER_CREATE,
                RocketTag.SKILL_ORDER_PAID,
                RocketTag.LOTTERY_GAME_LOTTERY_END,
                RocketTag.ADD_ITEM_TO_BAG,
                RocketTag.REDUCE_ITEM_TO_BAG
        )), normalMsgListener);
        consumer.subscribe(RocketTopic.TPC_DELAY, getSubscribeTags(Arrays.asList(
                RocketTag.SKILL_ORDER_PAY_TIMEOUT,
                RocketTag.SKILL_SERVICE_WAIT_START,
                RocketTag.SKILL_SERVICE_END_CHECK,
                RocketTag.SKILL_SERVICE_RECEIVE_CHECK
        )), delayMsgListener);
        consumer.start();
    }

    private String getSubscribeTags(List<String> tags) {
        int size = tags.size();
        StringBuilder sbTags = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sbTags.append(tags.get(i));
            if (i != size - 1) {
                sbTags.append("||");
            }
        }
        return sbTags.toString();
    }
}
