package com.starbuds.server.service.finance.rocket.handler;

import com.starbuds.server.common.pojo.mq.QueueSkillEndCheckMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillOrderPayTimeoutMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillReceiveCheckMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillStartMsg;
import com.starbuds.server.service.finance.provider.SkillOrderProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/22 18:17
 * @Description 延迟消息处理器
 */
@Slf4j
@Component
public class DelayMsgHandler {

    @Resource
    SkillOrderProvider skillOrderProvider;

    /**
     * 处理订单超时
     *
     * @param msg
     */
    public void handleSkillOrderPayTimeout(QueueSkillOrderPayTimeoutMsg msg) {
        log.info("处理订单超时检测 {}", msg.getOrderId());
        if (skillOrderProvider.isWaitPay(msg.getOrderId())) {
            log.info("订单{}已超时未支付", msg.getOrderId());
            skillOrderProvider.cancelOrder(msg.getOrderId());
        }
    }

    /**
     * 处理服务等待开始
     *
     * @param msg
     */
    public void handleSkillServiceWaitStart(QueueSkillStartMsg msg) {
        log.info("处理服务开始 {}", msg.getOrderId());
        if (skillOrderProvider.isStartService(msg.getOrderId())) {
            log.info("订单{}需开始", msg.getOrderId());
            skillOrderProvider.doStartService(msg.getOrderId(), msg.getRequireTime());
        }
    }

    /**
     * 处理服务结束检测
     *
     * @param msg
     */
    public void handleSkillServiceEndCheck(QueueSkillEndCheckMsg msg) {
        log.info("处理服务结束 {}", msg.getOrderId());
        if (!skillOrderProvider.isOrderEnd(msg.getOrderId())) {
            log.info("订单{}需结束", msg.getOrderId());
            skillOrderProvider.doConfirmService(msg.getOrderId(), msg.getEndTime());
        }
    }

    /**
     * 处理服务待接单超时检测
     *
     * @param msg
     */
    public void handleSkillServiceReceiveCheck(QueueSkillReceiveCheckMsg msg) {
        log.info("处理待接单超时 {}", msg.getOrderId());
        if (skillOrderProvider.isWaitReceive(msg.getOrderId())) {
            log.info("订单{}已超时接单", msg.getOrderId());
            skillOrderProvider.stopOrder(msg.getOrderId(), true);
        }
    }

}
