package com.starbuds.server.service.finance.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftSourceTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.message.push.PushActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.pojo.activity.LotteryHisRewardVO;
import com.starbuds.server.common.pojo.db.GiftAirHisDO;
import com.starbuds.server.common.pojo.db.PushTaskDO;
import com.starbuds.server.common.pojo.finance.LuckyGiftLotteryConfig;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.message.im.activity.WinItem;
import com.starbuds.server.common.pojo.message.im.activity.WinningRecordParam;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.rtc.RtcRoomGiftHis;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.GiftRankService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.biz.LotteryGameImMsgService;
import com.starbuds.server.common.service.rtc.RtcRoomGiftHisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.task.PushService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.exception.RtcRatioException;
import com.starbuds.server.service.finance.lucky.LuckyGiftProvider;
import com.starbuds.server.service.finance.provider.*;
import com.starbuds.server.service.finance.provider.lottery.LotteryGameProvider;
import com.starbuds.server.service.finance.provider.ratio.RtcRatioProvider;
import com.starbuds.server.service.finance.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkState;
import static com.starbuds.server.common.utils.DateUtil.getTimestamp;
import static com.starbuds.server.service.finance.util.SpringContextUtil.isDev;

/**
 * 普通消息处理
 *
 * @author Lay
 * @date 2020/6/5
 */
@Component
@Slf4j
public class NormalMsgHandler {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Resource
    SettleProvider settleProvider;

    @Resource
    LuckyGiftProvider luckyGiftProvider;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    WalletProvider walletProvider;

    @Resource
    FamilyWalletProvider familyWalletProvider;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    PushService pushService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    GiftRankService giftRankService;
    @Reference(version = "1.0.0")
    RtcRoomGiftHisService rtcRoomGiftHisService;

    @Value("${gift.noFamilyUserRatio}")
    private double giftNoFamilyUserRatio;

    @Value("${gift.familyUserRatio}")
    private double giftFamilyUserRatio;

    @Resource
    private CacheProvider cacheProvider;

    @Resource
    private RtcRatioProvider rtcRatioProvider;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Resource
    private BagProvider bagProvider;

    /**
     * 处理抽奖礼物奖项
     *
     * @param msg
     */
    public void handleLotteryGiftRewards(QueueLotteryGiftRewardsMsg msg) {
        Map<Long, Integer> targetGifts = msg.getGiftRewards();

        for (Map.Entry<Long, Integer> targetGift : targetGifts.entrySet()) {
            Long giftId = targetGift.getKey();
            Integer quantity = targetGift.getValue();
            Long receiverUserId = msg.getTargetUserId();
            Long senderUserId = msg.getUserId();

            Long orderId = identityService.getId();
            UserCardVO anchor = userService.getCard(receiverUserId);
            GiftPayItem giftPayItem = giftService.getPayItem(giftId);
            if (anchor != null && giftPayItem != null) {

                Long amount = giftPayItem.getGiftPrice() * quantity;
                Long anchorAmount = Long.valueOf(amount);

                LuckyGiftLotteryConfig lotteryConfig = luckyGiftProvider.getConfig();
                if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                    if (lotteryConfig == null) {
                        log.error("幸运礼物未配置");
                        return;
                    }

                    anchorAmount = BigDecimal.valueOf(amount).multiply(new BigDecimal(String.valueOf(lotteryConfig.getAnchorRatio()))).setScale(0, RoundingMode.DOWN).longValue();
                }

                // 异步主播结算
                settleProvider.settleByGift(giftPayItem, receiverUserId, senderUserId, quantity, anchorAmount, orderId);

                // 数量足够
                QueueGiftPaidMsg queueGiftPaidMsg = new QueueGiftPaidMsg();
                queueGiftPaidMsg.setOrderId(orderId);
                queueGiftPaidMsg.setGiftId(giftId);
                queueGiftPaidMsg.setGiftPrice(giftPayItem.getGiftPrice());
                queueGiftPaidMsg.setGiftPriceType(giftPayItem.getGiftPriceType());
                queueGiftPaidMsg.setQuantity(quantity);
                queueGiftPaidMsg.setAnchorAmount(anchorAmount);
                queueGiftPaidMsg.setSenderUserId(senderUserId);
                queueGiftPaidMsg.setReceiverUserId(receiverUserId);
                queueGiftPaidMsg.setFromBag(true);
                queueGiftPaidMsg.setChannelId(msg.getChannelId());
                queueGiftPaidMsg.setGiftSceneType(msg.getGiftSceneType());
                queueGiftPaidMsg.setChannelName(msg.getChannelName());
                queueGiftPaidMsg.setSourceType(GiftSourceTypeEnum.LOTTERY_GIFT.getValue());

                // 幸运礼物异步抽奖
                if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftPayItem.getGiftType())) {
                    luckyGiftProvider.asyncLottery(queueGiftPaidMsg);
                } else {
                    // 丢出礼物赠送消息到消息队列
                    rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.GIFT_PAID, queueGiftPaidMsg.serialize());
                }
            }
        }
    }

    /**
     * 处理礼物抽奖奖项（语聊）
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleLotteryGiftRewardsRtc(QueueLotteryGiftRewardsRtcMsg msg) throws RtcRatioException {
        Map<Long, List<LotteryHisRewardVO>> targetUserRewardMap = msg.getTargetUserRewardMap();

        Map<Long, Integer> targetUserSeats = msg.getTargetUserSeats();
        Long senderUserId = msg.getUserId();
        UserCardVO sender = userService.getCard(senderUserId);
        Set<Long> giftIds = targetUserRewardMap.values().stream().flatMap(Collection::stream).map(LotteryHisRewardVO::getRewardId).collect(Collectors.toSet());
        Map<Long, GiftItemVO> giftItemMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIds)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));

        List<QueueRtcGiftPaidMsg> giftPaidMsgList = new ArrayList<>();

        // 房间家族
//        Long roomFamilyId = rtcRoomService.getFamilyId(msg.getRoomId());
        RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(msg.getRoomId());
        Long roomFamilyId = rtcRatioProvider.getFamilyId(rtcRoomRatio);
        // 用户家族map
        Map<Long, Long> userFamilyMap = familyService.getFamilyIds(targetUserSeats.keySet());
        log.info("userFamilyMap {}", userFamilyMap);

        // 欧气榜
        List<GiftAirHisDO> airs = new ArrayList<>();
        long curTime = System.currentTimeMillis();

        for (Map.Entry<Long, List<LotteryHisRewardVO>> entity : targetUserRewardMap.entrySet()) {
            Long receiverUserId = entity.getKey();
            int roomFamilyUserCount = 0;
            for (LotteryHisRewardVO hisReward : entity.getValue()) {
                Long orderId = hisReward.getHisId();
                GiftItemVO giftItem = giftItemMap.get(hisReward.getRewardId());
                if (giftItem == null) {
                    log.error("处理礼物抽奖奖项（语聊）失败：无效的礼物id " + hisReward.getRewardId());
                    continue;
                }

                if (WalletTypeEnum.STAR_DIAMOND.getValue() == giftItem.getGiftPriceType()) {
                    Integer quantity = hisReward.getQuantity();
                    /*// 用户实际分成比例
                    BigDecimal ratio = null;
                    // 用户家族
                    Long familyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                    // 是否设置分成比例
                    Double userRatio = rtcRoomService.getUserRatio(msg.getRoomId());
                    if (userRatio != null) {
                        ratio = Convert.toBigDecimal(userRatio);
                    } else {
                        ratio = Convert.toBigDecimal(giftNoFamilyUserRatio);
                        // 用户家族
                        if (roomFamilyId != null && roomFamilyId.equals(familyId)) {
                            ratio = Convert.toBigDecimal(giftFamilyUserRatio);
                            roomFamilyUserCount++;
                        }
                    }*/
                    // 用户家族
                    Long userFamilyId = Optional.ofNullable(userFamilyMap).map(e -> e.get(receiverUserId)).orElse(null);
                    if (roomFamilyId != null && roomFamilyId.equals(userFamilyId)) {
                        roomFamilyUserCount++;
                    }
                    // 用户实际分成比例
                    BigDecimal userRatio = rtcRatioProvider.getUserRatio(userFamilyId, rtcRoomRatio);
                    rtcRatioProvider.checkRatio(userRatio);
                    log.info("roomFamilyId {} familyId {} ratio {} receiverUserId {}", roomFamilyId, userFamilyId, userRatio, receiverUserId);

                    String remark = String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftItem.getGiftName());
                    Long dustAmount = BigDecimal.valueOf(giftItem.getGiftPrice()).multiply(BigDecimal.valueOf(quantity))
                            .multiply(userRatio).setScale(0, RoundingMode.DOWN).longValue();
                    walletProvider.addBalance(WalletTypeEnum.STAR_DUST, receiverUserId, dustAmount, remark, orderId);

                    //增加礼物记录
                    RtcRoomGiftHis rmh = new RtcRoomGiftHis();
                    rmh.setId(identityService.getId());
                    rmh.setUserId(senderUserId);
                    rmh.setTargetUserId(receiverUserId);
                    rmh.setGiftId(giftItem.getGiftId());
                    rmh.setGiftType(StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue());
                    rmh.setTargetUserfamilyId(userFamilyId == null ? 0L : userFamilyId);
                    rmh.setRoomFamilyId(roomFamilyId == null ? 0L : roomFamilyId);
                    rmh.setRoomId(msg.getRoomId());
                    rmh.setOrderId(orderId);
                    rmh.setPrice(dustAmount);
                    rmh.setRemark(remark);
                    setInsertTime(rmh);
                    rtcRoomGiftHisService.insertRtcRoomGiftHis(rmh);


                    // 是否家族房间
                    if (roomFamilyId != null) {
//                        BigDecimal giftFamilyRatio = cacheProvider.familyRatio(roomFamilyId);
                        BigDecimal giftFamilyRatio = rtcRatioProvider.getFamilyRatio(rtcRoomRatio);
                        rtcRatioProvider.checkRatio(userRatio, giftFamilyRatio);
                        BigDecimal presidentAmount = BigDecimal.valueOf(giftItem.getGiftPrice())
                                .multiply(BigDecimal.valueOf(quantity))
                                .multiply(giftFamilyRatio).setScale(2, RoundingMode.DOWN);
                        // 返到家族钱包
                        familyWalletProvider.addBalance(roomFamilyId, presidentAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftItem.getGiftName()), orderId);

                        // 处理礼物抽奖奖项（语聊）
                        log.info(LogUtil.devLog("处理礼物抽奖奖项-语聊-familyIncome-{} Gift{id={} price={} quantity={}} {} * {} * {}=[{}]"),
                                roomFamilyId, giftItem.getGiftId(), giftItem.getGiftPrice(), quantity, giftItem.getGiftPrice(), quantity, giftFamilyRatio, presidentAmount);
                    }

                    // 构造欧气榜
                    if (giftItem.getGiftPrice() >= 10000) {
                        GiftAirHisDO air = new GiftAirHisDO();
                        air.setGiftId(hisReward.getRewardId());
                        air.setSenderUserId(senderUserId);
                        air.setReceiveUserId(receiverUserId);
                        air.setQuantity(Convert.toLong(quantity));
                        air.setCreateTime(curTime);
                        airs.add(air);
                    }

                    // 构造消息
                    Map<Long, Integer> userSeatMap = new HashMap<>();
                    userSeatMap.put(receiverUserId, targetUserSeats.get(receiverUserId));
                    QueueRtcGiftPaidMsg rtcGiftPaidMsg = new QueueRtcGiftPaidMsg();
                    rtcGiftPaidMsg.setOrderId(orderId);
                    rtcGiftPaidMsg.setGiftId(giftItem.getGiftId());
                    rtcGiftPaidMsg.setGiftType(giftItem.getGiftType());
                    rtcGiftPaidMsg.setGiftPrice(giftItem.getGiftPrice());
                    rtcGiftPaidMsg.setGiftPriceType(giftItem.getGiftPriceType());
                    rtcGiftPaidMsg.setQuantity(quantity);
                    rtcGiftPaidMsg.setSenderUserId(senderUserId);
                    rtcGiftPaidMsg.setTargetUserSeat(userSeatMap);
                    rtcGiftPaidMsg.setFromBag(false);
                    rtcGiftPaidMsg.setChannelId(msg.getChannelId());
                    rtcGiftPaidMsg.setRoomId(msg.getRoomId());
                    rtcGiftPaidMsg.setAllSeat(BoolIntVal.False);
                    rtcGiftPaidMsg.setRoomFamilyUserCount(roomFamilyUserCount);
                    rtcGiftPaidMsg.setFromLottery(true);
                    rtcGiftPaidMsg.setRoomBusinessType(rtcRoomRatio.getBusinessType());
                    giftPaidMsgList.add(rtcGiftPaidMsg);
                }
            }
        }

        // 写入欧气榜
        if (CollectionUtil.isNotEmpty(airs)) {
            giftRankService.addGiftAirRank(airs);
        }

        int sendSize = giftPaidMsgList.size();
        List<QueueRtcGiftPaidMsg> sendFailList = new ArrayList<>(sendSize);
        for (QueueRtcGiftPaidMsg rtcGiftPaidMsg : giftPaidMsgList) {
            // 丢出礼物赠送消息到消息队列
            boolean send = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_GIFT_PAID, rtcGiftPaidMsg.serialize());
            if (Boolean.FALSE.equals(send)) {
                sendFailList.add(rtcGiftPaidMsg);
            }
        }

        if (!CollectionUtils.isEmpty(sendFailList)) {
            String errMsg = "丢出礼物赠送消息到消息队列失败 total=%s failCount=%s failList=%s";
            if (isDev()) {
                log.error(LogUtil.devLog(String.format(errMsg, sendSize, sendFailList.size(), JSON.toJSONString(sendFailList, true))));
            } else {
                log.error(String.format(errMsg, sendSize, sendFailList.size(), JSON.toJSONString(sendFailList)));
            }
        }
    }

    /**
     * 处理主播提现
     *
     * @param msg
     */
    public void handleAnchorWithdraw(QueueAnchorWithdrawMsg msg) {

        Long userId = msg.getUserId();
        BigDecimal amount = msg.getAmount();

        PushTaskDO taskDO = new PushTaskDO();
        taskDO.setTaskId(identityService.getId());
        taskDO.setTaskName("系统消息-主播星币转出");
        taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
        taskDO.setTaskCreateTime(System.currentTimeMillis());
        taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
        taskDO.setTargetValue(userId.toString());
        taskDO.setAdminId(0L);
        taskDO.setRemark("系统自动推送");
        taskDO.setPushTitle("星币转出");

        PushMsgParam pushParam = new PushMsgParam();
        pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
        pushParam.setModuleType(ModuleTypeEnum.WALLET.getDesc());
        pushParam.setModuleId(userId.toString());

        taskDO.setExtParameters(JSON.toJSONString(pushParam));

        if (msg.isSuccess()) {
            statPlatformService.addPlatformStat(Optional.ofNullable(userService.getChannelId(userId)).orElse(0L), StatPlatformTypeEnum.WITHDRAW_ANCHOR, amount);
            taskDO.setPushBody("您的星币转出审核通过>>>");
        } else {
            String remark = msg.getRemark();
            if (StringUtils.isNotEmpty(remark) && remark.length() > 20) {
                remark = remark.substring(0, 20) + "...";
            }
            taskDO.setPushBody("您的星币转出失败！" + remark);
        }

        pushService.addPushTask(taskDO);

    }

    /**
     * 处理用户提现
     *
     * @param msg
     */
    public void handleUserWithdraw(QueueUserWithdrawMsg msg) {
        log.info("处理用户提现，{}", JSON.toJSONString(msg));
        Long userId = msg.getUserId();
        BigDecimal amount = msg.getAmount();

        PushTaskDO taskDO = new PushTaskDO();
        taskDO.setTaskId(identityService.getId());
        taskDO.setTaskName("系统消息-用户粉钻转出");
        taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
        taskDO.setTaskCreateTime(System.currentTimeMillis());
        taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
        taskDO.setTargetValue(userId.toString());
        taskDO.setAdminId(0L);
        taskDO.setRemark("系统自动推送");
        taskDO.setPushTitle("粉钻转出");

        PushMsgParam pushParam = new PushMsgParam();
        pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
        pushParam.setModuleType(ModuleTypeEnum.WALLET.getDesc());
        pushParam.setModuleId(userId.toString());

        taskDO.setExtParameters(JSON.toJSONString(pushParam));

        if (msg.isSuccess()) {
            statPlatformService.addPlatformStat(Optional.ofNullable(userService.getChannelId(userId)).orElse(0L), StatPlatformTypeEnum.WITHDRAW_USER, amount);
            taskDO.setPushBody("您的粉钻转出审核通过>>>");
        } else {
            String remark = msg.getRemark();
            if (StringUtils.isNotEmpty(remark) && remark.length() > 20) {
                remark = remark.substring(0, 20) + "...";
            }
            taskDO.setPushBody("您的粉钻转出失败！" + remark);
        }

        pushService.addPushTask(taskDO);

    }

    /**
     * 处理打款给用户（手机顶部消息弹框）
     *
     * @param msg
     */
    public void handleTransferToUser(QueueTransferUserMsg msg) {
        log.info("处理打款给用户，{}", JSON.toJSONString(msg));
        Long userId = msg.getUserId();

        PushTaskDO taskDO = new PushTaskDO();
        taskDO.setTaskId(identityService.getId());
        taskDO.setTaskName("系统消息-用户粉钻转出");
        taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
        taskDO.setTaskCreateTime(System.currentTimeMillis());
        taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
        taskDO.setTargetValue(userId.toString());
        taskDO.setAdminId(0L);
        taskDO.setRemark("系统自动推送");
        taskDO.setPushTitle("粉钻转出");
        taskDO.setPushBody("到账成功");

        PushMsgParam pushParam = new PushMsgParam();
        pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
        pushParam.setModuleType(ModuleTypeEnum.WALLET.getDesc());
        pushParam.setModuleId(userId.toString());

        taskDO.setExtParameters(JSON.toJSONString(pushParam));

        if (msg.isSuccess()) {
            taskDO.setPushBody("您的粉钻转出已到账，快去钱包看看吧>>>");
        }
        pushService.addPushTask(taskDO);

    }

    /**
     * 处理订单支付成功
     *
     * @param msg
     */
    public void handleSkillOrderPaid(QueueSkillOrderPaidMsg msg) {
        log.info("处理订单支付成功，{}", msg.getOrderId());

        // 推送通知
        PushTaskDO taskDO = new PushTaskDO();
        taskDO.setTaskId(identityService.getId());
        taskDO.setTaskName("系统消息-技能服务");
        taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
        taskDO.setTaskCreateTime(System.currentTimeMillis());
        taskDO.setTaskType(TaskTypeEnum.SINGLE_PUSH.getValue());
        taskDO.setTargetValue(Convert.toStr(msg.getReceiveUserId()));

        taskDO.setAdminId(0L);
        taskDO.setRemark("系统自动推送");
        taskDO.setPushTitle("您有一笔新的订单");
        String skillName = skillService.getSkillName(msg.getSkillId());

        String pushMsg = String.format("【%s*%d】订单计划%s开始", skillName, msg.getQuantity(), DateUtil.format(new Date(msg.getRequireTime()), "yyyy-MM-dd HH:mm"));
        taskDO.setPushBody(pushMsg);

        PushMsgParam pushParam = new PushMsgParam();
        pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
        pushParam.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
        pushParam.setModuleId(Convert.toStr(msg.getOrderId()));
        taskDO.setExtParameters(JSON.toJSONString(pushParam));
        pushService.addPushTask(taskDO);

        // 系统消息
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle("新订单通知");
        String body = String.format("你有一条【%s*%d】订单，订单计划%s开始，请尽快接单！", skillName, msg.getQuantity(), DateUtil.format(new Date(msg.getRequireTime()), "yyyy-MM-dd HH:mm"));
        systemMsg.setBody(body);
        systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
        systemMsg.setModuleType(ModuleTypeEnum.ORDER_DETAILS.getDesc());
        systemMsg.setModuleId(Convert.toStr(msg.getOrderId()));
        imService.sendCustomImMsg(null, msg.getReceiveUserId(), systemMsg, null);

        // 超时未接单检测消息
        QueueSkillReceiveCheckMsg checkMsg = new QueueSkillReceiveCheckMsg();
        checkMsg.setOrderId(msg.getOrderId());
        checkMsg.setRequireTime(msg.getRequireTime());
        checkMsg.setUserId(msg.getUserId());
        checkMsg.setReceiveUserId(msg.getReceiveUserId());
        rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SKILL_SERVICE_RECEIVE_CHECK, checkMsg.serialize(), msg.getTimestamp() + 30 * 60 * 1000L);
    }

    /**
     * 处理礼物抽奖奖项（语音速配）
     *
     * @param msg
     */
    public void handleLotteryGiftRewardsVoiceChat(QueueLotteryGiftRewardsVoiceChatMsg msg) {
        Long receiveUserId = msg.getReceiveUserId();
        Long senderUserId = msg.getUserId();
        UserCardVO sender = userService.getCard(senderUserId);
        Set<Long> giftIds = msg.getUserRewards().stream().map(LotteryHisRewardVO::getRewardId).collect(Collectors.toSet());
        Map<Long, GiftItemVO> giftItemMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIds)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));

        List<QueueVoiceGiftPaidMsg> giftPaidMsgList = new ArrayList<>();

        List<LotteryHisRewardVO> userRewards = msg.getUserRewards();
        for (LotteryHisRewardVO hisReward : userRewards) {
            GiftItemVO giftItem = giftItemMap.get(hisReward.getRewardId());
            if (WalletTypeEnum.STAR_DIAMOND.getValue() == giftItem.getGiftPriceType()) {
                Integer quantity = hisReward.getQuantity();
                Long dustAmount = BigDecimal.valueOf(giftItem.getGiftPrice()).multiply(BigDecimal.valueOf(quantity)).multiply(new BigDecimal("0.7")).setScale(0, RoundingMode.DOWN).longValue();
                walletProvider.addBalance(WalletTypeEnum.STAR_DUST, msg.getReceiveUserId(), dustAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftItem.getGiftName()), msg.getOrderId());

                // 构造消息
                QueueVoiceGiftPaidMsg giftPaidMsg = new QueueVoiceGiftPaidMsg();
                giftPaidMsg.setOrderId(msg.getOrderId());
                giftPaidMsg.setGiftId(giftItem.getGiftId());
                giftPaidMsg.setGiftType(giftItem.getGiftType());
                giftPaidMsg.setGiftPrice(giftItem.getGiftPrice());
                giftPaidMsg.setGiftPriceType(giftItem.getGiftPriceType());
                giftPaidMsg.setQuantity(quantity);
                giftPaidMsg.setSenderUserId(senderUserId);
                giftPaidMsg.setReceiveUserId(receiveUserId);
                giftPaidMsg.setFromBag(false);
                giftPaidMsg.setChannelId(msg.getChannelId());
                giftPaidMsg.setHisId(msg.getHisId());
                giftPaidMsgList.add(giftPaidMsg);

                for (QueueVoiceGiftPaidMsg item : giftPaidMsgList) {
                    // 丢出礼物赠送消息到消息队列
                    rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.VOICE_GIFT_PAID, item.serialize());
                }
            }
        }
    }

    /**
     * 处理礼物抽奖奖项（私聊）
     *
     * @param msg
     */
    public void handleLotteryGiftRewardsPrivateChat(QueueLotteryGiftRewardsPrivateChatMsg msg) {
        Long receiveUserId = msg.getReceiveUserId();
        Long senderUserId = msg.getUserId();
        UserCardVO sender = userService.getCard(senderUserId);
        Set<Long> giftIds = msg.getUserRewards().stream().map(LotteryHisRewardVO::getRewardId).collect(Collectors.toSet());
        Map<Long, GiftItemVO> giftItemMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIds)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));

        List<QueueBaseGiftPaidMsg> giftPaidMsgList = new ArrayList<>();

        List<LotteryHisRewardVO> userRewards = msg.getUserRewards();
        for (LotteryHisRewardVO hisReward : userRewards) {
            GiftItemVO giftItem = giftItemMap.get(hisReward.getRewardId());
            if (WalletTypeEnum.STAR_DIAMOND.getValue() == giftItem.getGiftPriceType()) {
                Integer quantity = hisReward.getQuantity();
                Long dustAmount = BigDecimal.valueOf(giftItem.getGiftPrice()).multiply(BigDecimal.valueOf(quantity)).multiply(new BigDecimal("0.6")).setScale(0, RoundingMode.DOWN).longValue();
                walletProvider.addBalance(WalletTypeEnum.STAR_DUST, msg.getReceiveUserId(), dustAmount, String.format("[%s](%d)赠送%d个%s", sender.getUserName(), senderUserId, quantity, giftItem.getGiftName()), msg.getOrderId());

                // 构造消息
                QueueBaseGiftPaidMsg giftPaidMsg = new QueueBaseGiftPaidMsg();
                giftPaidMsg.setOrderId(msg.getOrderId());
                giftPaidMsg.setGiftId(giftItem.getGiftId());
                giftPaidMsg.setGiftType(giftItem.getGiftType());
                giftPaidMsg.setGiftPrice(giftItem.getGiftPrice());
                giftPaidMsg.setGiftPriceType(giftItem.getGiftPriceType());
                giftPaidMsg.setQuantity(quantity);
                giftPaidMsg.setSenderUserId(senderUserId);
                giftPaidMsg.setReceiveUserId(receiveUserId);
                giftPaidMsg.setFromBag(false);
                giftPaidMsg.setChannelId(msg.getChannelId());
                giftPaidMsgList.add(giftPaidMsg);

                for (QueueBaseGiftPaidMsg item : giftPaidMsgList) {
                    // 丢出礼物赠送消息到消息队列
                    rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.PRIVATE_CHAT_GIFT_PAID, item.serialize());
                }
            }
        }
    }

    /**
     * 抽奖游戏结束
     *
     * @param msg
     */
    @Deprecated
    @Transactional(rollbackFor = Exception.class)
    public void handleLotteryGameEnd(QueueLotteryGameEndMsg msg) {
        log.info("收到抽奖游戏结束消息 {}", LogUtil.obj2JsonObj(msg));
        lotteryGameProvider.lotteryThenAward(msg.getSnapId(), msg.getPlayType());
    }

    private void setInsertTime(RtcRoomGiftHis rmh) {
        long timestamp = System.currentTimeMillis();
        rmh.setTimeY(getTimestamp(timestamp, 4L));
        rmh.setTimeM(getTimestamp(timestamp, 3L));
        rmh.setTimeD(getTimestamp(timestamp, 2L));
        rmh.setTimeH(getTimestamp(timestamp, 1L));
        rmh.setCreatetime(timestamp);
    }


}
