package com.starbuds.server.service.finance.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.QueueSkillEndCheckMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillOrderPayTimeoutMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillReceiveCheckMsg;
import com.starbuds.server.common.pojo.mq.QueueSkillStartMsg;
import com.starbuds.server.service.finance.rocket.handler.DelayMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/22 18:16
 * @Description 延迟消息监听器
 */
@Component
@Slf4j
public class DelayMsgListener implements MessageListener {

    @Resource
    DelayMsgHandler delayMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext context) {
        try {
            log.info("收到延迟队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.SKILL_ORDER_PAY_TIMEOUT:
                    delayMsgHandler.handleSkillOrderPayTimeout(JSON.parseObject(message.getBody(), QueueSkillOrderPayTimeoutMsg.class));
                    break;
                case RocketTag.SKILL_SERVICE_WAIT_START:
                    delayMsgHandler.handleSkillServiceWaitStart(JSON.parseObject(message.getBody(), QueueSkillStartMsg.class));
                    break;
                case RocketTag.SKILL_SERVICE_END_CHECK:
                    delayMsgHandler.handleSkillServiceEndCheck(JSON.parseObject(message.getBody(), QueueSkillEndCheckMsg.class));
                    break;
                case RocketTag.SKILL_SERVICE_RECEIVE_CHECK:
                    delayMsgHandler.handleSkillServiceReceiveCheck(JSON.parseObject(message.getBody(), QueueSkillReceiveCheckMsg.class));
                    break;
                default:
                    break;
            }
            return Action.CommitMessage;
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
    }
}
