package com.starbuds.server.service.finance.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.finance.provider.BagProvider;
import com.starbuds.server.service.finance.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/6/5
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Resource
    private BagProvider bagProvider;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到普通队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.LOTTERY_GIFT_REWARDS:
                    normalMsgHandler.handleLotteryGiftRewards(JSON.parseObject(message.getBody(), QueueLotteryGiftRewardsMsg.class));
                    break;
                case RocketTag.LOTTERY_GIFT_REWARDS_RTC:
                    normalMsgHandler.handleLotteryGiftRewardsRtc(JSON.parseObject(message.getBody(), QueueLotteryGiftRewardsRtcMsg.class));
                    break;
                case RocketTag.LOTTERY_GIFT_REWARDS_VOICE_CHAT:
                    normalMsgHandler.handleLotteryGiftRewardsVoiceChat(JSON.parseObject(message.getBody(), QueueLotteryGiftRewardsVoiceChatMsg.class));
                    break;
                case RocketTag.LOTTERY_GIFT_REWARDS_PRIVATE_CHAT:
                    normalMsgHandler.handleLotteryGiftRewardsPrivateChat(JSON.parseObject(message.getBody(), QueueLotteryGiftRewardsPrivateChatMsg.class));
                    break;
                case RocketTag.ANCHOR_WITHDRAW:
                    normalMsgHandler.handleAnchorWithdraw(JSON.parseObject(message.getBody(), QueueAnchorWithdrawMsg.class));
                    break;
                case RocketTag.USER_WITHDRAW:
                    normalMsgHandler.handleUserWithdraw(JSON.parseObject(message.getBody(), QueueUserWithdrawMsg.class));
                    break;
                case RocketTag.TRANSFER_TO_USER:
                    normalMsgHandler.handleTransferToUser(JSON.parseObject(message.getBody(), QueueTransferUserMsg.class));
                    break;
                case RocketTag.SKILL_ORDER_PAID:
                    normalMsgHandler.handleSkillOrderPaid(JSON.parseObject(message.getBody(), QueueSkillOrderPaidMsg.class));
                    break;
                case RocketTag.LOTTERY_GAME_LOTTERY_END:
                    normalMsgHandler.handleLotteryGameEnd(JSON.parseObject(message.getBody(), QueueLotteryGameEndMsg.class));
                    break;
                case RocketTag.ADD_ITEM_TO_BAG:
                    bagProvider.addItemToBagThenSendImMsgStatQueueMsg(JSON.parseObject(message.getBody(), QueueAddItemToBagMsg.class));
                    break;
                case RocketTag.REDUCE_ITEM_TO_BAG:
                    bagProvider.reduceItemToBag(JSON.parseObject(message.getBody(), QueueReduceItemToBagMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error(String.format("消息处理失败-%s-%s", message.getTag(), JSON.parseObject(message.getBody(), JSONObject.class)), e);
        }
        return Action.CommitMessage;
    }
}
