package com.starbuds.server.service.finance.util;

import com.aliyun.openservices.shade.com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.finance.CaiyuCashoutNotify;
import org.apache.commons.lang3.StringUtils;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.Map;
import java.util.TreeMap;

/**
 * 财鱼工具类
 */
public class CaiyuUtil {
    private static final String[] HEX_DIGITS =
            {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f"};
    private static final String JOIN_SYMBOL = "=";
    private static final String UNION_SYMBOL = "&";

    private CaiyuUtil() {
    }

    /**
     * 参数MD5加密
     *
     * @param params    参数MAP
     * @param secretKey 签名KEY
     * @return
     * @throws Exception
     */
    public static String paramsMD5Sign(Map<String, String> params, String secretKey) throws Exception {
        if (params == null || StringUtils.isBlank(secretKey)) {
            return "";
        }

        Map<String, String> sortedMap = new TreeMap<>(params);
        StringBuilder str = new StringBuilder();
        for (Map.Entry<String, String> entry : sortedMap.entrySet()) {
            if ("sign".equals(entry.getKey())) {
                continue;
            }
            if (StringUtils.isNotBlank(entry.getKey()) && StringUtils.isNotBlank(entry.getValue())) {
                str.append(entry.getKey()).append(JOIN_SYMBOL).append(entry.getValue()).append(UNION_SYMBOL);
            }
        }
        str.append(secretKey);

        return encode(str.toString());
    }

    /**
     * 获取MD5加密字符串
     *
     * @param origin
     * @return
     * @throws Exception
     */
    public static String encode(String origin) throws Exception {
        if (StringUtils.isBlank(origin)) {
            return null;
        }

        MessageDigest md = MessageDigest.getInstance("MD5");
        byte[] originBytes = origin.getBytes(StandardCharsets.UTF_8);
        byte[] digestBytes = md.digest(originBytes);
        return byteArrayToHexString(digestBytes);
    }

    /**
     * 二进制数组转为字符串
     *
     * @param b
     * @return
     */
    private static String byteArrayToHexString(byte[] b) {
        StringBuilder resultSb = new StringBuilder();
        for (byte aB : b) {
            resultSb.append(byteToHexString(aB));
        }

        return resultSb.toString();
    }

    /**
     * 二进制数转为字符串
     *
     * @param b
     * @return
     */
    private static String byteToHexString(byte b) {
        int n = b;
        if (n < 0) {
            n += 256;
        }
        int d1 = n / 16;
        int d2 = n % 16;
        return HEX_DIGITS[d1] + HEX_DIGITS[d2];
    }

    public static void main(String[] args) {
        CaiyuCashoutNotify c = new CaiyuCashoutNotify();
        c.setCashoutPartnerCode("123");
        c.setOrderId("order8900");
        c.setPartnerOrderId("pId123");
        c.setResult("result hahah");
        c.setSign("sign ssssdd");

        System.out.println(JSON.toJSONString(c));
    }
}
