package com.starbuds.server.service.finance.util;

import cn.hutool.core.util.RandomUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;

import javax.crypto.Cipher;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/24 16:39
 * @Description 云账户工具
 */
@Slf4j
public class CloudAccountUtil {

    /**
     * 获取加密后的data
     *
     * @param jsonStr
     * @param deskey
     * @return
     */
    public static String getData(String jsonStr, String deskey) {
        try {
            byte[] enc = CloudAccountUtil.TripleDesEncrypt(jsonStr.getBytes("utf-8"), deskey.getBytes("utf-8"));
            byte[] enc64 = Base64.encodeBase64(enc);
            return new String(enc64);
        } catch (Exception e) {
            log.info("转换出错{}", e);
        }
        return "";
    }

    /**
     * 解密data
     *
     * @param encryptStr
     * @param deskey
     * @return
     */
    public static String decodeData(String encryptStr, String deskey) {
        try {
            byte[] dec64 = Base64.decodeBase64(encryptStr.getBytes());
            byte[] dec = CloudAccountUtil.TripleDesDecrypt(dec64, deskey.getBytes());
            return new String(dec);
        } catch (Exception e) {
            log.info("转换出错{}", e);
        }
        return "";
    }

    /**
     * 获取签名
     *
     * @param data
     * @param mess
     * @param timestamp
     * @param appKey
     * @return
     */
    public static String getSign(String data, String mess, long timestamp, String appKey) {
        String str = String.format("data=%s&mess=%s&timestamp=%d&key=%s", data, mess, timestamp, appKey);
        return hmacSHA256(str, appKey);
    }

    private static byte[] TripleDesEncrypt(byte[] content, byte[] key) throws Exception {
        byte[] icv = new byte[8];
        System.arraycopy(key, 0, icv, 0, 8);
        return TripleDesEncrypt(content, key, icv);
    }

    private static byte[] TripleDesEncrypt(byte[] content, byte[] key, byte[] icv) throws
            Exception {
        final SecretKey secretKey = new SecretKeySpec(key, "DESede");
        final Cipher cipher = Cipher.getInstance("DESede/CBC/PKCS5Padding");
        final IvParameterSpec iv = new IvParameterSpec(icv);
        cipher.init(Cipher.ENCRYPT_MODE, secretKey, iv);
        return cipher.doFinal(content);
    }

    private static byte[] TripleDesDecrypt(byte[] content, byte[] key) throws Exception {
        byte[] icv = new byte[8];
        System.arraycopy(key, 0, icv, 0, 8);
        return TripleDesDecrypt(content, key, icv);
    }

    private static byte[] TripleDesDecrypt(byte[] content, byte[] key, byte[] icv) throws
            Exception {
        final SecretKey secretKey = new SecretKeySpec(key, "DESede");
        final Cipher cipher = Cipher.getInstance("DESede/CBC/PKCS5Padding");
        final IvParameterSpec iv = new IvParameterSpec(icv);
        cipher.init(Cipher.DECRYPT_MODE, secretKey, iv);
        return cipher.doFinal(content);
    }

    private static String hmacSHA256(String message, String key) {
        String outPut = null;
        try {
            Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
            SecretKeySpec secret_key = new SecretKeySpec(key.getBytes(), "HmacSHA256");
            sha256_HMAC.init(secret_key);
            byte[] bytes = sha256_HMAC.doFinal(message.getBytes());
            outPut = byteArrayToHexString(bytes);
        } catch (Exception e) {
            log.info("加密出错{}", e);
        }
        return outPut;
    }

    private static String byteArrayToHexString(byte[] b) {
        StringBuilder sb = new StringBuilder();
        String stmp;
        for (int n = 0; b != null && n < b.length; n++) {
            stmp = Integer.toHexString(b[n] & 0XFF);
            if (stmp.length() == 1) {
                sb.append('0');
            }
            sb.append(stmp);
        }
        return sb.toString().toLowerCase();
    }

    public static void main(String[] args) throws Exception {
        byte[] content = "{\"order_id\": \"201609010016562012987\",\"dealer_id\": \"2736123\",\"broker_id\": \"yiyun\",\"real_name\": \"张三\",\"card_no\": \"6228880199872220\",\"phone_no\": \"13488795491\",\"id_card\": \"123532612312312321\",\"pay\": \"100000.00\"}".getBytes("utf-8");
        byte[] key = "123456788765432112345678".getBytes("utf-8");
        byte[] enc = CloudAccountUtil.TripleDesEncrypt(content, key);
        byte[] enc64 = Base64.encodeBase64(enc);
        System.out.println("encrypt: " + new String(enc64));
        byte[] dec64 = Base64.decodeBase64(enc64);
        byte[] dec = CloudAccountUtil.TripleDesDecrypt(dec64, key);
        System.out.println("decrypt: " + new String(dec));
        String str = "{\"order_id\": \"201609010016562012987\",\"dealer_id\": \"2736123\",\"broker_id\": \"yiyun\",\"real_name\": \"张三\",\"card_no\": \"6228880199872220\",\"phone_no\": \"13488795491\",\"id_card\": \"123532612312312321\",\"pay\": \"100000.00\"}";
        String encryptStr = getData(str, "123456788765432112345678");
        System.out.println("加密后的data=" + encryptStr);
        System.out.println("加密前的data=" + decodeData(encryptStr, "123456788765432112345678"));
        long timestamp = System.currentTimeMillis() / 1000;
        String mess = RandomUtil.randomString(12);
        System.out.println("签名=" + getSign(encryptStr, mess, timestamp, "123456788765432112345678"));
    }
}

