package com.starbuds.server.service.finance.util;

import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.Mac;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.AlgorithmParameters;
import java.security.Key;
import java.security.Security;
import java.util.Base64;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/11/27 14:03
 * @Description 易纳税
 */
@Slf4j
public class EasyTaxUtil {

    /**
     * 创建签名
     *
     * @param data
     * @param timestamp
     * @param appKey
     * @param signType
     * @param requestId
     * @param dealerId
     * @return
     */
    public static String createSign(String data, String timestamp, String appKey, String signType, String requestId, String dealerId) {
        StringBuffer msg = new StringBuffer();
        msg.append("dealer_id=").append(dealerId).append("&request_id=").append(requestId)
                .append("&data=").append(data).append("&timestamp=").append(timestamp)
                .append("&sign_type=").append(signType).append("&app_key=").append(appKey);
        return sha256_HMAC(msg.toString(), appKey);

    }


    /**
     * sha256_HMAC 加密
     *
     * @param message
     * @param secret
     * @return
     */
    public static String sha256_HMAC(String message, String secret) {
        String hash = "";
        try {
            Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
            SecretKeySpec secret_key = new SecretKeySpec(secret.getBytes(), "HmacSHA256");
            sha256_HMAC.init(secret_key);
            byte[] bytes = sha256_HMAC.doFinal(message.getBytes());
            hash = byteArrayToHexString(bytes);
            log.info("sign = " + hash);

        } catch (Exception e) {
            log.info("Error HmacSHA256 ===========" + e.getMessage());
        }
        return hash;

    }

    /**
     * 将加密后的字节数组转换成字符串
     *
     * @param b
     * @return
     */
    private static String byteArrayToHexString(byte[] b) {
        StringBuilder hs = new StringBuilder();
        String stmp;
        int n;
        for (n = 0; b != null && n < b.length; n++) {
            stmp = Integer.toHexString(b[n] & 0XFF);
            if (stmp.length() == 1) {
                hs.append('0');
            }
            hs.append(stmp);
        }
        return hs.toString().toLowerCase();

    }

    /**
     * 加密数据
     *
     * @param order
     * @param securityKey
     * @param appKey
     * @return
     * @throws Exception
     */
    public static String encryptData(String order, String securityKey, String appKey) {
        Base64.Encoder encoder = Base64.getEncoder();
        String baseData = encoder.encodeToString(order.getBytes());
        String baseSessionKey = encoder.encodeToString(securityKey.getBytes());
        String baseIv = encoder.encodeToString(appKey.substring(0, 16).getBytes());
        // 导入支持AES/CBC/PKCS7Padding的Provider
        Security.addProvider(new BouncyCastleProvider());
        // 获取加密数据
        String encryptedData = null;
        try {
            encryptedData = encrypt(baseData, baseSessionKey, baseIv);
        } catch (Exception e) {
            log.info("出现异常", e);
        }
        return encryptedData;
    }

    /**
     * 加密
     *
     * @param data
     * @param sessionKey
     * @param iv
     * @return
     * @throws Exception
     */
    private static String encrypt(String data, String sessionKey, String iv) throws Exception {
        Base64.Decoder decoder = Base64.getDecoder();
        byte[] dataByte = decoder.decode(data);
        byte[] keyByte = decoder.decode(sessionKey);
        byte[] ivByte = decoder.decode(iv);

        String encryptedData;
        // 指定算法，模式，填充方式，创建一个Cipher
        Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
        // 生成key对象
        Key sKeySpec = new SecretKeySpec(keyByte, "AES");
        // 把向量初始化到算法参数
        AlgorithmParameters params = AlgorithmParameters.getInstance("AES");
        params.init(new IvParameterSpec(ivByte));
        // 指定用户，密钥，参数 初始化Cipher对象
        cipher.init(Cipher.ENCRYPT_MODE, sKeySpec, params);
        // 指定加密
        byte[] result = cipher.doFinal(dataByte);
        // 对结果进行Base64编码，否则会得到一串乱码，不便于后续操作
        Base64.Encoder encoder = Base64.getEncoder();
        encryptedData = encoder.encodeToString(result);
        return encryptedData;
    }

    /**
     * 解密
     *
     * @param encryptedData
     * @param securityKey
     * @param appKey
     * @return
     * @throws Exception
     */
    public static String decryptData(String encryptedData, String securityKey, String appKey) {
        Base64.Encoder encoder = Base64.getEncoder();
        String baseSessionKey = encoder.encodeToString(securityKey.getBytes());
        String baseIv = encoder.encodeToString(appKey.getBytes());
        Security.addProvider(new BouncyCastleProvider());
        String str = null;
        try {
            str = decrypt(encryptedData, baseSessionKey, baseIv);
        } catch (Exception e) {
            log.info("出现异常", e);
        }
        log.info("encrypt data is : {} ", str);
        return str;
    }

    /**
     * 解密
     *
     * @param encryptedData
     * @param sessionKey
     * @param iv
     * @return
     * @throws Exception
     */
    private static String decrypt(String encryptedData, String sessionKey, String iv) throws Exception {
        Base64.Decoder decoder = Base64.getDecoder();
        byte[] dataByte = decoder.decode(encryptedData);
        byte[] keyByte = decoder.decode(sessionKey);
        byte[] ivByte = decoder.decode(iv);
        String data;
        Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
        Key sKeySpec = new SecretKeySpec(keyByte, "AES");
        AlgorithmParameters params = AlgorithmParameters.getInstance("AES");
        params.init(new IvParameterSpec(ivByte));
        cipher.init(Cipher.DECRYPT_MODE, sKeySpec, params);
        byte[] result = cipher.doFinal(dataByte);
        data = new String(result);
        return data;
    }
}
