package com.starbuds.server.service.finance.util;

import okhttp3.*;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author aoe
 * @date 2021/1/7
 */
public class HttpUtil {

    private static OkHttpClient client;
    private static OkHttpClient clientMaxTimout;

    private static final MediaType MEDIA_TYPE_JSON = MediaType.get("application/json; charset=utf-8");

    static {
        client = new OkHttpClient();
        clientMaxTimout = new OkHttpClient.Builder()
                .callTimeout(10, TimeUnit.SECONDS)
                .readTimeout(60, TimeUnit.SECONDS)
                .build();
    }

    private HttpUtil() {
    }

    public static String post(String url, String json) throws IOException {
        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String postMaxTimout(String url, String json) throws IOException {
        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        try (Response response = clientMaxTimout.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String postMaxTimout(String url, Map<String, String> headerMap, String bodyJson) throws IOException {
        RequestBody body = RequestBody.create(bodyJson, MEDIA_TYPE_JSON);
        Request.Builder builder = new Request.Builder();
        if (headerMap != null) {
            for (Map.Entry<String, String> header : headerMap.entrySet()) {
                builder.addHeader(header.getKey(), header.getValue());
            }
        }
        Request request = builder
                .url(url)
                .post(body)
                .build();
        try (Response response = clientMaxTimout.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String postMaxTimout(String url, Map<String, String> params) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (params != null) {
            FormBody.Builder body = new FormBody.Builder();
            for (Map.Entry<String, String> param : params.entrySet()) {
                body.add(param.getKey(), param.getValue());
            }
            builder.post(body.build());
        }

        Request request = builder.url(url).build();
        try (Response response = clientMaxTimout.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String get(String url) throws IOException {
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String get(String url, Map<String, String> headerMap) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (headerMap != null) {
            for (Map.Entry<String, String> header : headerMap.entrySet()) {
                builder.addHeader(header.getKey(), header.getValue());
            }
        }
        Request request = builder
                .url(url)
                .get()
                .build();
        try (Response response = client.newCall(request).execute()) {
            return response.body().string();
        }
    }
}
