package com.starbuds.server.service.common.validation;

import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.common.validation.Validator;
import org.hibernate.validator.HibernateValidator;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Optional;

/**
 * @author aoe
 * @date 2021/12/10
 */
public class ValidatorTest {

    private Validator validator;

    @BeforeEach
    void init() {
        validator = new Validator(HibernateValidator.class);
    }

    @Test
    void hasFail() {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        Assertions.assertTrue(validator.hasFail(msg));

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        Assertions.assertTrue(validator.hasFail(msg));

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        msg.setSource(AddItemToBagSourceEnum.RED_BAG);
        msg.setItemId(1L);
        msg.setItemType(BagItemTypeEnum.GIFT);
        msg.setQuantity(1);
        msg.setExpireTime(-1L);
        msg.setUserId(1L);
        Assertions.assertTrue(validator.hasFail(msg));

        msg.setRemark("");
        Assertions.assertTrue(validator.hasFail(msg));
    }

    @Test
    void hasNoFail() {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        msg.setSource(AddItemToBagSourceEnum.RED_BAG);
        msg.setItemId(1L);
        msg.setItemName("名称");
        msg.setItemType(BagItemTypeEnum.GIFT);
        msg.setQuantity(1);
        msg.setExpireTime(-1L);
        msg.setUserId(1L);
        msg.setRemark("haha");
        Assertions.assertFalse(validator.hasFail(msg));
    }

    @Test
    void getFailMsgOfOne() {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        Optional errMsg = validator.getFailMsgOfOne(msg);
        Assertions.assertTrue(errMsg.isPresent());

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        errMsg = validator.getFailMsgOfOne(msg);
        Assertions.assertTrue(errMsg.isPresent());

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        msg.setSource(AddItemToBagSourceEnum.RED_BAG);
        msg.setItemId(1L);
        msg.setItemType(BagItemTypeEnum.GIFT);
        msg.setQuantity(1);
        msg.setExpireTime(-1L);
        msg.setUserId(1L);
        errMsg = validator.getFailMsgOfOne(msg);
        Assertions.assertTrue(errMsg.isPresent());

        msg.setRemark("");
        errMsg = validator.getFailMsgOfOne(msg);
        Assertions.assertTrue(errMsg.isPresent());
    }

    @Test
    void verified() {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        Assertions.assertFalse(validator.verified(msg));

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        Assertions.assertFalse(validator.verified(msg));

        msg = new QueueAddItemToBagMsg();
        msg.setOrderId(1L);
        msg.setSource(AddItemToBagSourceEnum.RED_BAG);
        msg.setItemId(1L);
        msg.setItemName("name");
        msg.setItemType(BagItemTypeEnum.GIFT);
        msg.setQuantity(1);
        msg.setExpireTime(-1L);
        msg.setUserId(1L);
        Assertions.assertFalse(validator.verified(msg));

        msg.setRemark("");
        Assertions.assertFalse(validator.verified(msg));

        msg.setRemark("d");
        Assertions.assertTrue(validator.verified(msg));
    }

}
