package com.starbuds.server.service.finance.impl;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.RecordList;
import com.starbuds.server.common.pojo.finance.UserWalletHisVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.IssueGoldCoinHisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.provider.WalletProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import test.BaseTest;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.List;

import static com.starbuds.server.common.constant.finance.IssueGoldCoinOrderTypeEnum.DAILY_SIGN_IN;
import static com.starbuds.server.common.constant.finance.WalletTypeEnum.GOLD_COIN;

@Slf4j
public class IssueGoldCoinHisServiceImplTest extends BaseTest {

    @Reference(version = "1.0.0")
    private IssueGoldCoinHisService issueGoldCoinHisService;

    @Resource
    private WalletProvider walletProvider;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    // 141075591200768L 大卡车
    // 272343851073536L 80家族长
    // 46898517901319 早餐3 逍遥
    // 265700434247680 大王叫我来巡山 九溪
    private long USER_ID = Long.parseLong("265700434247680");

    // 发放金币
    @Test
    public void testPayGoldCoin() {
        long balance = goldCoinBalance(USER_ID);
        long orderId = identityService.getId();
        int amount = 1000000;
        boolean pay = issueGoldCoinHisService.payGoldCoin(USER_ID, amount, orderId, DAILY_SIGN_IN, "连续签到1天");
        Assertions.assertTrue(pay);

        long balance1 = goldCoinBalance(USER_ID);
        Assertions.assertEquals(balance1, balance + amount);
        log.info(LogUtil.devLog("系统发放金币 balance={} balance1={} amount={}"), balance, balance1, amount);

        LocalDate now = LocalDate.now();
        Integer yearMonth = Integer.parseInt(now.getYear() + "" + now.getMonthValue());
        RecordList<UserWalletHisVO, Long> record = walletService.getUserWalletHisListByTypeAndMonth(null, 20,
                USER_ID, GOLD_COIN, yearMonth, BoolIntVal.True);
        List<UserWalletHisVO> list = record.getList();
        Assertions.assertTrue(list.size() > 0);
        log.info("系统发放金币明细 {}", LogUtil.obj2JsonObj(list));

        // 同一个id发放2次
//        Assert.expectThrows(Exception.class, () -> issueGoldCoinHisService.payGoldCoin(USER_ID, amount, orderId,
//                DAILY_SIGN_IN, "连续签到3天"));
        long balance2 = goldCoinBalance(USER_ID);
        Assertions.assertEquals(balance2, balance1);
    }

    private Long goldCoinBalance(Long userId) {
        return walletProvider.getBalance(userId, GOLD_COIN);
    }
}