package com.starbuds.server.service.finance.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.service.finance.impl.api.YunxiaokeApiService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.math.BigDecimal;
import java.nio.charset.Charset;

@Slf4j
public class YunxiaokeApiServiceTest {

    private YunxiaokeApiService yunxiaokeApiService;

    public YunxiaokeApiServiceTest() {
        yunxiaokeApiService = new YunxiaokeApiService();
//        yunxiaokeApiService.testProdInit();
        yunxiaokeApiService.testInit();
    }

    @Test
    public void getBalance() {
        BigDecimal balance = yunxiaokeApiService.getBalance();
        log.info("可用余额 {}", balance);
    }

    @Test
    public void accounts() {
        JSONObject res = yunxiaokeApiService.accounts();
        log.info("账户信息 {}", JSON.toJSONString(res, true));
    }

    @Test
    public void servers() {
        JSONObject res = yunxiaokeApiService.servers();
        log.info("3.5 供应商信息查询 {}", JSON.toJSONString(res, true));
    }

    // 转账
    @Test
    public void doWithdraw() {
        yunxiaokeApiService.doWithdraw(null);
    }

    // 订单详情
    @Test
    public void orderInfo() {
        JSONObject res = yunxiaokeApiService.orderInfo("532662ee-c285-11eb-a9ea-0242c0a8c00e");
    }

    @Test
    public void signature() {
        String bodyStr = "{\"order_uuid\": \"8ce22270-c2cb-11eb-8f5e-0242c0a8c00e\", \"order_no\": \"20210601-192146-1146\", \"customer_user_uuid\": \"119b0dec-c1d7-11eb-90cd-0242c0a8c00e\", \"customer_user_name\": \"\\u5feb\\u9c7c\\u79d1\\u6280\\u5bf9\\u63a5\", \"customer_account_uuid\": \"34d63930-c1d7-11eb-9f22-0242c0a8c00e\", \"server_user_uuid\": \"f46e79d4-9cce-11eb-8900-0242c0a8c00c\", \"customer_server_name\": \"\\u4f01\\u4e1a\\u7f51\\u94f6\\u65b020161361\", \"salary_amount\": \"1.00\", \"customer_total_amount\": \"1.07\", \"trade_mark\": null, \"to_launch_date\": \"2021-06-01\", \"fee_mode\": \"after_fee\", \"order_status\": 50, \"tasks_count\": 1, \"customer_fee\": \"0.07\", \"customer_extra_tax\": \"0.00\", \"call_back\": {\"url\": \"/api-notify/v1/yunxiaoke/ordersNotify\", \"host\": \"api.panygo.com\", \"port\": \"80\", \"is_https\": 0}, \"task_list\": [{\"task_uuid\": \"8ce4568a-c2cb-11eb-8f5e-0242c0a8c00e\", \"task_no\": \"1\", \"social_no\": \"610302198512132517\", \"name\": \"\\u9ec4\\u9e4f\", \"mobile_no\": \"18158183582\", \"card_no\": \"6222021001015057760\", \"salary_amount\": \"1.00\", \"order_status\": 50, \"successed_time\": \"2021-06-01 19:26:42\", \"identity_type\": \"\\u8eab\\u4efd\\u8bc1\"}]}";
        String signature = Base64.encodeBase64String(DigestUtils.getSha512Digest().digest(bodyStr.getBytes(Charset.forName("UTF-8"))));
        Assertions.assertEquals("gDs2ejeedw4vKyN562btbOMaHWTIgOnv2P0l/ZG5VYstNohLwOSU8slhtmvqKtNkHOyT2p72iqyzJ7izLJum0w==", signature);
    }
}