package com.starbuds.server.service.finance.impl.lottery;

import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.service.activity.LotteryGameUserQualificationHisService;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import test.BaseTest;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @author laoliu
 * @date 2021/11/25 16:46
 * @description
 */
public class LotteryGameQualificationImplTest extends BaseTest {

    private static Map<LotteryGameQualificationImpl.StrategiesEnum, LotteryGameStrategy> strategies = new HashMap<>();

    static {
        strategies.put(LotteryGameQualificationImpl.StrategiesEnum.FIRST_TIME, new LotteryGameStrategyFirstTimeImpl());
        strategies.put(LotteryGameQualificationImpl.StrategiesEnum.RANDOM_BY_RATE, new LotteryGameStrategyRandomByRateImpl());
    }

    @Resource
    private LotteryGameUserQualificationHisService lotteryGameUserQualificationHisService;

    @Test
    public void testFirstTimeStrategies() {

        LotteryGameSummary summary = new LotteryGameSummary();

        int flag = 1;
        for (int i = 0; i < 10; i++) {
            if (!strategies.get(LotteryGameQualificationImpl.StrategiesEnum.FIRST_TIME).getResult(summary)) {
                flag = 0;
            }
        }
        Assert.assertEquals(flag, 1);

    }

    @Test
    public void testRandomByRateStrategies() {

        LotteryGameSummary summary = new LotteryGameSummary();
        Double rate = 0.3;
        int time = 100;
        summary.setClickRate(BigDecimal.valueOf(rate));

        List<Boolean> list = new ArrayList<>();
        for (int i = 0; i < time; i++) {

            list.add(strategies.get(LotteryGameQualificationImpl.StrategiesEnum.RANDOM_BY_RATE).getResult(summary));
        }

        List<Boolean> trueList = list.stream().filter(val -> val == Boolean.TRUE).collect(Collectors.toList());

        Assert.assertTrue((trueList.size() - time * rate) <= (time * 0.15));

    }

    @Test
    @Rollback
    @Transactional
    public void testLotteryGameUserQualificationHisService() {

        Long userId = 666666666L;
        Long snapId = 666666666L;
        LotteryGameUserQualificationHisDO hisDO = new LotteryGameUserQualificationHisDO();
        hisDO.setUserId(userId);
        hisDO.setSnapId(snapId);
        System.out.println("开始插入");
        lotteryGameUserQualificationHisService.create(hisDO);
        System.out.println("插入成功");

        Assert.assertTrue(lotteryGameUserQualificationHisService.exist(snapId, userId));
    }
}