package com.starbuds.server.service.finance.impl.lottery;

import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameUserJoinHisDO;
import com.starbuds.server.common.pojo.finance.params.SaveLotteryGameQualificationParam;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.finance.SaveLotteryGameQualificationService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.dao.LotteryGameUserJoinHisDAO;
import com.starbuds.server.service.finance.impl.lottery.tools.LotteryGameCache;
import com.starbuds.server.service.finance.impl.lottery.tools.LotteryGameQualificationParamTool;
import com.starbuds.server.service.finance.provider.BagProvider;
import com.starbuds.server.service.finance.provider.lottery.LotteryGameUserJoinHisProvider;
import com.starbuds.server.service.finance.provider.lottery.cost.LotteryGameUserQualificationHisProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.*;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import test.BaseTest;

import javax.annotation.Resource;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.LongStream;

@Slf4j
class SaveLotteryGameQualificationServiceImplTest extends BaseTest {

    @Reference(version = "1.0.0")
    private SaveLotteryGameQualificationService saveLotteryGameQualificationService;

    @Reference(version = "1.0.0")
    private CacheService cacheService;

    @Resource
    private LotteryGameUserJoinHisDAO lotteryGameUserJoinHisDAO;

    @Resource
    private LotteryGameUserJoinHisProvider lotteryGameUserJoinHisProvider;

    @Resource
    private BagProvider bagProvider;

    @Resource
    private LotteryGameUserQualificationHisProvider lotteryGameUserQualificationHisProvider;

    private LotteryGameQualificationParamTool qualificationParamTool;

    private LotteryGameCache lotteryGameCache;

    private long snapId;

    @BeforeEach
    void init() {
        log.info(LogUtil.devLog("init()init()init()init()init()init()init()init()init()init()"));
        qualificationParamTool = new LotteryGameQualificationParamTool();
        lotteryGameCache = LotteryGameCache.builder().cacheService(cacheService).build();
        snapId = System.currentTimeMillis();
    }

    @AfterEach
    void clean() {
        log.info(LogUtil.devLog("clean()clean()clean()clean()clean()clean()"));
        lotteryGameCache.clearRewardCount(snapId);
    }

    @Test
    @Rollback
    @Transactional
    void saveQualificationRedBag() {
        AtomicInteger quantity = new AtomicInteger(10);
        lotteryGameCache.initRewardCount(snapId, quantity.get());
        Long roomId = 90L;
        LongStream.rangeClosed(1, 20)
                .forEach(userId -> {
                    SaveLotteryGameQualificationParam param = qualificationParamTool.getRedBagParam(snapId, userId, roomId);
                    saveLotteryGameQualificationService.costItemAndSaveQualification(param)
                            .stream()
                            .forEach(his -> {
                                Assertions.assertNotNull(his);
                                Assertions.assertEquals(quantity.decrementAndGet(), lotteryGameCache.getRewardCount(snapId));
                                Assertions.assertNotNull(getUserJoinHis(snapId, userId, roomId));
                            });
                });
    }

    @Test
    @Rollback
    @Transactional
    void saveQualificationDivideCake1() {
        saveQualificationDivideCake(1);
    }

    @Test
    @Rollback
    @Transactional
    void saveQualificationDivideCake10() {
        saveQualificationDivideCake(10);
    }

    @Test
    @Rollback
    @Transactional
    void saveQualificationDivideCake20() {
        saveQualificationDivideCake(20);
    }

    void saveQualificationDivideCake(int costQuantity) {
        AtomicInteger quantity = new AtomicInteger(10);
        lotteryGameCache.initRewardCount(snapId, quantity.get());
        Long roomId = 90L;
        // 大卡车72😲
        long userId = 141075591200768L;
        // 道具 星空仙女碎片
        long costItemId = Long.parseLong("470831880732672");

        int qCount = lotteryGameUserQualificationHisProvider.getCount(snapId, userId);
        Assertions.assertEquals(0, qCount);

        int joinHisCount = lotteryGameUserJoinHisProvider.getCount(userId, snapId);
        Assertions.assertEquals(0, joinHisCount);
        int quantityByBag = bagProvider.getQuantityByItemId(userId, BagItemTypeEnum.PROPS, costItemId);

        SaveLotteryGameQualificationParam param = qualificationParamTool.getDivideCakeParam(snapId, userId, roomId,
                costItemId, costQuantity);
        saveLotteryGameQualificationService.costItemAndSaveQualification(param).stream()
                .forEach(his -> {
                    Assertions.assertNotNull(his);
                });
        int expectedQuantity = quantity.get() - costQuantity;
        Assertions.assertEquals(expectedQuantity, lotteryGameCache.getRewardCount(snapId));
        Assertions.assertNotNull(getUserJoinHis(snapId, userId, roomId));

        qCount = lotteryGameUserQualificationHisProvider.getCount(snapId, userId);
        Assertions.assertEquals(costQuantity, qCount);

        joinHisCount = lotteryGameUserJoinHisProvider.getCount(userId, snapId);
        Assertions.assertEquals(costQuantity, joinHisCount);

        int quantityByBagAfterJoin = bagProvider.getQuantityByItemId(userId, BagItemTypeEnum.PROPS, costItemId);
        Assertions.assertEquals(quantityByBag - costQuantity, quantityByBagAfterJoin,
                () -> String.format("%s - %s = %s", quantityByBag, costQuantity, quantityByBagAfterJoin));

        Assertions.assertEquals(expectedQuantity, lotteryGameCache.getRewardCount(snapId));
    }

    private LotteryGameUserJoinHisDO getUserJoinHis(Long snapId, Long userId, Long roomId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("snapId", snapId);
        qw.eq("userId", userId);
        qw.eq("roomId", roomId);
        return lotteryGameUserJoinHisDAO.selectOne(qw);
    }
}