package com.starbuds.server.service.finance.provider;

import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.service.data.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import test.BaseTest;

import javax.annotation.Resource;
import java.util.stream.IntStream;

import static java.lang.Thread.sleep;

@Slf4j
public class CacheProviderTest extends BaseTest {

    @Resource
    private CacheProvider cacheProvider;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Test
    void getRtcRoomRatio() {
        Long nullRoomId = 1L;
        RtcRoomRatio rtcRoomRatio = cacheProvider.getRtcRoomRatio(nullRoomId);
        Assertions.assertNull(rtcRoomRatio);
        Assertions.assertNull(redisService.get(RedisFinanceKeys.getRtcRoomRatio(nullRoomId)));

        Long roomId = 483988682571776L;
        Assertions.assertNull(redisService.get(RedisFinanceKeys.getRtcRoomRatio(roomId)));
        IntStream.range(0, 2).forEach(i -> {
            RtcRoomRatio roomRatio = cacheProvider.getRtcRoomRatio(roomId);
            Assertions.assertNotNull(roomRatio);
            Assertions.assertNotNull(redisService.get(RedisFinanceKeys.getRtcRoomRatio(roomId)));
        });

        // 等待缓存过期
        for (int i = 0; i < 11; i++) {
            try {
                sleep(1000);
            } catch (InterruptedException e) {
                log.error("等待缓存过期被打断，当前 " + i + " 秒", e);
            }
        }

        Assertions.assertNull(redisService.get(RedisFinanceKeys.getRtcRoomRatio(roomId)));
        IntStream.range(0, 2).forEach(i -> {
            RtcRoomRatio roomRatio = cacheProvider.getRtcRoomRatio(roomId);
            Assertions.assertNotNull(roomRatio);
            Assertions.assertNotNull(redisService.get(RedisFinanceKeys.getRtcRoomRatio(roomId)));
        });
    }

    /*@Test
    public void testFamilyRatio() {
        Long familyId = Long.parseLong("1");

        // Redis、数据库都不存在
        BigDecimal familyRatio = cacheProvider.familyRatio(familyId);
        Assertions.assertEquals(familyRatio, BigDecimal.valueOf(giftFamilyRatio));

        // Redis不存在；数据库存在
        familyId = Long.parseLong("435437793443840");
        String key = RedisFinanceKeys.getFamilyRatio(familyId);
        redisService.set(key, "a");
        familyRatio = cacheProvider.familyRatio(familyId);
        // 数据库中的家族收益比率
        BigDecimal ratioDb = BigDecimal.valueOf(familyDAO.selectFamilyRatio(familyId));
        Assertions.assertEquals(familyRatio, ratioDb);

        // Redis、数据都库存在
        familyRatio = cacheProvider.familyRatio(familyId);
        Assertions.assertEquals(familyRatio, ratioDb);
    }*/
}