package com.starbuds.server.service.finance.provider.lottery;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.finance.dao.LotteryGameUserQualificationHisDAO;
import com.starbuds.server.service.finance.dao.LotteryGameWinHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import test.BaseTest;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

@Slf4j
class LotteryGameProviderTest extends BaseTest {

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private CacheService cacheService;

    @Resource
    private LotteryGameUserQualificationHisDAO lotteryGameUserQualificationHisDAO;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Resource
    private LotteryGameWinHisDAO lotteryGameWinHisDAO;

    @Test
    @Transactional
    @Rollback
    public void lotteryThenAward() {
        long snapId = System.currentTimeMillis();

        String key = RedisActivityKeys.getLotteryGameWinItems(snapId);
        // 清理缓存
        cleanWinItemsFromRedis(key);
        // 向缓存中添加奖项
        saveWinItems2Redis(key);

        List<Long> userIds = getUserIds(60);
        List<LotteryGameUserQualificationHisDO> hisList = getQualificationHisList(snapId, userIds);
        // 创建抽奖资格记录
        int createCount = createQualificationHisList(hisList);
        Assertions.assertEquals(userIds.size(), createCount);

        log.info(LogUtil.devLog("lotteryGameProvider={}"), lotteryGameProvider);
        Boolean lotteryThenAward = lotteryGameProvider.lotteryThenAward(snapId, LotteryGamePlayTypeEnum.RED_BAG);
        Assertions.assertTrue(lotteryThenAward);

        List<LotteryGameWinHisDO> winList = lotteryGameWinHisDAO.selectListBySnapId(snapId);
        Assertions.assertEquals(30, winList.size());

        // 清理缓存
        cleanWinItemsFromRedis(key);
    }

    private int createQualificationHisList(List<LotteryGameUserQualificationHisDO> hisList) {
        return lotteryGameUserQualificationHisDAO.insertBatch(hisList);
    }

    private List<Long> getUserIds(int size) {
        return userService.getUserIds(1, size);
    }

    private List<LotteryGameUserQualificationHisDO> getQualificationHisList(long snapId, List<Long> userIds) {
        AtomicLong startHisId = new AtomicLong(System.currentTimeMillis());
        return userIds.stream()
                .map(userId -> {
                    LotteryGameUserQualificationHisDO his = new LotteryGameUserQualificationHisDO();
                    his.setHisId(startHisId.getAndAdd(1));
                    his.setCreateTime(his.getHisId());
                    his.setSnapId(snapId);
                    his.setUserId(userId);
                    his.setIsEffective(BoolIntVal.True);
                    return his;
                }).collect(Collectors.toList());
    }

    /**
     * 将中奖礼物放入Redis
     *
     * @return
     */
    private void saveWinItems2Redis(String key) {
        String jsonArrStr = "[{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2}]";

        List<String> jsonItems = JSON.parseArray(jsonArrStr, String.class);
        cacheService.set(key, JSON.toJSONString(jsonItems));
    }

    private boolean cleanWinItemsFromRedis(String key) {
        // 清理数据
        return cacheService.delKey(key);
    }
}